@@ -87,6 +87,22 @@ main(int argc, char **argv)
 				return 1;
 			}
 		}
+
+		// Inject a custom config dir.
+		if (strcmp(argv[i], "-cfgdir") == 0)
+		{
+			// We need an argument.
+			if (i != (argc - 1))
+			{
+				Q_strlcpy(cfgdir, argv[i + 1], sizeof(cfgdir));
+			}
+			else
+			{
+				printf("-cfgdir needs an argument\n");
+				return 1;
+			}
+
+		}
 	}
 
 	// Need to redirect stdout before anything happens.
