@@ -499,6 +499,13 @@ SCR_DrawLoading(void)
 void
 SCR_RunConsole(void)
 {
+	/* src_conspeed must be a positiv integer,
+	   otherwise things go wrong. Clamp it. */
+	if (scr_conspeed->value < 0.1f)
+	{
+		Cvar_Set("scr_conspeed", "0.1");
+	}
+
 	/* decide on the height of the console */
 	if (cls.key_dest == key_console)
 	{
@@ -1104,7 +1111,7 @@ SCR_ExecuteLayoutString(char *s)
 				Com_Error(ERR_DROP, "Pic >= MAX_IMAGES");
 			}
 
-			if (cl.configstrings[CS_IMAGES + value])
+			if (cl.configstrings[CS_IMAGES + value][0] != '\0')
 			{
 				SCR_AddDirtyPoint(x, y);
 				SCR_AddDirtyPoint(x + 23*scale, y + 23*scale);
@@ -1415,6 +1422,10 @@ SCR_Framecounter(void) {
 	static int frametimes[60] = {0};
 	static long long oldtime;
 
+	/* skip statistics without show fps */
+	if (cl_showfps->value < 1)
+		return;
+
 	newtime = Sys_Microseconds();
 	frametimes[frame] = (int)(newtime - oldtime);
 
@@ -1519,7 +1530,7 @@ SCR_UpdateScreen(void)
 		numframes = 2;
 		separation[0] = -gl1_stereo_separation->value / 2;
 		separation[1] = +gl1_stereo_separation->value / 2;
-	}		
+	}
 	else
 	{
 		separation[0] = 0;
@@ -1536,6 +1547,7 @@ SCR_UpdateScreen(void)
 			/* loading plaque over black screen */
 			int w, h;
 
+			R_EndWorldRenderpass();
 			if(i == 0){
 				R_SetPalette(NULL);
 			}
@@ -1560,6 +1572,7 @@ SCR_UpdateScreen(void)
 					cl.cinematicpalette_active = false;
 				}
 
+				R_EndWorldRenderpass();
 				M_Draw();
 			}
 			else if (cls.key_dest == key_console)
@@ -1570,10 +1583,12 @@ SCR_UpdateScreen(void)
 					cl.cinematicpalette_active = false;
 				}
 
+				R_EndWorldRenderpass();
 				SCR_DrawConsole();
 			}
 			else
 			{
+				R_EndWorldRenderpass();
 				SCR_DrawCinematic();
 			}
 		}
