@@ -310,12 +310,16 @@ extern qboolean	chat_team;
 void Char_Event(int key);
 void Key_Event(int key, qboolean down, qboolean special);
 void Key_Init(void);
+void Key_Shutdown(void);
 void Key_WriteBindings(FILE *f);
 void Key_ReadConsoleHistory();
 void Key_WriteConsoleHistory();
 void Key_SetBinding(int keynum, char *binding);
 void Key_MarkAllUp(void);
-void Haptic_Feedback(char *name, int effect_volume, int effect_x, int effect_y, int effect_z);
+void Haptic_Feedback(char *name, int effect_volume, int effect_duration,
+				   int effect_begin, int effect_end,
+				   int effect_attack, int effect_fade,
+				   int effect_x, int effect_y, int effect_z);
 int Key_GetMenuKey(int key);
 
 #endif
