@@ -28,6 +28,7 @@
 
 image_t gltextures[MAX_GLTEXTURES];
 int numgltextures;
+static int image_max = 0;
 int base_textureid; /* gltextures[i] = base_textureid+i */
 extern qboolean scrap_dirty;
 extern byte scrap_texels[MAX_SCRAPS][BLOCK_WIDTH * BLOCK_HEIGHT];
@@ -204,26 +205,25 @@ R_TextureMode(char *string)
 	{
 		if (gl_anisotropic->value > gl_config.max_anisotropy)
 		{
-			ri.Cvar_SetValue("gl_anisotropic", gl_config.max_anisotropy);
-		}
-		else if (gl_anisotropic->value < 1.0)
-		{
-			ri.Cvar_SetValue("gl_anisotropic", 1.0);
+			ri.Cvar_SetValue("r_anisotropic", gl_config.max_anisotropy);
 		}
 	}
 	else
 	{
-		ri.Cvar_SetValue("gl_anisotropic", 0.0);
+		ri.Cvar_SetValue("r_anisotropic", 0.0);
 	}
 
 	const char* nolerplist = gl_nolerp_list->string;
+	qboolean unfiltered2D = r_2D_unfiltered->value != 0;
 
 	/* change all the existing mipmap texture objects */
 	for (i = 0, glt = gltextures; i < numgltextures; i++, glt++)
 	{
-		if (nolerplist != NULL && strstr(nolerplist, glt->name) != NULL)
+		qboolean nolerp = false;
+		/* r_2D_unfiltered and r_nolerp_list allow rendering stuff unfiltered even if gl_filter_* is filtered */
+		if ( (unfiltered2D && glt->type == it_pic) || (nolerplist != NULL && strstr(nolerplist, glt->name) != NULL) )
 		{
-			continue; /* those (by default: font and crosshairs) always only use GL_NEAREST */
+			nolerp = true;
 		}
 
 		R_Bind(glt->texnum);
@@ -236,15 +236,24 @@ R_TextureMode(char *string)
 			if (gl_config.anisotropic && gl_anisotropic->value)
 			{
 				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT,
-						gl_anisotropic->value);
+						max(gl_anisotropic->value, 1.f));
 			}
 		}
 		else /* texture has no mipmaps */
 		{
-			// we can't use gl_filter_min which might be GL_*_MIPMAP_*
-			// also, there's no anisotropic filtering for textures w/o mipmaps
-			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_max);
-			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_filter_max);
+			if (nolerp)
+			{
+				// this texture shouldn't be filtered at all (no gl_nolerp_list or r_2D_unfiltered case)
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+			}
+			else
+			{
+				// we can't use gl_filter_min which might be GL_*_MIPMAP_*
+				// also, there's no anisotropic filtering for textures w/o mipmaps
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_max);
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_filter_max);
+			}
 		}
 	}
 }
@@ -296,9 +305,9 @@ R_TextureSolidMode(char *string)
 void
 R_ImageList_f(void)
 {
-	int i;
+	int i, used, texels;
 	image_t *image;
-	int texels;
+	qboolean	freeup;
 	const char *palstrings[2] = {
 		"RGB",
 		"PAL"
@@ -306,14 +315,23 @@ R_ImageList_f(void)
 
 	R_Printf(PRINT_ALL, "------------------\n");
 	texels = 0;
+	used = 0;
 
 	for (i = 0, image = gltextures; i < numgltextures; i++, image++)
 	{
+		char *in_use = "";
+
 		if (image->texnum <= 0)
 		{
 			continue;
 		}
 
+		if (image->registration_sequence == registration_sequence)
+		{
+			in_use = "*";
+			used++;
+		}
+
 		texels += image->upload_width * image->upload_height;
 
 		switch (image->type)
@@ -335,14 +353,16 @@ R_ImageList_f(void)
 				break;
 		}
 
-		R_Printf(PRINT_ALL, " %3i %3i %s: %s\n",
+		R_Printf(PRINT_ALL, " %3i %3i %s: %s %s\n",
 				image->upload_width, image->upload_height,
-				palstrings[image->paletted], image->name);
+				palstrings[image->paletted], image->name, in_use);
 	}
 
 	R_Printf(PRINT_ALL,
 			"Total texel count (not counting mipmaps): %i\n",
 			texels);
+	freeup = R_ImageHasFreeSpace();
+	R_Printf(PRINT_ALL, "Used %d of %d images%s.\n", used, image_max, freeup ? ", has free space" : "");
 }
 
 /*
@@ -753,7 +773,7 @@ R_Upload32(unsigned *data, int width, int height, qboolean mipmap)
 	if (mipmap && gl_config.anisotropic && gl_anisotropic->value)
 	{
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT,
-				gl_anisotropic->value);
+				max(gl_anisotropic->value, 1.f));
 	}
 	return res;
 }
@@ -765,16 +785,7 @@ R_Upload32(unsigned *data, int width, int height, qboolean mipmap)
 qboolean
 R_Upload8(byte *data, int width, int height, qboolean mipmap, qboolean is_sky)
 {
-	unsigned trans[512 * 256];
-	int i, s;
-	int p;
-
-	s = width * height;
-
-	if (s > sizeof(trans) / 4)
-	{
-		ri.Sys_Error(ERR_DROP, "R_Upload8: too large");
-	}
+	int s = width * height;
 
 	if (gl_config.palettedtexture && is_sky)
 	{
@@ -789,9 +800,11 @@ R_Upload8(byte *data, int width, int height, qboolean mipmap, qboolean is_sky)
 	}
 	else
 	{
-		for (i = 0; i < s; i++)
+		unsigned *trans = malloc(s * sizeof(unsigned));
+
+		for (int i = 0; i < s; i++)
 		{
-			p = data[i];
+			int p = data[i];
 			trans[i] = d_8to24table[p];
 
 			/* transparent, so scan around for
@@ -826,7 +839,9 @@ R_Upload8(byte *data, int width, int height, qboolean mipmap, qboolean is_sky)
 			}
 		}
 
-		return R_Upload32(trans, width, height, mipmap);
+		qboolean ret = R_Upload32(trans, width, height, mipmap);
+		free(trans);
+		return ret;
 	}
 }
 
@@ -841,8 +856,11 @@ R_LoadPic(char *name, byte *pic, int width, int realwidth,
 	int i;
 
 	qboolean nolerp = false;
-
-	if(gl_nolerp_list != NULL && gl_nolerp_list->string != NULL)
+	if (r_2D_unfiltered->value && type == it_pic)
+	{
+		nolerp = true;
+	}
+	else if(gl_nolerp_list != NULL && gl_nolerp_list->string != NULL)
 	{
 		nolerp = strstr(gl_nolerp_list->string, name) != NULL;
 	}
@@ -930,9 +948,22 @@ R_LoadPic(char *name, byte *pic, int width, int realwidth,
 
 		if (bits == 8)
 		{
-			image->has_alpha = R_Upload8(pic, width, height,
-						(image->type != it_pic && image->type != it_sky),
-						image->type == it_sky);
+			// resize 8bit images only when we forced such logic
+			if (r_scale8bittextures->value)
+			{
+				byte *image_converted = malloc(width * height * 4);
+				scale2x(pic, image_converted, width, height);
+				image->has_alpha = R_Upload8(image_converted, width * 2, height * 2,
+							(image->type != it_pic && image->type != it_sky),
+							image->type == it_sky);
+				free(image_converted);
+			}
+			else
+			{
+				image->has_alpha = R_Upload8(pic, width, height,
+							(image->type != it_pic && image->type != it_sky),
+							image->type == it_sky);
+			}
 		}
 		else
 		{
@@ -1024,6 +1055,75 @@ LoadWal(char *origname, imagetype_t type)
 	return image;
 }
 
+static image_t *
+LoadM8(char *origname, imagetype_t type)
+{
+	m8tex_t *mt;
+	int width, height, ofs, size;
+	image_t *image;
+	char name[256];
+	unsigned char *image_buffer = NULL;
+
+	Q_strlcpy(name, origname, sizeof(name));
+
+	/* Add the extension */
+	if (strcmp(COM_FileExtension(name), "m8"))
+	{
+		Q_strlcat(name, ".m8", sizeof(name));
+	}
+
+	size = ri.FS_LoadFile(name, (void **)&mt);
+
+	if (!mt)
+	{
+		R_Printf(PRINT_ALL, "%s: can't load %s\n", __func__, name);
+		return r_notexture;
+	}
+
+	if (size < sizeof(m8tex_t))
+	{
+		R_Printf(PRINT_ALL, "%s: can't load %s, small header\n", __func__, name);
+		ri.FS_FreeFile((void *)mt);
+		return r_notexture;
+	}
+
+	if (LittleLong (mt->version) != M8_VERSION)
+	{
+		R_Printf(PRINT_ALL, "%s: can't load %s, wrong magic value.\n", __func__, name);
+		ri.FS_FreeFile ((void *)mt);
+		return r_notexture;
+	}
+
+	width = LittleLong(mt->width[0]);
+	height = LittleLong(mt->height[0]);
+	ofs = LittleLong(mt->offsets[0]);
+
+	if ((ofs <= 0) || (width <= 0) || (height <= 0) ||
+	    (((size - ofs) / height) < width))
+	{
+		R_Printf(PRINT_ALL, "%s: can't load %s, small body\n", __func__, name);
+		ri.FS_FreeFile((void *)mt);
+		return r_notexture;
+	}
+
+	image_buffer = malloc (width * height * 4);
+	for(int i=0; i<width * height; i++)
+	{
+		unsigned char value = *((byte *)mt + ofs + i);
+		image_buffer[i * 4 + 0] = mt->palette[value].r;
+		image_buffer[i * 4 + 1] = mt->palette[value].g;
+		image_buffer[i * 4 + 2] = mt->palette[value].b;
+		image_buffer[i * 4 + 3] = value == 255 ? 0 : 255;
+	}
+
+	image = R_LoadPic(name, image_buffer, width, 0, height, 0, type, 32);
+	free(image_buffer);
+
+	ri.FS_FreeFile((void *)mt);
+
+	return image;
+}
+
 /*
  * Finds or loads the given image
  */
@@ -1083,7 +1183,7 @@ R_FindImage(char *name, imagetype_t type)
 
 	if (strcmp(ext, "pcx") == 0)
 	{
-		if (gl_retexturing->value)
+		if (r_retexturing->value)
 		{
 			GetPCXInfo(name, &realwidth, &realheight);
 			if(realwidth == 0)
@@ -1128,12 +1228,20 @@ R_FindImage(char *name, imagetype_t type)
 			image = R_LoadPic(name, pic, width, 0, height, 0, type, 8);
 		}
 	}
-	else if (strcmp(ext, "wal") == 0)
+	else if (strcmp(ext, "wal") == 0 || strcmp(ext, "m8") == 0)
 	{
-		if (gl_retexturing->value)
+		if (r_retexturing->value)
 		{
 			/* Get size of the original texture */
-			GetWalInfo(name, &realwidth, &realheight);
+			if (strcmp(ext, "m8") == 0)
+			{
+				GetM8Info(name, &realwidth, &realheight);
+			}
+			else
+			{
+				GetWalInfo(name, &realwidth, &realheight);
+			}
+
 			if(realwidth == 0)
 			{
 				/* No texture found */
@@ -1149,6 +1257,10 @@ R_FindImage(char *name, imagetype_t type)
 				image = R_LoadPic(name, pic, width, realwidth, height,
 						realheight, type, 32);
 			}
+			else if (strcmp(ext, "m8") == 0)
+			{
+				image = LoadM8(namewe, type);
+			}
 			else
 			{
 				/* WAL if no TGA/PNG/JPEG available (exists always) */
@@ -1161,6 +1273,16 @@ R_FindImage(char *name, imagetype_t type)
 				return NULL;
 			}
 		}
+		else if (strcmp(ext, "m8") == 0)
+		{
+			image = LoadM8(name, type);
+
+			if (!image)
+			{
+				/* No texture found */
+				return NULL;
+			}
+		}
 		else /* gl_retexture is not set */
 		{
 			image = LoadWal(name, type);
@@ -1183,6 +1305,12 @@ R_FindImage(char *name, imagetype_t type)
 		strcat(tmp_name, ".wal");
 		GetWalInfo(tmp_name, &realwidth, &realheight);
 
+		if (realwidth == 0 || realheight == 0) {
+			strcpy(tmp_name, namewe);
+			strcat(tmp_name, ".m8");
+			GetM8Info(tmp_name, &realwidth, &realheight);
+		}
+
 		if (realwidth == 0 || realheight == 0) {
 			/* It's a sky or model skin. */
 			strcpy(tmp_name, namewe);
@@ -1199,6 +1327,10 @@ R_FindImage(char *name, imagetype_t type)
 		{
 			image = R_LoadPic(name, pic, width, realwidth, height, realheight, type, 32);
 		}
+		else
+		{
+			return NULL;
+		}
 	}
 	else
 	{
@@ -1257,12 +1389,40 @@ R_FreeUnusedImages(void)
 	}
 }
 
+qboolean
+R_ImageHasFreeSpace(void)
+{
+	int		i, used;
+	image_t	*image;
+
+	used = 0;
+
+	for (i = 0, image = gltextures; i < numgltextures; i++, image++)
+	{
+		if (!image->name[0])
+			continue;
+		if (image->registration_sequence == registration_sequence)
+		{
+			used ++;
+		}
+	}
+
+	if (image_max < used)
+	{
+		image_max = used;
+	}
+
+	// should same size of free slots as currently used
+	return (numgltextures + used) < MAX_GLTEXTURES;
+}
+
 void
 R_InitImages(void)
 {
 	int i, j;
 
 	registration_sequence = 1;
+	image_max = 0;
 
 	/* init intensity conversions */
 	intensity = ri.Cvar_Get("gl1_intensity", "2", CVAR_ARCHIVE);
