@@ -71,15 +71,6 @@ static byte notex[4][4] = {
 	{0, 1, 1, 1}
 };
 
-typedef struct _TargaHeader
-{
-	unsigned char id_length, colormap_type, image_type;
-	unsigned short colormap_index, colormap_length;
-	unsigned char colormap_size;
-	unsigned short x_origin, y_origin, width, height;
-	unsigned char pixel_size, attributes;
-} TargaHeader;
-
 void
 R_InitParticleTexture(void)
 {
