@@ -26,10 +26,8 @@
 
 #include "header/local.h"
 
-extern int modfilelen;
-
 void
-LoadSP2(model_t *mod, void *buffer)
+LoadSP2(model_t *mod, void *buffer, int modfilelen)
 {
 	dsprite_t *sprin, *sprout;
 	int i;
