@@ -46,7 +46,8 @@ int gl_filter_min = GL_LINEAR_MIPMAP_NEAREST;
 int gl_filter_max = GL_LINEAR;
 
 gl3image_t gl3textures[MAX_GL3TEXTURES];
-int numgl3textures;
+int numgl3textures = 0;
+static int image_max = 0;
 
 void
 GL3_TextureMode(char *string)
@@ -64,7 +65,7 @@ GL3_TextureMode(char *string)
 
 	if (i == num_modes)
 	{
-		R_Printf(PRINT_ALL, "bad filter name\n");
+		R_Printf(PRINT_ALL, "bad filter name '%s' (probably from gl_texturemode)\n", string);
 		return;
 	}
 
@@ -76,34 +77,56 @@ GL3_TextureMode(char *string)
 	{
 		if (gl_anisotropic->value > gl3config.max_anisotropy)
 		{
-			ri.Cvar_SetValue("gl_anisotropic", gl3config.max_anisotropy);
-		}
-		else if (gl_anisotropic->value < 1.0)
-		{
-			ri.Cvar_SetValue("gl_anisotropic", 1.0);
+			ri.Cvar_SetValue("r_anisotropic", gl3config.max_anisotropy);
 		}
 	}
 	else
 	{
-		ri.Cvar_SetValue("gl_anisotropic", 0.0);
+		ri.Cvar_SetValue("r_anisotropic", 0.0);
 	}
 
 	gl3image_t *glt;
 
-	/* change all the existing mipmap texture objects */
+	const char* nolerplist = gl_nolerp_list->string;
+	qboolean unfiltered2D = r_2D_unfiltered->value != 0;
+
+	/* change all the existing texture objects */
 	for (i = 0, glt = gl3textures; i < numgl3textures; i++, glt++)
 	{
-		if ((glt->type != it_pic) && (glt->type != it_sky))
+		qboolean nolerp = false;
+		/* r_2D_unfiltered and gl_nolerp_list allow rendering stuff unfiltered even if gl_filter_* is filtered */
+		if ( (unfiltered2D && glt->type == it_pic) || (nolerplist != NULL && strstr(nolerplist, glt->name) != NULL) )
+		{
+			nolerp = true;
+		}
+
+		GL3_SelectTMU(GL_TEXTURE0);
+		GL3_Bind(glt->texnum);
+		if ((glt->type != it_pic) && (glt->type != it_sky)) /* mipmapped texture */
 		{
-			GL3_SelectTMU(GL_TEXTURE0);
-			GL3_Bind(glt->texnum);
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_min);
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_filter_max);
 
 			/* Set anisotropic filter if supported and enabled */
 			if (gl3config.anisotropic && gl_anisotropic->value)
 			{
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, gl_anisotropic->value);
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, max(gl_anisotropic->value, 1.f));
+			}
+		}
+		else /* texture has no mipmaps */
+		{
+			if (nolerp)
+			{
+				// this texture shouldn't be filtered at all (no gl_nolerp_list or r_2D_unfiltered case)
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+			}
+			else
+			{
+				// we can't use gl_filter_min which might be GL_*_MIPMAP_*
+				// also, there's no anisotropic filtering for textures w/o mipmaps
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_max);
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_filter_max);
 			}
 		}
 	}
@@ -194,7 +217,7 @@ GL3_Upload32(unsigned *data, int width, int height, qboolean mipmap)
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_min);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_filter_max);
 	}
-	else
+	else // if the texture has no mipmaps, we can't use gl_filter_min which might be GL_*_MIPMAP_*
 	{
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_max);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_filter_max);
@@ -202,7 +225,7 @@ GL3_Upload32(unsigned *data, int width, int height, qboolean mipmap)
 
 	if (mipmap && gl3config.anisotropic && gl_anisotropic->value)
 	{
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, gl_anisotropic->value);
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, max(gl_anisotropic->value, 1.f));
 	}
 
 	return res;
@@ -215,20 +238,12 @@ GL3_Upload32(unsigned *data, int width, int height, qboolean mipmap)
 qboolean
 GL3_Upload8(byte *data, int width, int height, qboolean mipmap, qboolean is_sky)
 {
-	unsigned trans[512 * 256];
-	int i, s;
-	int p;
+	int s = width * height;
+	unsigned *trans = malloc(s * sizeof(unsigned));
 
-	s = width * height;
-
-	if (s > sizeof(trans) / 4)
+	for (int i = 0; i < s; i++)
 	{
-		ri.Sys_Error(ERR_DROP, "GL3_Upload8: too large");
-	}
-
-	for (i = 0; i < s; i++)
-	{
-		p = data[i];
+		int p = data[i];
 		trans[i] = d_8to24table[p];
 
 		/* transparent, so scan around for
@@ -263,7 +278,9 @@ GL3_Upload8(byte *data, int width, int height, qboolean mipmap, qboolean is_sky)
 		}
 	}
 
-	return GL3_Upload32(trans, width, height, mipmap);
+	qboolean ret = GL3_Upload32(trans, width, height, mipmap);
+	free(trans);
+	return ret;
 }
 
 typedef struct
@@ -298,21 +315,16 @@ FloodFillSkin(byte *skin, int skinwidth, int skinheight)
 	byte fillcolor = *skin; /* assume this is the pixel to fill */
 	floodfill_t fifo[FLOODFILL_FIFO_SIZE];
 	int inpt = 0, outpt = 0;
-	int filledcolor = -1;
+	int filledcolor = 0;
 	int i;
 
-	if (filledcolor == -1)
+	/* attempt to find opaque black */
+	for (i = 0; i < 256; ++i)
 	{
-		filledcolor = 0;
-
-		/* attempt to find opaque black */
-		for (i = 0; i < 256; ++i)
+		if (LittleLong(d_8to24table[i]) == (255 << 0)) /* alpha 1.0 */
 		{
-			if (LittleLong(d_8to24table[i]) == (255 << 0)) /* alpha 1.0 */
-			{
-				filledcolor = i;
-				break;
-			}
+			filledcolor = i;
+			break;
 		}
 	}
 
@@ -369,8 +381,11 @@ GL3_LoadPic(char *name, byte *pic, int width, int realwidth,
 	int i;
 
 	qboolean nolerp = false;
-
-	if (gl_nolerp_list != NULL && gl_nolerp_list->string != NULL)
+	if (r_2D_unfiltered->value && type == it_pic)
+	{
+		nolerp = true;
+	}
+	else if (gl_nolerp_list != NULL && gl_nolerp_list->string != NULL)
 	{
 		nolerp = strstr(gl_nolerp_list->string, name) != NULL;
 	}
@@ -424,9 +439,22 @@ GL3_LoadPic(char *name, byte *pic, int width, int realwidth,
 
 	if (bits == 8)
 	{
-		image->has_alpha = GL3_Upload8(pic, width, height,
-					(image->type != it_pic && image->type != it_sky),
-					image->type == it_sky);
+		// resize 8bit images only when we forced such logic
+		if (r_scale8bittextures->value)
+		{
+			byte *image_converted = malloc(width * height * 4);
+			scale2x(pic, image_converted, width, height);
+			image->has_alpha = GL3_Upload8(image_converted, width * 2, height * 2,
+						(image->type != it_pic && image->type != it_sky),
+						image->type == it_sky);
+			free(image_converted);
+		}
+		else
+		{
+			image->has_alpha = GL3_Upload8(pic, width, height,
+						(image->type != it_pic && image->type != it_sky),
+						image->type == it_sky);
+		}
 	}
 	else
 	{
@@ -599,6 +627,75 @@ LoadWal(char *origname, imagetype_t type)
 	return image;
 }
 
+static gl3image_t *
+LoadM8(char *origname, imagetype_t type)
+{
+	m8tex_t *mt;
+	int width, height, ofs, size;
+	gl3image_t *image;
+	char name[256];
+	unsigned char *image_buffer = NULL;
+
+	Q_strlcpy(name, origname, sizeof(name));
+
+	/* Add the extension */
+	if (strcmp(COM_FileExtension(name), "m8"))
+	{
+		Q_strlcat(name, ".m8", sizeof(name));
+	}
+
+	size = ri.FS_LoadFile(name, (void **)&mt);
+
+	if (!mt)
+	{
+		R_Printf(PRINT_ALL, "%s: can't load %s\n", __func__, name);
+		return gl3_notexture;
+	}
+
+	if (size < sizeof(m8tex_t))
+	{
+		R_Printf(PRINT_ALL, "%s: can't load %s, small header\n", __func__, name);
+		ri.FS_FreeFile((void *)mt);
+		return gl3_notexture;
+	}
+
+	if (LittleLong (mt->version) != M8_VERSION)
+	{
+		R_Printf(PRINT_ALL, "LoadWal: can't load %s, wrong magic value.\n", name);
+		ri.FS_FreeFile ((void *)mt);
+		return gl3_notexture;
+	}
+
+	width = LittleLong(mt->width[0]);
+	height = LittleLong(mt->height[0]);
+	ofs = LittleLong(mt->offsets[0]);
+
+	if ((ofs <= 0) || (width <= 0) || (height <= 0) ||
+	    (((size - ofs) / height) < width))
+	{
+		R_Printf(PRINT_ALL, "%s: can't load %s, small body\n", __func__, name);
+		ri.FS_FreeFile((void *)mt);
+		return gl3_notexture;
+	}
+
+	image_buffer = malloc (width * height * 4);
+	for(int i=0; i<width * height; i++)
+	{
+		unsigned char value = *((byte *)mt + ofs + i);
+		image_buffer[i * 4 + 0] = mt->palette[value].r;
+		image_buffer[i * 4 + 1] = mt->palette[value].g;
+		image_buffer[i * 4 + 2] = mt->palette[value].b;
+		image_buffer[i * 4 + 3] = value == 255 ? 0 : 255;
+	}
+
+	image = GL3_LoadPic(name, image_buffer, width, 0, height, 0, type, 32);
+	free(image_buffer);
+
+	ri.FS_FreeFile((void *)mt);
+
+	return image;
+}
+
 /*
  * Finds or loads the given image
  */
@@ -658,7 +755,7 @@ GL3_FindImage(char *name, imagetype_t type)
 
 	if (strcmp(ext, "pcx") == 0)
 	{
-		if (gl_retexturing->value)
+		if (r_retexturing->value)
 		{
 			GetPCXInfo(name, &realwidth, &realheight);
 			if(realwidth == 0)
@@ -703,12 +800,20 @@ GL3_FindImage(char *name, imagetype_t type)
 			image = GL3_LoadPic(name, pic, width, 0, height, 0, type, 8);
 		}
 	}
-	else if (strcmp(ext, "wal") == 0)
+	else if (strcmp(ext, "wal") == 0 || strcmp(ext, "m8") == 0)
 	{
-		if (gl_retexturing->value)
+		if (r_retexturing->value)
 		{
 			/* Get size of the original texture */
-			GetWalInfo(name, &realwidth, &realheight);
+			if (strcmp(ext, "m8") == 0)
+			{
+				GetM8Info(name, &realwidth, &realheight);
+			}
+			else
+			{
+				GetWalInfo(name, &realwidth, &realheight);
+			}
+
 			if(realwidth == 0)
 			{
 				/* No texture found */
@@ -723,6 +828,10 @@ GL3_FindImage(char *name, imagetype_t type)
 				/* upload tga or png or jpg */
 				image = GL3_LoadPic(name, pic, width, realwidth, height, realheight, type, 32);
 			}
+			else if (strcmp(ext, "m8") == 0)
+			{
+				image = LoadM8(namewe, type);
+			}
 			else
 			{
 				/* WAL if no TGA/PNG/JPEG available (exists always) */
@@ -735,6 +844,16 @@ GL3_FindImage(char *name, imagetype_t type)
 				return NULL;
 			}
 		}
+		else if (strcmp(ext, "m8") == 0)
+		{
+			image = LoadM8(name, type);
+
+			if (!image)
+			{
+				/* No texture found */
+				return NULL;
+			}
+		}
 		else /* gl_retexture is not set */
 		{
 			image = LoadWal(name, type);
@@ -757,6 +876,12 @@ GL3_FindImage(char *name, imagetype_t type)
 		strcat(tmp_name, ".wal");
 		GetWalInfo(tmp_name, &realwidth, &realheight);
 
+		if (realwidth == 0 || realheight == 0) {
+			strcpy(tmp_name, namewe);
+			strcat(tmp_name, ".m8");
+			GetM8Info(tmp_name, &realwidth, &realheight);
+		}
+
 		if (realwidth == 0 || realheight == 0) {
 			/* It's a sky or model skin. */
 			strcpy(tmp_name, namewe);
@@ -772,6 +897,8 @@ GL3_FindImage(char *name, imagetype_t type)
 		if(LoadSTB(name, ext, &pic, &width, &height))
 		{
 			image = GL3_LoadPic(name, pic, width, realwidth, height, realheight, type, 32);
+		} else {
+			return NULL;
 		}
 	}
 	else
@@ -831,6 +958,33 @@ GL3_FreeUnusedImages(void)
 	}
 }
 
+qboolean
+GL3_ImageHasFreeSpace(void)
+{
+	int		i, used;
+	gl3image_t	*image;
+
+	used = 0;
+
+	for (i = 0, image = gl3textures; i < numgl3textures; i++, image++)
+	{
+		if (!image->name[0])
+			continue;
+		if (image->registration_sequence == registration_sequence)
+		{
+			used ++;
+		}
+	}
+
+	if (image_max < used)
+	{
+		image_max = used;
+	}
+
+	// should same size of free slots as currently used
+	return (numgl3textures + used) < MAX_GL3TEXTURES;
+}
+
 void
 GL3_ShutdownImages(void)
 {
@@ -869,7 +1023,8 @@ static qboolean IsNPOT(int v)
 void
 GL3_ImageList_f(void)
 {
-	int i, texels=0;
+	int i, used, texels;
+	qboolean	freeup;
 	gl3image_t *image;
 	const char *formatstrings[2] = {
 		"RGB ",
@@ -881,15 +1036,25 @@ GL3_ImageList_f(void)
 	};
 
 	R_Printf(PRINT_ALL, "------------------\n");
+	texels = 0;
+	used = 0;
 
 	for (i = 0, image = gl3textures; i < numgl3textures; i++, image++)
 	{
 		int w, h;
+		char *in_use = "";
 		qboolean isNPOT = false;
 		if (image->texnum == 0)
 		{
 			continue;
 		}
+
+		if (image->registration_sequence == registration_sequence)
+		{
+			in_use = "*";
+			used++;
+		}
+
 		w = image->width;
 		h = image->height;
 
@@ -919,9 +1084,11 @@ GL3_ImageList_f(void)
 				break;
 		}
 
-		R_Printf(PRINT_ALL, " %3i %3i %s %s: %s\n", w, h,
-		         formatstrings[image->has_alpha], potstrings[isNPOT], image->name);
+		R_Printf(PRINT_ALL, " %3i %3i %s %s: %s %s\n", w, h,
+		         formatstrings[image->has_alpha], potstrings[isNPOT], image->name, in_use);
 	}
 
 	R_Printf(PRINT_ALL, "Total texel count (not counting mipmaps): %i\n", texels);
+	freeup = GL3_ImageHasFreeSpace();
+	R_Printf(PRINT_ALL, "Used %d of %d images%s.\n", used, image_max, freeup ? ", has free space" : "");
 }
