@@ -50,8 +50,6 @@ refdef_t gl3_newrefdef;
 
 viddef_t vid;
 gl3model_t *gl3_worldmodel;
-gl3model_t *currentmodel;
-entity_t *currententity;
 
 float gl3depthmin=0.0f, gl3depthmax=1.0f;
 
@@ -81,7 +79,8 @@ const hmm_mat4 gl3_identityMat4 = {{
 
 cvar_t *gl_msaa_samples;
 cvar_t *r_vsync;
-cvar_t *gl_retexturing;
+cvar_t *r_retexturing;
+cvar_t *r_scale8bittextures;
 cvar_t *vid_fullscreen;
 cvar_t *r_mode;
 cvar_t *r_customwidth;
@@ -108,6 +107,7 @@ cvar_t *r_norefresh;
 cvar_t *r_drawentities;
 cvar_t *r_drawworld;
 cvar_t *gl_nolerp_list;
+cvar_t *r_2D_unfiltered;
 cvar_t *gl_nobind;
 cvar_t *r_lockpvs;
 cvar_t *r_novis;
@@ -122,6 +122,7 @@ cvar_t *gl_lightmap;
 cvar_t *gl_shadows;
 cvar_t *gl3_debugcontext;
 cvar_t *gl3_usebigvbo;
+cvar_t *r_fixsurfsky;
 
 // Yaw-Pitch-Roll
 // equivalent to R_z * R_y * R_x where R_x is the trans matrix for rotating around X axis for aroundXdeg
@@ -197,8 +198,9 @@ GL3_Register(void)
 
 	gl_drawbuffer = ri.Cvar_Get("gl_drawbuffer", "GL_BACK", 0);
 	r_vsync = ri.Cvar_Get("r_vsync", "1", CVAR_ARCHIVE);
-	gl_msaa_samples = ri.Cvar_Get ( "gl_msaa_samples", "0", CVAR_ARCHIVE );
-	gl_retexturing = ri.Cvar_Get("gl_retexturing", "1", CVAR_ARCHIVE);
+	gl_msaa_samples = ri.Cvar_Get ( "r_msaa_samples", "0", CVAR_ARCHIVE );
+	r_retexturing = ri.Cvar_Get("r_retexturing", "1", CVAR_ARCHIVE);
+	r_scale8bittextures = ri.Cvar_Get("r_scale8bittextures", "0", CVAR_ARCHIVE);
 	gl3_debugcontext = ri.Cvar_Get("gl3_debugcontext", "0", 0);
 	r_mode = ri.Cvar_Get("r_mode", "4", CVAR_ARCHIVE);
 	r_customwidth = ri.Cvar_Get("r_customwidth", "1024", CVAR_ARCHIVE);
@@ -216,13 +218,16 @@ GL3_Register(void)
 	r_drawentities = ri.Cvar_Get("r_drawentities", "1", 0);
 	r_drawworld = ri.Cvar_Get("r_drawworld", "1", 0);
 	r_fullbright = ri.Cvar_Get("r_fullbright", "0", 0);
+	r_fixsurfsky = ri.Cvar_Get("r_fixsurfsky", "0", CVAR_ARCHIVE);
 
 	/* don't bilerp characters and crosshairs */
-	gl_nolerp_list = ri.Cvar_Get("gl_nolerp_list", "pics/conchars.pcx pics/ch1.pcx pics/ch2.pcx pics/ch3.pcx", 0);
+	gl_nolerp_list = ri.Cvar_Get("r_nolerp_list", "pics/conchars.pcx pics/ch1.pcx pics/ch2.pcx pics/ch3.pcx", 0);
+	/* don't bilerp any 2D elements */
+	r_2D_unfiltered = ri.Cvar_Get("r_2D_unfiltered", "0", CVAR_ARCHIVE);
 	gl_nobind = ri.Cvar_Get("gl_nobind", "0", 0);
 
 	gl_texturemode = ri.Cvar_Get("gl_texturemode", "GL_LINEAR_MIPMAP_NEAREST", CVAR_ARCHIVE);
-	gl_anisotropic = ri.Cvar_Get("gl_anisotropic", "0", CVAR_ARCHIVE);
+	gl_anisotropic = ri.Cvar_Get("r_anisotropic", "0", CVAR_ARCHIVE);
 
 	vid_fullscreen = ri.Cvar_Get("vid_fullscreen", "0", CVAR_ARCHIVE);
 	vid_gamma = ri.Cvar_Get("vid_gamma", "1.2", CVAR_ARCHIVE);
@@ -233,7 +238,7 @@ GL3_Register(void)
 	gl3_overbrightbits = ri.Cvar_Get("gl3_overbrightbits", "1.3", CVAR_ARCHIVE);
 
 	gl_lightmap = ri.Cvar_Get("gl_lightmap", "0", 0);
-	gl_shadows = ri.Cvar_Get("gl_shadows", "0", CVAR_ARCHIVE);
+	gl_shadows = ri.Cvar_Get("r_shadows", "0", CVAR_ARCHIVE);
 
 	r_modulate = ri.Cvar_Get("r_modulate", "1", CVAR_ARCHIVE);
 	gl_zfix = ri.Cvar_Get("gl_zfix", "0", 0);
@@ -268,7 +273,7 @@ GL3_Register(void)
 	//gl_modulate = ri.Cvar_Get("gl_modulate", "1", CVAR_ARCHIVE);
 	//r_mode = ri.Cvar_Get("r_mode", "4", CVAR_ARCHIVE);
 	//gl_lightmap = ri.Cvar_Get("gl_lightmap", "0", 0);
-	//gl_shadows = ri.Cvar_Get("gl_shadows", "0", CVAR_ARCHIVE);
+	//gl_shadows = ri.Cvar_Get("r_shadows", "0", CVAR_ARCHIVE);
 	//gl_nobind = ri.Cvar_Get("gl_nobind", "0", 0);
 	gl_showtris = ri.Cvar_Get("gl_showtris", "0", 0);
 	gl_showbbox = Cvar_Get("gl_showbbox", "0", 0);
@@ -282,7 +287,7 @@ GL3_Register(void)
 	//gl_texturemode = ri.Cvar_Get("gl_texturemode", "GL_LINEAR_MIPMAP_NEAREST", CVAR_ARCHIVE);
 	gl1_texturealphamode = ri.Cvar_Get("gl1_texturealphamode", "default", CVAR_ARCHIVE);
 	gl1_texturesolidmode = ri.Cvar_Get("gl1_texturesolidmode", "default", CVAR_ARCHIVE);
-	//gl_anisotropic = ri.Cvar_Get("gl_anisotropic", "0", CVAR_ARCHIVE);
+	//gl_anisotropic = ri.Cvar_Get("r_anisotropic", "0", CVAR_ARCHIVE);
 	//r_lockpvs = ri.Cvar_Get("r_lockpvs", "0", 0);
 
 	//gl1_palettedtexture = ri.Cvar_Get("gl1_palettedtexture", "0", CVAR_ARCHIVE); NOPE.
@@ -297,9 +302,9 @@ GL3_Register(void)
 
 	//r_customwidth = ri.Cvar_Get("r_customwidth", "1024", CVAR_ARCHIVE);
 	//r_customheight = ri.Cvar_Get("r_customheight", "768", CVAR_ARCHIVE);
-	//gl_msaa_samples = ri.Cvar_Get ( "gl_msaa_samples", "0", CVAR_ARCHIVE );
+	//gl_msaa_samples = ri.Cvar_Get ( "r_msaa_samples", "0", CVAR_ARCHIVE );
 
-	//gl_retexturing = ri.Cvar_Get("gl_retexturing", "1", CVAR_ARCHIVE);
+	//r_retexturing = ri.Cvar_Get("r_retexturing", "1", CVAR_ARCHIVE);
 
 
 	gl1_stereo = ri.Cvar_Get( "gl1_stereo", "0", CVAR_ARCHIVE );
@@ -371,9 +376,6 @@ GL3_SetMode(void)
 
 	fullscreen = (int)vid_fullscreen->value;
 
-	vid_fullscreen->modified = false;
-	r_mode->modified = false;
-
 	/* a bit hackish approach to enable custom resolutions:
 	   Glimp_SetMode needs these values set for mode -1 */
 	vid.width = r_customwidth->value;
@@ -399,7 +401,7 @@ GL3_SetMode(void)
 			if (gl_msaa_samples->value != 0.0f)
 			{
 				R_Printf(PRINT_ALL, "gl_msaa_samples was %d - will try again with gl_msaa_samples = 0\n", (int)gl_msaa_samples->value);
-				ri.Cvar_SetValue("gl_msaa_samples", 0.0f);
+				ri.Cvar_SetValue("r_msaa_samples", 0.0f);
 				gl_msaa_samples->modified = false;
 
 				if ((err = SetMode_impl(&vid.width, &vid.height, r_mode->value, 0)) == rserr_ok)
@@ -774,7 +776,7 @@ GL3_DrawBeam(entity_t *e)
 }
 
 static void
-GL3_DrawSpriteModel(entity_t *e)
+GL3_DrawSpriteModel(entity_t *e, gl3model_t *currentmodel)
 {
 	float alpha = 1.0F;
 	gl3_3D_vtx_t verts[4];
@@ -853,7 +855,7 @@ GL3_DrawSpriteModel(entity_t *e)
 }
 
 static void
-GL3_DrawNullModel(void)
+GL3_DrawNullModel(entity_t *currententity)
 {
 	vec3_t shadelight;
 
@@ -863,7 +865,7 @@ GL3_DrawNullModel(void)
 	}
 	else
 	{
-		GL3_LightPoint(currententity->origin, shadelight);
+		GL3_LightPoint(currententity, currententity->origin, shadelight);
 	}
 
 	hmm_mat4 origModelMat = gl3state.uni3DData.transModelMat4;
@@ -978,7 +980,7 @@ GL3_DrawEntitiesOnList(void)
 	/* draw non-transparent first */
 	for (i = 0; i < gl3_newrefdef.num_entities; i++)
 	{
-		currententity = &gl3_newrefdef.entities[i];
+		entity_t *currententity = &gl3_newrefdef.entities[i];
 
 		if (currententity->flags & RF_TRANSLUCENT)
 		{
@@ -991,11 +993,11 @@ GL3_DrawEntitiesOnList(void)
 		}
 		else
 		{
-			currentmodel = currententity->model;
+			gl3model_t *currentmodel = currententity->model;
 
 			if (!currentmodel)
 			{
-				GL3_DrawNullModel();
+				GL3_DrawNullModel(currententity);
 				continue;
 			}
 
@@ -1005,10 +1007,10 @@ GL3_DrawEntitiesOnList(void)
 					GL3_DrawAliasModel(currententity);
 					break;
 				case mod_brush:
-					GL3_DrawBrushModel(currententity);
+					GL3_DrawBrushModel(currententity, currentmodel);
 					break;
 				case mod_sprite:
-					GL3_DrawSpriteModel(currententity);
+					GL3_DrawSpriteModel(currententity, currentmodel);
 					break;
 				default:
 					ri.Sys_Error(ERR_DROP, "Bad modeltype");
@@ -1024,7 +1026,7 @@ GL3_DrawEntitiesOnList(void)
 
 	for (i = 0; i < gl3_newrefdef.num_entities; i++)
 	{
-		currententity = &gl3_newrefdef.entities[i];
+		entity_t *currententity = &gl3_newrefdef.entities[i];
 
 		if (!(currententity->flags & RF_TRANSLUCENT))
 		{
@@ -1037,11 +1039,11 @@ GL3_DrawEntitiesOnList(void)
 		}
 		else
 		{
-			currentmodel = currententity->model;
+			gl3model_t *currentmodel = currententity->model;
 
 			if (!currentmodel)
 			{
-				GL3_DrawNullModel();
+				GL3_DrawNullModel(currententity);
 				continue;
 			}
 
@@ -1051,10 +1053,10 @@ GL3_DrawEntitiesOnList(void)
 					GL3_DrawAliasModel(currententity);
 					break;
 				case mod_brush:
-					GL3_DrawBrushModel(currententity);
+					GL3_DrawBrushModel(currententity, currentmodel);
 					break;
 				case mod_sprite:
-					GL3_DrawSpriteModel(currententity);
+					GL3_DrawSpriteModel(currententity, currentmodel);
 					break;
 				default:
 					ri.Sys_Error(ERR_DROP, "Bad modeltype");
@@ -1581,7 +1583,7 @@ GL3_GetSpecialBufferModeForStereoMode(enum stereo_modes stereo_mode) {
 #endif // 0
 
 static void
-GL3_SetLightLevel(void)
+GL3_SetLightLevel(entity_t *currententity)
 {
 	vec3_t shadelight = {0};
 
@@ -1591,7 +1593,7 @@ GL3_SetLightLevel(void)
 	}
 
 	/* save off light value for server to look at */
-	GL3_LightPoint(gl3_newrefdef.vieworg, shadelight);
+	GL3_LightPoint(currententity, gl3_newrefdef.vieworg, shadelight);
 
 	/* pick the greatest component, which should be the
 	 * same as the mono value returned by software */
@@ -1623,7 +1625,7 @@ static void
 GL3_RenderFrame(refdef_t *fd)
 {
 	GL3_RenderView(fd);
-	GL3_SetLightLevel();
+	GL3_SetLightLevel(NULL);
 	GL3_SetGL2D();
 
 	if(v_blend[3] != 0.0f)
@@ -1687,12 +1689,6 @@ GL3_Clear(void)
 void
 GL3_BeginFrame(float camera_separation)
 {
-	/* change modes if necessary */
-	if (r_mode->modified)
-	{
-		vid_fullscreen->modified = true;
-	}
-
 #if 0 // TODO: stereo stuff
 	gl_state.camera_separation = camera_separation;
 	// force a vid_restart if gl1_stereo has been modified.
@@ -1773,11 +1769,14 @@ GL3_BeginFrame(float camera_separation)
 	}
 
 	/* texturemode stuff */
-	if (gl_texturemode->modified || (gl3config.anisotropic && gl_anisotropic->modified))
+	if (gl_texturemode->modified || (gl3config.anisotropic && gl_anisotropic->modified)
+	    || gl_nolerp_list->modified || r_2D_unfiltered->modified)
 	{
 		GL3_TextureMode(gl_texturemode->string);
 		gl_texturemode->modified = false;
 		gl_anisotropic->modified = false;
+		gl_nolerp_list->modified = false;
+		r_2D_unfiltered->modified = false;
 	}
 
 	if (r_vsync->modified)
@@ -1822,6 +1821,17 @@ GL3_SetPalette(const unsigned char *palette)
 	glClearColor(1, 0, 0.5, 0.5);
 }
 
+/*
+=====================
+GL3_EndWorldRenderpass
+=====================
+*/
+static qboolean
+GL3_EndWorldRenderpass( void )
+{
+	return true;
+}
+
 Q2_DLL_EXPORTED refexport_t
 GetRefAPI(refimport_t imp)
 {
@@ -1862,8 +1872,13 @@ GetRefAPI(refimport_t imp)
 	re.SetPalette = GL3_SetPalette;
 
 	re.BeginFrame = GL3_BeginFrame;
+	re.EndWorldRenderpass = GL3_EndWorldRenderpass;
 	re.EndFrame = GL3_EndFrame;
 
+    // Tell the client that we're unsing the
+	// new renderer restart API.
+    ri.Vid_RequestRestart(RESTART_NO);
+
 	return re;
 }
 
