@@ -29,15 +29,40 @@
 
 enum { MAX_MOD_KNOWN = 512 };
 
-static gl3model_t *loadmodel;
 YQ2_ALIGNAS_TYPE(int) static byte mod_novis[MAX_MAP_LEAFS / 8];
 gl3model_t mod_known[MAX_MOD_KNOWN];
 static int mod_numknown;
+static int mod_max = 0;
 int registration_sequence;
-static byte *mod_base;
 
-/* the inline * models from the current map are kept seperate */
-gl3model_t mod_inline[MAX_MOD_KNOWN];
+//===============================================================================
+
+static qboolean
+Mod_HasFreeSpace(void)
+{
+	int		i, used;
+	gl3model_t	*mod;
+
+	used = 0;
+
+	for (i=0, mod=mod_known ; i < mod_numknown ; i++, mod++)
+	{
+		if (!mod->name[0])
+			continue;
+		if (mod->registration_sequence == registration_sequence)
+		{
+			used ++;
+		}
+	}
+
+	if (mod_max < used)
+	{
+		mod_max = used;
+	}
+
+	// should same size of free slots as currently used
+	return (mod_numknown + mod_max) < MAX_MOD_KNOWN;
+}
 
 mleaf_t *
 GL3_Mod_PointInLeaf(vec3_t p, gl3model_t *model)
@@ -76,8 +101,8 @@ GL3_Mod_PointInLeaf(vec3_t p, gl3model_t *model)
 	return NULL; /* never reached */
 }
 
-byte*
-GL3_Mod_ClusterPVS(int cluster, gl3model_t *model)
+const byte*
+GL3_Mod_ClusterPVS(int cluster, const gl3model_t *model)
 {
 	if ((cluster == -1) || !model->vis)
 	{
@@ -92,35 +117,49 @@ GL3_Mod_ClusterPVS(int cluster, gl3model_t *model)
 void
 GL3_Mod_Modellist_f(void)
 {
-	int i;
+	int i, total, used;
 	gl3model_t *mod;
-	int total;
+	qboolean freeup;
 
 	total = 0;
+	used = 0;
 	R_Printf(PRINT_ALL, "Loaded models:\n");
 
 	for (i = 0, mod = mod_known; i < mod_numknown; i++, mod++)
 	{
+		char *in_use = "";
+
+		if (mod->registration_sequence == registration_sequence)
+		{
+			in_use = "*";
+			used ++;
+		}
+
 		if (!mod->name[0])
 		{
 			continue;
 		}
 
-		R_Printf(PRINT_ALL, "%8i : %s\n", mod->extradatasize, mod->name);
+		R_Printf(PRINT_ALL, "%8i : %s %s\n",
+			mod->extradatasize, mod->name, in_use);
 		total += mod->extradatasize;
 	}
 
 	R_Printf(PRINT_ALL, "Total resident: %i\n", total);
+	// update statistics
+	freeup = Mod_HasFreeSpace();
+	R_Printf(PRINT_ALL, "Used %d of %d models%s.\n", used, mod_max, freeup ? ", has free space" : "");
 }
 
 void
 GL3_Mod_Init(void)
 {
+	mod_max = 0;
 	memset(mod_novis, 0xff, sizeof(mod_novis));
 }
 
 static void
-Mod_LoadLighting(lump_t *l)
+Mod_LoadLighting(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	if (!l->filelen)
 	{
@@ -133,7 +172,7 @@ Mod_LoadLighting(lump_t *l)
 }
 
 static void
-Mod_LoadVisibility(lump_t *l)
+Mod_LoadVisibility(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int i;
 
@@ -156,7 +195,7 @@ Mod_LoadVisibility(lump_t *l)
 }
 
 static void
-Mod_LoadVertexes(lump_t *l)
+Mod_LoadVertexes(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	dvertex_t *in;
 	mvertex_t *out;
@@ -185,10 +224,10 @@ Mod_LoadVertexes(lump_t *l)
 }
 
 static void
-Mod_LoadSubmodels(lump_t *l)
+Mod_LoadSubmodels(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	dmodel_t *in;
-	mmodel_t *out;
+	gl3model_t *out;
 	int i, j, count;
 
 	in = (void *)(mod_base + l->fileofs);
@@ -207,6 +246,19 @@ Mod_LoadSubmodels(lump_t *l)
 
 	for (i = 0; i < count; i++, in++, out++)
 	{
+		if (i == 0)
+		{
+			// copy parent as template for first model
+			memcpy(out, loadmodel, sizeof(*out));
+		}
+		else
+		{
+			// copy first as template for model
+			memcpy(out, loadmodel->submodels, sizeof(*out));
+		}
+
+		Com_sprintf (out->name, sizeof(out->name), "*%d", i);
+
 		for (j = 0; j < 3; j++)
 		{
 			/* spread the mins / maxs by a pixel */
@@ -216,14 +268,22 @@ Mod_LoadSubmodels(lump_t *l)
 		}
 
 		out->radius = Mod_RadiusFromBounds(out->mins, out->maxs);
-		out->headnode = LittleLong(in->headnode);
-		out->firstface = LittleLong(in->firstface);
-		out->numfaces = LittleLong(in->numfaces);
+		out->firstnode = LittleLong(in->headnode);
+		out->firstmodelsurface = LittleLong(in->firstface);
+		out->nummodelsurfaces = LittleLong(in->numfaces);
+		// visleafs
+		out->numleafs = 0;
+		//  check limits
+		if (out->firstnode >= loadmodel->numnodes)
+		{
+			ri.Sys_Error(ERR_DROP, "%s: Inline model %i has bad firstnode",
+					__func__, i);
+		}
 	}
 }
 
 static void
-Mod_LoadEdges(lump_t *l)
+Mod_LoadEdges(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	dedge_t *in;
 	medge_t *out;
@@ -251,7 +311,7 @@ Mod_LoadEdges(lump_t *l)
 }
 
 static void
-Mod_LoadTexinfo(lump_t *l)
+Mod_LoadTexinfo(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	texinfo_t *in;
 	mtexinfo_t *out, *step;
@@ -297,6 +357,12 @@ Mod_LoadTexinfo(lump_t *l)
 
 		out->image = GL3_FindImage(name, it_wall);
 
+		if (!out->image || out->image == gl3_notexture)
+		{
+			Com_sprintf(name, sizeof(name), "textures/%s.m8", in->texture);
+			out->image = GL3_FindImage(name, it_wall);
+		}
+
 		if (!out->image)
 		{
 			R_Printf(PRINT_ALL, "Couldn't load %s\n", name);
@@ -321,7 +387,7 @@ Mod_LoadTexinfo(lump_t *l)
  * Fills in s->texturemins[] and s->extents[]
  */
 static void
-Mod_CalcSurfaceExtents(msurface_t *s)
+Mod_CalcSurfaceExtents(gl3model_t *loadmodel, msurface_t *s)
 {
 	float mins[2], maxs[2], val;
 	int i, j, e;
@@ -379,7 +445,7 @@ Mod_CalcSurfaceExtents(msurface_t *s)
 extern void
 GL3_SubdivideSurface(msurface_t *fa, gl3model_t* loadmodel);
 
-static int calcTexinfoAndFacesSize(const lump_t *fl, const lump_t *tl)
+static int calcTexinfoAndFacesSize(byte *mod_base, const lump_t *fl, const lump_t *tl)
 {
 	dface_t* face_in = (void *)(mod_base + fl->fileofs);
 	texinfo_t* texinfo_in = (void *)(mod_base + tl->fileofs);
@@ -455,7 +521,7 @@ static int calcTexinfoAndFacesSize(const lump_t *fl, const lump_t *tl)
 }
 
 static void
-Mod_LoadFaces(lump_t *l)
+Mod_LoadFaces(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	dface_t *in;
 	msurface_t *out;
@@ -463,8 +529,6 @@ Mod_LoadFaces(lump_t *l)
 	int planenum, side;
 	int ti;
 
-	cvar_t* gl_fixsurfsky = ri.Cvar_Get("gl_fixsurfsky", "0", CVAR_ARCHIVE);
-
 	in = (void *)(mod_base + l->fileofs);
 
 	if (l->filelen % sizeof(*in))
@@ -479,8 +543,6 @@ Mod_LoadFaces(lump_t *l)
 	loadmodel->surfaces = out;
 	loadmodel->numsurfaces = count;
 
-	currentmodel = loadmodel;
-
 	GL3_LM_BeginBuildingLightmaps(loadmodel);
 
 	for (surfnum = 0; surfnum < count; surfnum++, in++, out++)
@@ -510,7 +572,7 @@ Mod_LoadFaces(lump_t *l)
 
 		out->texinfo = loadmodel->texinfo + ti;
 
-		Mod_CalcSurfaceExtents(out);
+		Mod_CalcSurfaceExtents(loadmodel, out);
 
 		/* lighting info */
 		for (i = 0; i < MAX_LIGHTMAPS_PER_SURFACE; i++)
@@ -543,7 +605,7 @@ Mod_LoadFaces(lump_t *l)
 			GL3_SubdivideSurface(out, loadmodel); /* cut up polygon for warps */
 		}
 
-		if (gl_fixsurfsky->value)
+		if (r_fixsurfsky->value)
 		{
 			if (out->texinfo->flags & SURF_SKY)
 			{
@@ -559,7 +621,7 @@ Mod_LoadFaces(lump_t *l)
 
 		if (!(out->texinfo->flags & SURF_WARP))
 		{
-			GL3_LM_BuildPolygonFromSurface(out);
+			GL3_LM_BuildPolygonFromSurface(loadmodel, out);
 		}
 	}
 
@@ -581,7 +643,7 @@ Mod_SetParent(mnode_t *node, mnode_t *parent)
 }
 
 static void
-Mod_LoadNodes(lump_t *l)
+Mod_LoadNodes(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int i, j, count, p;
 	dnode_t *in;
@@ -635,7 +697,7 @@ Mod_LoadNodes(lump_t *l)
 }
 
 static void
-Mod_LoadLeafs(lump_t *l)
+Mod_LoadLeafs(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	dleaf_t *in;
 	mleaf_t *out;
@@ -685,7 +747,7 @@ Mod_LoadLeafs(lump_t *l)
 }
 
 static void
-Mod_LoadMarksurfaces(lump_t *l)
+Mod_LoadMarksurfaces(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int i, j, count;
 	short *in;
@@ -719,7 +781,7 @@ Mod_LoadMarksurfaces(lump_t *l)
 }
 
 static void
-Mod_LoadSurfedges(lump_t *l)
+Mod_LoadSurfedges(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int i, count;
 	int *in, *out;
@@ -752,7 +814,7 @@ Mod_LoadSurfedges(lump_t *l)
 }
 
 static void
-Mod_LoadPlanes(lump_t *l)
+Mod_LoadPlanes(gl3model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int i, j;
 	cplane_t *out;
@@ -819,9 +881,9 @@ Mod_LoadBrushModel(gl3model_t *mod, void *buffer, int modfilelen)
 {
 	int i;
 	dheader_t *header;
-	mmodel_t *bm;
+	byte *mod_base;
 
-	if (loadmodel != mod_known)
+	if (mod != mod_known)
 	{
 		ri.Sys_Error(ERR_DROP, "Loaded a brush model after the world");
 	}
@@ -854,62 +916,30 @@ Mod_LoadBrushModel(gl3model_t *mod, void *buffer, int modfilelen)
 		hunkSize += calcLumpHunkSize(&header->lumps[LUMP_SURFEDGES], sizeof(int), sizeof(int));
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_LIGHTING], 1, 1);
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_PLANES], sizeof(dplane_t), sizeof(cplane_t)*2);
-	hunkSize += calcTexinfoAndFacesSize(&header->lumps[LUMP_FACES], &header->lumps[LUMP_TEXINFO]);
+	hunkSize += calcTexinfoAndFacesSize(mod_base, &header->lumps[LUMP_FACES], &header->lumps[LUMP_TEXINFO]);
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_LEAFFACES], sizeof(short), sizeof(msurface_t *)); // yes, out is indeeed a pointer!
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_VISIBILITY], 1, 1);
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_LEAFS], sizeof(dleaf_t), sizeof(mleaf_t));
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_NODES], sizeof(dnode_t), sizeof(mnode_t));
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_MODELS], sizeof(dmodel_t), sizeof(mmodel_t));
 
-	loadmodel->extradata = Hunk_Begin(hunkSize);
-	loadmodel->type = mod_brush;
+	mod->extradata = Hunk_Begin(hunkSize);
+	mod->type = mod_brush;
 
 	/* load into heap */
-	Mod_LoadVertexes(&header->lumps[LUMP_VERTEXES]);
-	Mod_LoadEdges(&header->lumps[LUMP_EDGES]);
-	Mod_LoadSurfedges(&header->lumps[LUMP_SURFEDGES]);
-	Mod_LoadLighting(&header->lumps[LUMP_LIGHTING]);
-	Mod_LoadPlanes(&header->lumps[LUMP_PLANES]);
-	Mod_LoadTexinfo(&header->lumps[LUMP_TEXINFO]);
-	Mod_LoadFaces(&header->lumps[LUMP_FACES]);
-	Mod_LoadMarksurfaces(&header->lumps[LUMP_LEAFFACES]);
-	Mod_LoadVisibility(&header->lumps[LUMP_VISIBILITY]);
-	Mod_LoadLeafs(&header->lumps[LUMP_LEAFS]);
-	Mod_LoadNodes(&header->lumps[LUMP_NODES]);
-	Mod_LoadSubmodels(&header->lumps[LUMP_MODELS]);
+	Mod_LoadVertexes(mod, mod_base, &header->lumps[LUMP_VERTEXES]);
+	Mod_LoadEdges(mod, mod_base, &header->lumps[LUMP_EDGES]);
+	Mod_LoadSurfedges(mod, mod_base, &header->lumps[LUMP_SURFEDGES]);
+	Mod_LoadLighting(mod, mod_base, &header->lumps[LUMP_LIGHTING]);
+	Mod_LoadPlanes(mod, mod_base, &header->lumps[LUMP_PLANES]);
+	Mod_LoadTexinfo(mod, mod_base, &header->lumps[LUMP_TEXINFO]);
+	Mod_LoadFaces(mod, mod_base, &header->lumps[LUMP_FACES]);
+	Mod_LoadMarksurfaces(mod, mod_base, &header->lumps[LUMP_LEAFFACES]);
+	Mod_LoadVisibility(mod, mod_base, &header->lumps[LUMP_VISIBILITY]);
+	Mod_LoadLeafs(mod, mod_base, &header->lumps[LUMP_LEAFS]);
+	Mod_LoadNodes(mod, mod_base, &header->lumps[LUMP_NODES]);
+	Mod_LoadSubmodels (mod, mod_base, &header->lumps[LUMP_MODELS]);
 	mod->numframes = 2; /* regular and alternate animation */
-
-	/* set up the submodels */
-	for (i = 0; i < mod->numsubmodels; i++)
-	{
-		gl3model_t *starmod;
-
-		bm = &mod->submodels[i];
-		starmod = &mod_inline[i];
-
-		*starmod = *loadmodel;
-
-		starmod->firstmodelsurface = bm->firstface;
-		starmod->nummodelsurfaces = bm->numfaces;
-		starmod->firstnode = bm->headnode;
-
-		if (starmod->firstnode >= loadmodel->numnodes)
-		{
-			ri.Sys_Error(ERR_DROP, "%s: Inline model %i has bad firstnode",
-					__func__, i);
-		}
-
-		VectorCopy(bm->maxs, starmod->maxs);
-		VectorCopy(bm->mins, starmod->mins);
-		starmod->radius = bm->radius;
-
-		if (i == 0)
-		{
-			*loadmodel = *starmod;
-		}
-
-		starmod->numleafs = bm->visleafs;
-	}
 }
 
 static void
@@ -940,7 +970,7 @@ extern void GL3_LoadSP2(gl3model_t *mod, void *buffer, int modfilelen);
  * Loads in a model for the given name
  */
 static gl3model_t *
-Mod_ForName(char *name, qboolean crash)
+Mod_ForName (char *name, gl3model_t *parent_model, qboolean crash)
 {
 	gl3model_t *mod;
 	unsigned *buf;
@@ -952,17 +982,17 @@ Mod_ForName(char *name, qboolean crash)
 	}
 
 	/* inline models are grabbed only from worldmodel */
-	if (name[0] == '*')
+	if (name[0] == '*' && parent_model)
 	{
 		i = (int)strtol(name + 1, (char **)NULL, 10);
 
-		if ((i < 1) || !gl3_worldmodel || (i >= gl3_worldmodel->numsubmodels))
+		if (i < 1 || i >= parent_model->numsubmodels)
 		{
 			ri.Sys_Error(ERR_DROP, "%s: bad inline model number",
 					__func__);
 		}
 
-		return &mod_inline[i];
+		return &parent_model->submodels[i];
 	}
 
 	/* search the currently loaded models */
@@ -1015,8 +1045,6 @@ Mod_ForName(char *name, qboolean crash)
 		return NULL;
 	}
 
-	loadmodel = mod;
-
 	/* call the apropriate loader */
 	switch (LittleLong(*(unsigned *)buf))
 	{
@@ -1038,7 +1066,7 @@ Mod_ForName(char *name, qboolean crash)
 			break;
 	}
 
-	loadmodel->extradatasize = Hunk_End();
+	mod->extradatasize = Hunk_End();
 
 	ri.FS_FreeFile(buf);
 
@@ -1071,7 +1099,7 @@ GL3_BeginRegistration(char *model)
 		Mod_Free(&mod_known[0]);
 	}
 
-	gl3_worldmodel = Mod_ForName(fullname, true);
+	gl3_worldmodel = Mod_ForName(fullname, NULL, true);
 
 	gl3_viewcluster = -1;
 }
@@ -1084,7 +1112,7 @@ GL3_RegisterModel(char *name)
 	dsprite_t *sprout;
 	dmdl_t *pheader;
 
-	mod = Mod_ForName(name, false);
+	mod = Mod_ForName(name, gl3_worldmodel, false);
 
 	if (mod)
 	{
@@ -1129,6 +1157,12 @@ GL3_EndRegistration(void)
 	int i;
 	gl3model_t *mod;
 
+	if (Mod_HasFreeSpace() && GL3_ImageHasFreeSpace())
+	{
+		// should be enough space for load next maps
+		return;
+	}
+
 	for (i = 0, mod = mod_known; i < mod_numknown; i++, mod++)
 	{
 		if (!mod->name[0])
