@@ -169,7 +169,7 @@ R_RecursiveClipBPoly(entity_t *currententity, bedge_t *pedges, mnode_t *pnode, m
 {
 	bedge_t		*psideedges[2], *pnextedge, *ptedge;
 	int		i, side, lastside;
-	mplane_t	*splitplane, tplane;
+	cplane_t	*splitplane, tplane;
 	mvertex_t	*pvert, *plastvert, *ptvert;
 	mnode_t		*pn;
 	qboolean	makeclippededge;
@@ -356,7 +356,7 @@ R_DrawSolidClippedSubmodelPolygons(entity_t *currententity, const model_t *curre
 
 	for (i=0 ; i<numsurfaces ; i++, psurf++)
 	{
-		mplane_t *pplane;
+		cplane_t *pplane;
 		bedge_t  *pbedge;
 		medge_t  *pedge;
 		vec_t    dot;
@@ -441,7 +441,7 @@ R_DrawSubmodelPolygons(entity_t *currententity, const model_t *currentmodel, int
 
 	for (i=0 ; i<numsurfaces ; i++, psurf++)
 	{
-		mplane_t *pplane;
+		cplane_t *pplane;
 		// find which side of the node we are on
 		pplane = psurf->plane;
 
@@ -550,7 +550,7 @@ R_RecursiveWorldNode (entity_t *currententity, const model_t *currentmodel, mnod
 	{
 		float dot;
 		int side;
-		mplane_t *plane;
+		cplane_t *plane;
 
 		// node is just a decision point, so go down the apropriate sides
 		// find which side of the node we are on
@@ -587,7 +587,7 @@ R_RecursiveWorldNode (entity_t *currententity, const model_t *currentmodel, mnod
 		{
 			msurface_t *surf;
 
-			surf = r_worldmodel->surfaces + node->firstsurface;
+			surf = currentmodel->surfaces + node->firstsurface;
 
 			if (dot < -BACKFACE_EPSILON)
 			{
@@ -631,7 +631,7 @@ R_RenderWorld
 ================
 */
 void
-R_RenderWorld (void)
+R_RenderWorld (entity_t *currententity)
 {
 	const model_t *currentmodel = r_worldmodel;
 
@@ -643,10 +643,10 @@ R_RenderWorld (void)
 	c_drawnode=0;
 
 	// auto cycle the world frame for texture animation
-	r_worldentity.frame = (int)(r_newrefdef.time*2);
+	currententity->frame = (int)(r_newrefdef.time*2);
 
 	VectorCopy (r_origin, modelorg);
 	r_pcurrentvertbase = currentmodel->vertexes;
 
-	R_RecursiveWorldNode (&r_worldentity, currentmodel, currentmodel->nodes, ALIAS_XY_CLIP_MASK, false);
+	R_RecursiveWorldNode (currententity, currentmodel, currentmodel->nodes, ALIAS_XY_CLIP_MASK, false);
 }
