@@ -78,9 +78,9 @@ void
 RE_Draw_CharScaled(int x, int y, int c, float scale)
 {
 	pixel_t	*dest;
-	byte	*source;
+	byte	*source, *pic_pixels;
 	int	drawline;
-	int	row, col, u, xpos, ypos, iscale;
+	int	row, col, v, iscale, sscale, width, height;
 
 	iscale = (int) scale;
 
@@ -89,54 +89,62 @@ RE_Draw_CharScaled(int x, int y, int c, float scale)
 
 	c &= 255;
 
-	if (c == 32 || c == 32+128)
+	if ((c&127) == 32)
 		return;
 
 	if (y <= -8)
 		return;	// totally off screen
 
-	if ( ( y + 8 ) > vid.height )	// status text was missing in sw...
+	if ( ( y + 8 ) > vid_buffer_height )	// status text was missing in sw...
 		return;
 
 	row = c>>4;
 	col = c&15;
-	source = draw_chars->pixels[0] + (row<<10) + (col<<3);
+
+	width = draw_chars->asset_width * iscale;
+	height = draw_chars->asset_height * iscale;
+	pic_pixels = Get_BestImageSize(draw_chars, &width, &height);
+
+	sscale = width / draw_chars->asset_width;
+	pic_pixels += ((row<<10) * sscale + (col<<3)) * sscale;
 
 	if (y < 0)
 	{	// clipped
 		drawline = 8 + y;
-		source -= 128*y;
+		pic_pixels -= width * y * sscale;
 		y = 0;
 	}
 	else
 		drawline = 8;
 
-	dest = vid_buffer + y * vid.width + x;
+	dest = vid_buffer + y * vid_buffer_width + x;
 
 	// clipped last lines
-	if ((y + iscale * (drawline + 1)) > vid.height)
+	if ((y + iscale * (drawline + 1)) > vid_buffer_height)
 	{
-		drawline = (vid.height - y) / iscale;
+		drawline = (vid_buffer_height - y) / scale;
 	}
 
 	VID_DamageBuffer(x, y);
-	VID_DamageBuffer(x + (iscale << 3), y + (iscale * drawline));
+	VID_DamageBuffer(x + (scale * 8), y + (scale * drawline));
 
-	while (drawline--)
+	drawline = drawline * scale;
+
+	for (v=0 ; v < drawline; v++, dest += vid_buffer_width)
 	{
-		for (ypos=0; ypos < iscale; ypos ++)
+		int f, fstep, u;
+		int sv = v * height / (iscale * draw_chars->asset_height);
+		source = pic_pixels + sv * width;
+		f = 0;
+		fstep = (width << SHIFT16XYZ) / (scale * draw_chars->asset_width);
+		for (u=0 ; u < (8 * iscale) ; u++)
 		{
-			for(u=0; u < 8; u++)
+			if (source[f>>16] != TRANSPARENT_COLOR)
 			{
-				if (source[u] != TRANSPARENT_COLOR)
-					for (xpos=0; xpos < iscale; xpos ++)
-					{
-						dest[u * iscale + xpos] = source[u];
-					}
+				dest[u] = source[f>>16];
 			}
-			dest += vid.width;
+			f += fstep;
 		}
-		source += 128;
 	}
 }
 
@@ -156,8 +164,8 @@ RE_Draw_GetPicSize (int *w, int *h, char *name)
 		*w = *h = -1;
 		return;
 	}
-	*w = gl->width;
-	*h = gl->height;
+	*w = gl->asset_width;
+	*h = gl->asset_height;
 }
 
 /*
@@ -172,12 +180,16 @@ RE_Draw_StretchPicImplementation (int x, int y, int w, int h, const image_t *pic
 	byte	*source;
 	int		height;
 	int		skip;
+	int		pic_height, pic_width;
+	byte	*pic_pixels;
 
 	if ((x < 0) ||
-		(x + w > vid.width) ||
-		(y + h > vid.height))
+		(x + w > vid_buffer_width) ||
+		(y + h > vid_buffer_height))
 	{
-		ri.Sys_Error(ERR_FATAL, "%s: bad coordinates", __func__);
+		R_Printf(PRINT_ALL, "%s: bad coordinates %dx%d[%dx%d]",
+			__func__, x, y, w, h);
+		return;
 	}
 
 	VID_DamageBuffer(x, y);
@@ -193,51 +205,102 @@ RE_Draw_StretchPicImplementation (int x, int y, int w, int h, const image_t *pic
 	else
 		skip = 0;
 
-	dest = vid_buffer + y * vid.width + x;
+	dest = vid_buffer + y * vid_buffer_width + x;
 
-	if (w == pic->width)
+	pic_height = h;
+	pic_width = w;
+	pic_pixels = Get_BestImageSize(pic, &pic_width, &pic_height);
+
+	if (!pic->transparent)
 	{
-		int v;
-
-		for (v=0 ; v<height ; v++, dest += vid.width)
+		if (w == pic_width)
 		{
-			int sv = (skip + v)*pic->height/h;
-			source = pic->pixels[0] + sv*pic->width;
-			memcpy (dest, source, w);
+			int v;
+
+			for (v=0 ; v<height ; v++, dest += vid_buffer_width)
+			{
+				int sv = (skip + v) * pic_height / h;
+				memcpy (dest, pic_pixels + sv*pic_width, w);
+			}
+		}
+		else
+		{
+			int v;
+
+			// size of screen tile to pic pixel
+			int picupscale = h / pic_height;
+
+			for (v=0 ; v<height ; v++, dest += vid_buffer_width)
+			{
+				int f, fstep, u;
+				int sv = (skip + v) * pic_height/h;
+				source = pic_pixels + sv*pic_width;
+				f = 0;
+				fstep = (pic_width << SHIFT16XYZ) / w;
+				for (u=0 ; u<w ; u++)
+				{
+					dest[u] = source[f>>16];
+					f += fstep;
+				}
+				if (picupscale > 1)
+				{
+					int i;
+					int pu = min(height-v, picupscale);
+					pixel_t	*dest_orig = dest;
+
+					// copy first line to fill whole sector
+					for (i=1; i < pu; i++)
+					{
+						// go to next line
+						dest += vid_buffer_width;
+						memcpy (dest, dest_orig, w);
+					}
+					// skip updated lines
+					v += (picupscale - 1);
+				}
+			}
 		}
 	}
 	else
 	{
-		int v;
-		// size of screen tile to pic pixel
-		int picupscale = h / pic->height;
+		source = pic_pixels;
 
-		for (v=0 ; v<height ; v++, dest += vid.width)
+		if (h == pic_height && w == pic_width)
 		{
-			int f, fstep, u;
-			int sv = (skip + v)*pic->height/h;
-			source = pic->pixels[0] + sv*pic->width;
-			f = 0;
-			fstep = (pic->width << SHIFT16XYZ) / w;
-			for (u=0 ; u<w ; u++)
-			{
-				dest[u] = source[f>>16];
-				f += fstep;
-			}
-			if (picupscale > 1)
-			{
-				int i;
-				pixel_t	*dest_orig = dest;
+			int v;
 
-				// copy first line to fill whole sector
-				for (i=1; i < picupscale; i++)
+			for (v = 0; v < pic_height; v++)
+			{
+				int u;
+
+				for (u = 0; u < pic_width; u++)
 				{
-					// go to next line
-					dest += vid.width;
-					memcpy (dest, dest_orig, w);
+					if (source[u] != TRANSPARENT_COLOR)
+						dest[u] = source[u];
+				}
+				dest += vid_buffer_width;
+				source += pic_width;
+			}
+		}
+		else
+		{
+			int v;
+
+			for (v=0 ; v<height ; v++, dest += vid_buffer_width)
+			{
+				int f, fstep, u;
+				int sv = (skip + v) * pic_height/h;
+				source = pic_pixels + sv*pic_width;
+				f = 0;
+				fstep = (pic_width << SHIFT16XYZ) / w;
+				for (u=0 ; u<w ; u++)
+				{
+					if (source[f>>16] != TRANSPARENT_COLOR)
+					{
+						dest[u] = source[f>>16];
+					}
+					f += fstep;
 				}
-				// skip updated lines
-				v += (picupscale - 1);
 			}
 		}
 	}
@@ -271,11 +334,49 @@ void
 RE_Draw_StretchRaw (int x, int y, int w, int h, int cols, int rows, byte *data)
 {
 	image_t	pic;
+	byte	*image_scaled;
 
-	pic.pixels[0] = data;
+	// we have only one image size
+	pic.mip_levels = 1;
+
+	if (r_retexturing->value)
+	{
+		if (cols < (w / 3) || rows < (h / 3))
+		{
+			image_scaled = malloc(cols * rows * 9);
+
+			scale3x(data, image_scaled, cols, rows);
+
+			cols = cols * 3;
+			rows = rows * 3;
+		}
+		else
+		{
+			image_scaled = malloc(cols * rows * 4);
+
+			scale2x(data, image_scaled, cols, rows);
+
+			cols = cols * 2;
+			rows = rows * 2;
+		}
+	}
+	else
+	{
+		image_scaled = data;
+	}
+
+	pic.pixels[0] = image_scaled;
 	pic.width = cols;
 	pic.height = rows;
+	pic.asset_width = cols;
+	pic.asset_height = rows;
+
 	RE_Draw_StretchPicImplementation (x, y, w, h, &pic);
+
+	if (r_retexturing->value)
+	{
+		free(image_scaled);
+	}
 }
 
 /*
@@ -287,12 +388,7 @@ void
 RE_Draw_PicScaled(int x, int y, char *name, float scale)
 {
 	image_t		*pic;
-	pixel_t		*dest;
-	byte 		*source;
-	int			v, u, xpos, ypos, iscale;
-	int			height;
 
-	iscale = (int)scale;
 	pic = RE_Draw_FindPic (name);
 	if (!pic)
 	{
@@ -300,98 +396,10 @@ RE_Draw_PicScaled(int x, int y, char *name, float scale)
 		return;
 	}
 
-	if ((x < 0) ||
-		(x + pic->width * scale > vid.width) ||
-		(y + pic->height * scale > vid.height))
-	{
-		R_Printf(PRINT_ALL, "Draw_Pic: bad coordinates\n");
-		return;
-	}
-
-	height = pic->height;
-	source = pic->pixels[0];
-	if (y < 0)
-	{
-		height += y;
-		source += pic->width*-y;
-		y = 0;
-	}
-
-	dest = vid_buffer + y * vid.width + x;
-
-	VID_DamageBuffer(x, y);
-	VID_DamageBuffer(x + iscale * pic->width, y + iscale * pic->height);
-
-	if (!pic->transparent)
-	{
-		if (iscale == 1)
-		{
-			for (v=0; v<height; v++)
-			{
-				memcpy(dest, source, pic->width);
-				dest += vid.width;
-				source += pic->width;
-			}
-		}
-		else
-		{
-			for (v=0; v<height; v++)
-			{
-				for(ypos=0; ypos < iscale; ypos++)
-				{
-					pixel_t *dest_u = dest;
-					pixel_t *source_u = source;
-					u = pic->width;
-					do
-					{
-						xpos = iscale;
-						do
-						{
-							*dest_u++ = *source_u;
-						} while (--xpos > 0);
-						source_u++;
-					} while (--u > 0);
-					dest += vid.width;
-				}
-				source += pic->width;
-			}
-		}
-	}
-	else
-	{
-		if (iscale == 1)
-		{
-			for (v=0; v<height; v++)
-			{
-				for (u=0; u<pic->width; u++)
-				{
-					if (source[u] != TRANSPARENT_COLOR)
-						dest[u] = source[u];
-				}
-				dest += vid.width;
-				source += pic->width;
-			}
-		}
-		else
-		{
-			for (v=0; v<height; v++)
-			{
-				for(ypos=0; ypos < iscale; ypos++)
-				{
-					for (u=0; u<pic->width; u++)
-					{
-						if (source[u] != TRANSPARENT_COLOR)
-							for(xpos=0; xpos < iscale; xpos++)
-							{
-								dest[u * iscale + xpos] = source[u];
-							}
-					}
-					dest += vid.width;
-				}
-				source += pic->width;
-			}
-		}
-	}
+	RE_Draw_StretchPicImplementation (
+		x, y,
+		scale * pic->asset_width, scale * pic->asset_height,
+		pic);
 }
 
 /*
@@ -421,10 +429,10 @@ RE_Draw_TileClear (int x, int y, int w, int h, char *name)
 		h += y;
 		y = 0;
 	}
-	if (x + w > vid.width)
-		w = vid.width - x;
-	if (y + h > vid.height)
-		h = vid.height - y;
+	if (x + w > vid_buffer_width)
+		w = vid_buffer_width - x;
+	if (y + h > vid_buffer_height)
+		h = vid_buffer_height - y;
 	if (w <= 0 || h <= 0)
 		return;
 
@@ -438,12 +446,12 @@ RE_Draw_TileClear (int x, int y, int w, int h, char *name)
 		return;
 	}
 	x2 = x + w;
-	pdest = vid_buffer + y * vid.width;
-	for (i=0 ; i<h ; i++, pdest += vid.width)
+	pdest = vid_buffer + y * vid_buffer_width;
+	for (i=0 ; i<h ; i++, pdest += vid_buffer_width)
 	{
-		psrc = pic->pixels[0] + pic->width * ((i+y)&63);
+		psrc = pic->pixels[0] + pic->width * ((i+y) % pic->height);
 		for (j=x ; j<x2 ; j++)
-			pdest[j] = psrc[j&63];
+			pdest[j] = psrc[j % pic->width];
 	}
 }
 
@@ -461,10 +469,10 @@ RE_Draw_Fill (int x, int y, int w, int h, int c)
 	pixel_t	*dest;
 	int	v;
 
-	if (x+w > vid.width)
-		w = vid.width - x;
-	if (y+h > vid.height)
-		h = vid.height - y;
+	if (x+w > vid_buffer_width)
+		w = vid_buffer_width - x;
+	if (y+h > vid_buffer_height)
+		h = vid_buffer_height - y;
 
 	if (x < 0)
 	{
@@ -483,8 +491,8 @@ RE_Draw_Fill (int x, int y, int w, int h, int c)
 	VID_DamageBuffer(x, y);
 	VID_DamageBuffer(x + w, y + h);
 
-	dest = vid_buffer + y * vid.width + x;
-	for (v=0 ; v<h ; v++, dest += vid.width)
+	dest = vid_buffer + y * vid_buffer_width + x;
+	for (v=0 ; v<h ; v++, dest += vid_buffer_width)
 		memset(dest, c, w);
 }
 //=============================================================================
@@ -501,17 +509,17 @@ RE_Draw_FadeScreen (void)
 	int x,y;
 
 	VID_DamageBuffer(0, 0);
-	VID_DamageBuffer(vid.width, vid.height);
+	VID_DamageBuffer(vid_buffer_width, vid_buffer_height);
 
-	for (y=0 ; y<vid.height ; y++)
+	for (y=0 ; y<vid_buffer_height ; y++)
 	{
 		int t;
 		pixel_t *pbuf;
 
-		pbuf = vid_buffer + vid.width * y;
+		pbuf = vid_buffer + vid_buffer_width * y;
 		t = (y & 1) << 1;
 
-		for (x=0 ; x<vid.width ; x++)
+		for (x=0 ; x<vid_buffer_width ; x++)
 		{
 			if ((x & 3) != t)
 				pbuf[x] = 0;
