@@ -53,8 +53,6 @@ refimport_t	ri;
 
 static unsigned	d_8to24table[256];
 
-entity_t	r_worldentity;
-
 char		skyname[MAX_QPATH];
 vec3_t		skyaxis;
 
@@ -115,7 +113,7 @@ float		xscaleinv, yscaleinv;
 float		xscaleshrink, yscaleshrink;
 float		aliasxscale, aliasyscale, aliasxcenter, aliasycenter;
 
-mplane_t	screenedge[4];
+cplane_t	screenedge[4];
 
 //
 // refresh flags
@@ -135,6 +133,7 @@ float	se_time1, se_time2, de_time1, de_time2;
 
 cvar_t	*r_lefthand;
 cvar_t	*r_gunfov;
+cvar_t	*r_farsee;
 static cvar_t	*sw_aliasstats;
 cvar_t	*sw_clearcolor;
 cvar_t	*sw_drawflat;
@@ -145,8 +144,10 @@ cvar_t	*sw_surfcacheoverride;
 cvar_t	*sw_waterwarp;
 static cvar_t	*sw_overbrightbits;
 cvar_t	*sw_custom_particles;
+static cvar_t	*sw_anisotropic;
 cvar_t	*sw_texture_filtering;
-cvar_t	*sw_retexturing;
+cvar_t	*r_retexturing;
+cvar_t	*r_scale8bittextures;
 cvar_t	*sw_gunzposition;
 static cvar_t	*sw_partialrefresh;
 
@@ -163,6 +164,7 @@ static cvar_t  *r_customheight;
 
 static cvar_t	*r_speeds;
 cvar_t	*r_lightlevel;	//FIXME HACK
+cvar_t	*r_fixsurfsky;
 
 static cvar_t	*vid_fullscreen;
 static cvar_t	*vid_gamma;
@@ -248,9 +250,9 @@ VID_DamageZBuffer(int u, int v)
 static void
 VID_NoDamageZBuffer(void)
 {
-	vid_zminu = vid.width;
+	vid_zminu = vid_buffer_width;
 	vid_zmaxu = 0;
-	vid_zminv = vid.height;
+	vid_zminv = vid_buffer_height;
 	vid_zmaxv = 0;
 }
 
@@ -282,9 +284,9 @@ static void
 VID_WholeDamageZBuffer(void)
 {
 	vid_zminu = 0;
-	vid_zmaxu = vid.width;
+	vid_zmaxu = vid_buffer_width;
 	vid_zminv = 0;
-	vid_zmaxv = vid.height;
+	vid_zmaxv = vid_buffer_height;
 }
 
 /*
@@ -321,9 +323,9 @@ VID_DamageBuffer(int u, int v)
 static void
 VID_NoDamageBuffer(void)
 {
-	vid_minu = vid.width;
+	vid_minu = vid_buffer_width;
 	vid_maxu = 0;
-	vid_minv = vid.height;
+	vid_minv = vid_buffer_height;
 	vid_maxv = 0;
 }
 
@@ -332,9 +334,9 @@ static void
 VID_WholeDamageBuffer(void)
 {
 	vid_minu = 0;
-	vid_maxu = vid.width;
+	vid_maxu = vid_buffer_width;
 	vid_minv = 0;
-	vid_maxv = vid.height;
+	vid_maxv = vid_buffer_height;
 }
 
 /*
@@ -343,12 +345,12 @@ R_InitTurb
 ================
 */
 static void
-R_InitTurb (void)
+R_InitTurb (int width)
 {
 	int		i;
 
-	memset(blanktable, 0, (vid.width+CYCLE) * sizeof(int));
-	for (i = 0; i < (vid.width+CYCLE); i++)
+	memset(blanktable, 0, (width+CYCLE) * sizeof(int));
+	for (i = 0; i < (width+CYCLE); i++)
 	{
 		sintable[i] = AMP + sin(i*3.14159*2/CYCLE)*AMP;
 		intsintable[i] = AMP2 + sin(i*3.14159*2/CYCLE)*AMP2; // AMP2, not 20
@@ -373,7 +375,9 @@ R_RegisterVariables (void)
 	sw_overbrightbits = ri.Cvar_Get("sw_overbrightbits", "1.0", CVAR_ARCHIVE);
 	sw_custom_particles = ri.Cvar_Get("sw_custom_particles", "0", CVAR_ARCHIVE);
 	sw_texture_filtering = ri.Cvar_Get("sw_texture_filtering", "0", CVAR_ARCHIVE);
-	sw_retexturing = ri.Cvar_Get("sw_retexturing", "0", CVAR_ARCHIVE);
+	sw_anisotropic = ri.Cvar_Get("r_anisotropic", "0", CVAR_ARCHIVE);
+	r_retexturing = ri.Cvar_Get("r_retexturing", "1", CVAR_ARCHIVE);
+	r_scale8bittextures = ri.Cvar_Get("r_scale8bittextures", "0", CVAR_ARCHIVE);
 	sw_gunzposition = ri.Cvar_Get("sw_gunzposition", "8", CVAR_ARCHIVE);
 
 	// On MacOS texture is cleaned up after render and code have to copy a whole
@@ -389,6 +393,7 @@ R_RegisterVariables (void)
 
 	r_lefthand = ri.Cvar_Get( "hand", "0", CVAR_USERINFO | CVAR_ARCHIVE );
 	r_gunfov = ri.Cvar_Get( "r_gunfov", "80", CVAR_ARCHIVE );
+	r_farsee = ri.Cvar_Get("r_farsee", "0", CVAR_LATCH | CVAR_ARCHIVE);
 	r_speeds = ri.Cvar_Get ("r_speeds", "0", 0);
 	r_fullbright = ri.Cvar_Get ("r_fullbright", "0", 0);
 	r_drawentities = ri.Cvar_Get ("r_drawentities", "1", 0);
@@ -401,6 +406,7 @@ R_RegisterVariables (void)
 	r_vsync = ri.Cvar_Get("r_vsync", "1", CVAR_ARCHIVE);
 	r_customwidth = ri.Cvar_Get("r_customwidth", "1024", CVAR_ARCHIVE);
 	r_customheight = ri.Cvar_Get("r_customheight", "768", CVAR_ARCHIVE);
+	r_fixsurfsky = ri.Cvar_Get("r_fixsurfsky", "0", CVAR_ARCHIVE);
 
 	vid_fullscreen = ri.Cvar_Get( "vid_fullscreen", "0", CVAR_ARCHIVE );
 	vid_gamma = ri.Cvar_Get( "vid_gamma", "1.0", CVAR_ARCHIVE );
@@ -425,7 +431,7 @@ R_UnRegister (void)
 }
 
 static void RE_ShutdownContext(void);
-static void SWimp_CreateRender(void);
+static void SWimp_CreateRender(int width, int height);
 static int RE_InitContext(void *win);
 static qboolean RE_SetMode(void);
 
@@ -437,9 +443,6 @@ R_Init
 static qboolean
 RE_Init(void)
 {
-	R_Printf(PRINT_ALL, "Refresh: " REF_VERSION "\n");
-	R_Printf(PRINT_ALL, "Client: " YQ2VERSION "\n\n");
-
 	R_RegisterVariables ();
 	R_InitImages ();
 	Mod_Init ();
@@ -472,6 +475,8 @@ RE_Init(void)
 	// create the window
 	ri.Vid_MenuInit();
 
+	R_Printf(PRINT_ALL, "ref_soft version: "REF_VERSION"\n");
+
 	return true;
 }
 
@@ -544,7 +549,9 @@ R_ReallocateMapBuffers (void)
 			r_outofsurfaces = false;
 		}
 
-		if (r_cnumsurfs < NUMSTACKSURFACES)
+		if ((r_farsee->value > 0) && (r_cnumsurfs < NUMSTACKSURFACES))
+			r_cnumsurfs = NUMSTACKSURFACES * 2;
+		else if (r_cnumsurfs < NUMSTACKSURFACES)
 			r_cnumsurfs = NUMSTACKSURFACES;
 
 		// edge_t->surf limited size to short
@@ -619,7 +626,9 @@ R_ReallocateMapBuffers (void)
 			r_outofedges = false;
 		}
 
-		if (r_numallocatededges < NUMSTACKEDGES)
+		if ((r_farsee->value > 0) && (r_numallocatededges < NUMSTACKEDGES * 2))
+			r_numallocatededges = NUMSTACKEDGES * 2;
+		else if (r_numallocatededges < NUMSTACKEDGES)
 			r_numallocatededges = NUMSTACKEDGES;
 
 		r_edges = malloc (r_numallocatededges * sizeof(edge_t));
@@ -707,8 +716,8 @@ R_ReallocateMapBuffers (void)
 		}
 
 		// used up to 8 * width spans for render, allocate once  before use
-		if (r_numallocatededgebasespans < vid.width * 8)
-			r_numallocatededgebasespans = vid.width * 8;
+		if (r_numallocatededgebasespans < vid_buffer_width * 8)
+			r_numallocatededgebasespans = vid_buffer_width * 8;
 
 		edge_basespans  = malloc(r_numallocatededgebasespans * sizeof(espan_t));
 		if (!edge_basespans)
@@ -735,7 +744,7 @@ cluster
 static void
 R_MarkLeaves (void)
 {
-	byte	*vis;
+	const byte	*vis;
 	mnode_t	*node;
 	int		i;
 	mleaf_t	*leaf;
@@ -848,8 +857,13 @@ R_DrawEntitiesOnList (void)
 				R_AliasDrawModel(currententity, currentmodel);
 				break;
 
-			default:
+			case mod_brush:
 				break;
+
+			default:
+				R_Printf(PRINT_ALL, "%s: Bad modeltype %d\n",
+					__func__, currentmodel->type);
+				return;
 			}
 		}
 	}
@@ -893,8 +907,13 @@ R_DrawEntitiesOnList (void)
 				R_AliasDrawModel(currententity, currentmodel);
 				break;
 
-			default:
+			case mod_brush:
 				break;
+
+			default:
+				R_Printf(PRINT_ALL, "%s: Bad modeltype %d\n",
+					__func__, currentmodel->type);
+				return;
 			}
 		}
 	}
@@ -965,7 +984,7 @@ R_FindTopnode (vec3_t mins, vec3_t maxs)
 
 	while (1)
 	{
-		mplane_t *splitplane;
+		cplane_t *splitplane;
 		int sides;
 
 		if (node->visframe != r_visframecount)
@@ -1078,12 +1097,12 @@ R_DrawBEntitiesOnList (void)
 	{
 		entity_t *currententity = &r_newrefdef.entities[i];
 		const model_t *currentmodel = currententity->model;
+		if ( currententity->flags & RF_BEAM )
+			continue;
 		if (!currentmodel)
 			continue;
 		if (currentmodel->nummodelsurfaces == 0)
 			continue;	// clip brush only
-		if ( currententity->flags & RF_BEAM )
-			continue;
 		if (currentmodel->type != mod_brush)
 			continue;
 		// see if the bounding box lets us trivially reject, also sets
@@ -1144,7 +1163,7 @@ Render the map
 ================
 */
 static void
-R_EdgeDrawing (void)
+R_EdgeDrawing (entity_t *currententity)
 {
 	if ( r_newrefdef.rdflags & RDF_NOWORLDMODEL )
 		return;
@@ -1162,7 +1181,7 @@ R_EdgeDrawing (void)
 
 	// Build the Global Edget Table
 	// Also populate the surface stack and count # surfaces to render (surf_max is the max)
-	R_RenderWorld ();
+	R_RenderWorld (currententity);
 
 	if (r_dspeeds->value)
 	{
@@ -1180,7 +1199,7 @@ R_EdgeDrawing (void)
 
 	// Use the Global Edge Table to maintin the Active Edge Table: Draw the world as scanlines
 	// Write the Z-Buffer (but no read)
-	R_ScanEdges (surface_p);
+	R_ScanEdges (currententity, surface_p);
 }
 
 //=======================================================================
@@ -1281,6 +1300,7 @@ static void
 RE_RenderFrame (refdef_t *fd)
 {
 	r_newrefdef = *fd;
+	entity_t	ent;
 
 	if (!r_worldmodel && !( r_newrefdef.rdflags & RDF_NOWORLDMODEL ) )
 	{
@@ -1294,7 +1314,8 @@ RE_RenderFrame (refdef_t *fd)
 	VectorCopy (fd->viewangles, r_refdef.viewangles);
 
 	// compare current position with old
-	if (!VectorCompareRound(fd->vieworg, lastvieworg) ||
+	if (vid_buffer_width <= 640 ||
+	    !VectorCompareRound(fd->vieworg, lastvieworg) ||
 	    !VectorCompare(fd->viewangles, lastviewangles))
 	{
 		fastmoving = true;
@@ -1320,9 +1341,14 @@ RE_RenderFrame (refdef_t *fd)
 	// For each dlight_t* passed via r_newrefdef.dlights, mark polygons affected by a light.
 	R_PushDlights (r_worldmodel);
 
+	// TODO: rearange code same as in GL*_DrawWorld?
+	/* auto cycle the world frame for texture animation */
+	memset(&ent, 0, sizeof(ent));
+	ent.frame = (int)(r_newrefdef.time * 2);
+
 	// Build the Global Edge Table and render it via the Active Edge Table
 	// Render the map
-	R_EdgeDrawing ();
+	R_EdgeDrawing (&ent);
 
 	if (r_dspeeds->value)
 	{
@@ -1357,10 +1383,10 @@ RE_RenderFrame (refdef_t *fd)
 		dp_time2 = SDL_GetTicks();
 
 	// Perform pixel palette blending ia the pics/colormap.pcx lower part lookup table.
-	R_DrawAlphaSurfaces(&r_worldentity);
+	R_DrawAlphaSurfaces(&ent);
 
 	// Save off light value for server to look at (BIG HACK!)
-	R_SetLightLevel (&r_worldentity);
+	R_SetLightLevel (&ent);
 
 	if (r_dowarp)
 		D_WarpScreen ();
@@ -1392,9 +1418,6 @@ RE_RenderFrame (refdef_t *fd)
 static void
 R_InitGraphics( int width, int height )
 {
-	vid.width  = width;
-	vid.height = height;
-
 	// free z buffer
 	if ( d_pzbuffer )
 	{
@@ -1410,7 +1433,7 @@ R_InitGraphics( int width, int height )
 		sc_base = NULL;
 	}
 
-	d_pzbuffer = malloc(vid.width * vid.height * sizeof(zvalue_t));
+	d_pzbuffer = malloc(width * height * sizeof(zvalue_t));
 
 	R_InitCaches();
 
@@ -1427,8 +1450,10 @@ RE_BeginFrame( float camera_separation )
 {
 	// pallete without changes
 	palette_changed = false;
+	// run without speed optimization
+	fastmoving = false;
 
-	while (r_mode->modified || vid_fullscreen->modified || r_vsync->modified)
+	while (r_vsync->modified)
 	{
 		RE_SetMode();
 	}
@@ -1443,7 +1468,8 @@ RE_BeginFrame( float camera_separation )
 		// we need redraw everything
 		VID_WholeDamageBuffer();
 		// and backbuffer should be zeroed
-		memset(swap_buffers + ((swap_current + 1)&1), 0, vid.height * vid.width * sizeof(pixel_t));
+		memset(swap_buffers + ((swap_current + 1)&1), 0,
+			vid_buffer_height * vid_buffer_width * sizeof(pixel_t));
 
 		vid_gamma->modified = false;
 		sw_overbrightbits->modified = false;
@@ -1463,8 +1489,6 @@ RE_SetMode(void)
 
 	fullscreen = (int)vid_fullscreen->value;
 
-	vid_fullscreen->modified = false;
-	r_mode->modified = false;
 	r_vsync->modified = false;
 
 	/* a bit hackish approach to enable custom resolutions:
@@ -1478,7 +1502,6 @@ RE_SetMode(void)
 	*/
 	if ((err = SWimp_SetMode(&vid.width, &vid.height, r_mode->value, fullscreen)) == rserr_ok)
 	{
-		R_InitGraphics( vid.width, vid.height );
 		if (r_mode->value == -1)
 		{
 			sw_state.prev_mode = 4; /* safe default for custom mode */
@@ -1492,13 +1515,10 @@ RE_SetMode(void)
 	{
 		if (err == rserr_invalid_fullscreen)
 		{
-			R_InitGraphics( vid.width, vid.height );
-
 			ri.Cvar_SetValue("vid_fullscreen", 0);
-			vid_fullscreen->modified = false;
 			R_Printf(PRINT_ALL, "%s() - fullscreen unavailable in this mode\n", __func__);
 
-			if ((SWimp_SetMode(&vid.width, &vid.height, r_mode->value, 0)) == rserr_ok)
+			if (SWimp_SetMode(&vid.width, &vid.height, r_mode->value, 0) == rserr_ok)
 			{
 				return true;
 			}
@@ -1519,7 +1539,7 @@ RE_SetMode(void)
 		}
 
 		/* try setting it back to something safe */
-		if ((SWimp_SetMode(&vid.width, &vid.height, sw_state.prev_mode, 0)) != rserr_ok)
+		if (SWimp_SetMode(&vid.width, &vid.height, sw_state.prev_mode, 0) != rserr_ok)
 		{
 			R_Printf(PRINT_ALL, "%s() - could not revert to safe mode\n", __func__);
 			return false;
@@ -1713,6 +1733,11 @@ RE_SetSky (char *name, float rotate, vec3_t axis)
 	{
 		Com_sprintf (pathname, sizeof(pathname), "env/%s%s.pcx", skyname, suf[r_skysideimage[i]]);
 		r_skytexinfo[i].image = R_FindImage (pathname, it_sky);
+		if (!r_skytexinfo[i].image)
+		{
+			Com_sprintf (pathname, sizeof(pathname), "pics/Skies/%s%s.m8", skyname, suf[r_skysideimage[i]]);
+			r_skytexinfo[i].image = R_FindImage (pathname, it_sky);
+		}
 	}
 }
 
@@ -1788,6 +1813,17 @@ static int RE_PrepareForWindow(void)
 	return flags;
 }
 
+/*
+=====================
+RE_EndWorldRenderpass
+=====================
+*/
+static qboolean
+RE_EndWorldRenderpass( void )
+{
+	return true;
+}
+
 /*
 ===============
 GetRefAPI
@@ -1834,8 +1870,13 @@ GetRefAPI(refimport_t imp)
 
 	refexport.SetPalette = RE_SetPalette;
 	refexport.BeginFrame = RE_BeginFrame;
+	refexport.EndWorldRenderpass = RE_EndWorldRenderpass;
 	refexport.EndFrame = RE_EndFrame;
 
+    // Tell the client that we're unsing the
+	// new renderer restart API.
+    ri.Vid_RequestRestart(RESTART_NO);
+
 	Swap_Init ();
 
 	return refexport;
@@ -1859,6 +1900,9 @@ static SDL_Window	*window = NULL;
 static SDL_Texture	*texture = NULL;
 static SDL_Renderer	*renderer = NULL;
 
+int vid_buffer_height = 0;
+int vid_buffer_width = 0;
+
 static int
 RE_InitContext(void *win)
 {
@@ -1866,7 +1910,7 @@ RE_InitContext(void *win)
 
 	if(win == NULL)
 	{
-		ri.Sys_Error(ERR_FATAL, "RE_InitContext() must not be called with NULL argument!");
+		ri.Sys_Error(ERR_FATAL, "%s() must not be called with NULL argument!", __func__);
 		return false;
 	}
 
@@ -1895,6 +1939,9 @@ RE_InitContext(void *win)
 	   This will show the new, black contents of the window. */
 	SDL_RenderPresent(renderer);
 
+	vid_buffer_height = vid.height;
+	vid_buffer_width = vid.width;
+
 	texture = SDL_CreateTexture(renderer,
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
 				    SDL_PIXELFORMAT_BGRA8888,
@@ -1902,7 +1949,10 @@ RE_InitContext(void *win)
 				    SDL_PIXELFORMAT_ARGB8888,
 #endif
 				    SDL_TEXTUREACCESS_STREAMING,
-				    vid.width, vid.height);
+				    vid_buffer_width, vid_buffer_height);
+
+	R_InitGraphics(vid_buffer_width, vid_buffer_height);
+	SWimp_CreateRender(vid_buffer_width, vid_buffer_height);
 
 	return true;
 }
@@ -2034,7 +2084,7 @@ RE_CopyFrame (Uint32 * pixels, int pitch, int vmin, int vmax)
 	Uint32 *sdl_palette = (Uint32 *)sw_state.currentpalette;
 
 	// no gaps between images rows
-	if (pitch == vid.width)
+	if (pitch == vid_buffer_width)
 	{
 		const Uint32	*max_pixels;
 		Uint32	*pixels_pos;
@@ -2043,29 +2093,32 @@ RE_CopyFrame (Uint32 * pixels, int pitch, int vmin, int vmax)
 		max_pixels = pixels + vmax;
 		buffer_pos = vid_buffer + vmin;
 
-		for (pixels_pos = pixels + vmin; pixels_pos < max_pixels; pixels_pos++)
+		pixels_pos = pixels + vmin;
+
+		while ( pixels_pos < max_pixels)
 		{
 			*pixels_pos = sdl_palette[*buffer_pos];
 			buffer_pos++;
+			pixels_pos++;
 		}
 	}
 	else
 	{
 		int y,x, buffer_pos, ymin, ymax;
 
-		ymin = vmin / vid.width;
-		ymax = vmax / vid.width;
+		ymin = vmin / vid_buffer_width;
+		ymax = vmax / vid_buffer_width;
 
-		buffer_pos = ymin * vid.width;
+		buffer_pos = ymin * vid_buffer_width;
 		pixels += ymin * pitch;
 		for (y=ymin; y < ymax;  y++)
 		{
-			for (x=0; x < vid.width; x ++)
+			for (x=0; x < vid_buffer_width; x ++)
 			{
 				pixels[x] = sdl_palette[vid_buffer[buffer_pos + x]];
 			}
 			pixels += pitch;
-			buffer_pos += vid.width;
+			buffer_pos += vid_buffer_width;
 		}
 	}
 }
@@ -2109,15 +2162,17 @@ RE_CleanFrame(void)
 	int pitch;
 	Uint32 *pixels;
 
-	memset(swap_buffers, 0, vid.height * vid.width * sizeof(pixel_t) * 2);
+	memset(swap_buffers, 0,
+		vid_buffer_height * vid_buffer_width * sizeof(pixel_t) * 2);
 
 	if (SDL_LockTexture(texture, NULL, (void**)&pixels, &pitch))
 	{
 		Com_Printf("Can't lock texture: %s\n", SDL_GetError());
 		return;
 	}
+
 	// only cleanup texture without flush texture to screen
-	memset(pixels, 0, pitch * vid.height);
+	memset(pixels, 0, pitch * vid_buffer_height);
 	SDL_UnlockTexture(texture);
 
 	// All changes flushed
@@ -2143,8 +2198,14 @@ RE_FlushFrame(int vmin, int vmax)
 	{
 		// On MacOS texture is cleaned up after render,
 		// code have to copy a whole screen to the texture
-		RE_CopyFrame (pixels, pitch / sizeof(Uint32), 0, vid.height * vid.width);
+		RE_CopyFrame (pixels, pitch / sizeof(Uint32), 0, vid_buffer_height * vid_buffer_width);
 	}
+
+	if ((sw_anisotropic->value > 0) && !fastmoving)
+	{
+		SmoothColorImage(pixels + vmin, vmax - vmin, sw_anisotropic->value);
+	}
+
 	SDL_UnlockTexture(texture);
 
 	SDL_RenderCopy(renderer, texture, NULL, NULL);
@@ -2178,22 +2239,22 @@ RE_EndFrame (void)
 	{
 		vid_minv = 0;
 	}
-	if (vid_maxu > vid.width)
+	if (vid_maxu > vid_buffer_width)
 	{
-		vid_maxu = vid.width;
+		vid_maxu = vid_buffer_width;
 	}
-	if (vid_maxv > vid.height)
+	if (vid_maxv > vid_buffer_height)
 	{
-		vid_maxv = vid.height;
+		vid_maxv = vid_buffer_height;
 	}
 
-	vmin = vid_minu + vid_minv  * vid.width;
-	vmax = vid_maxu + vid_maxv  * vid.width;
+	vmin = vid_minu + vid_minv  * vid_buffer_width;
+	vmax = vid_maxu + vid_maxv  * vid_buffer_width;
 
 	// fix to correct limit
-	if (vmax > (vid.height * vid.width))
+	if (vmax > (vid_buffer_height * vid_buffer_width))
 	{
-		vmax = vid.height * vid.width;
+		vmax = vid_buffer_height * vid_buffer_width;
 	}
 
 	// if palette changed need to flush whole buffer
@@ -2210,9 +2271,9 @@ RE_EndFrame (void)
 
 		// search difference end
 		vmax = RE_BufferDifferenceEnd(vmin, vmax);
-		if (vmax > (vid.height * vid.width))
+		if (vmax > (vid_buffer_height * vid_buffer_width))
 		{
-			vmax = vid.height * vid.width;
+			vmax = vid_buffer_height * vid_buffer_width;
 		}
 	}
 
@@ -2247,22 +2308,42 @@ SWimp_SetMode(int *pwidth, int *pheight, int mode, int fullscreen )
 
 	R_Printf(PRINT_ALL, " %dx%d (vid_fullscreen %i)\n", *pwidth, *pheight, fullscreen);
 
+	if (fullscreen == 2)
+	{
+		int real_height, real_width;
+
+		if(ri.GLimp_GetDesktopMode(&real_width, &real_height))
+		{
+			if (real_height)
+			{
+				if (real_height != *pheight)
+				{
+					*pwidth = ((*pheight) * real_width) / real_height;
+				}
+				else
+				{
+					*pwidth = real_width;
+				}
+			}
+		}
+
+		R_Printf(PRINT_ALL, "Used corrected %dx%d mode\n", *pwidth, *pheight);
+	}
+
 	if (!ri.GLimp_InitGraphics(fullscreen, pwidth, pheight))
 	{
 		// failed to set a valid mode in windowed mode
 		return rserr_invalid_mode;
 	}
 
-	SWimp_CreateRender();
-
 	return retval;
 }
 
 static void
-SWimp_CreateRender(void)
+SWimp_CreateRender(int width, int height)
 {
 	swap_current = 0;
-	swap_buffers = malloc(vid.height * vid.width * sizeof(pixel_t) * 2);
+	swap_buffers = malloc(height * width * sizeof(pixel_t) * 2);
 	if (!swap_buffers)
 	{
 		ri.Sys_Error(ERR_FATAL, "%s: Can't allocate swapbuffer.", __func__);
@@ -2270,20 +2351,20 @@ SWimp_CreateRender(void)
 		return;
 	}
 	swap_frames[0] = swap_buffers;
-	swap_frames[1] = swap_buffers + vid.height * vid.width * sizeof(pixel_t);
+	swap_frames[1] = swap_buffers + height * width * sizeof(pixel_t);
 	vid_buffer = swap_frames[swap_current&1];
 	// Need to rewrite whole frame
 	VID_WholeDamageBuffer();
 
-	sintable = malloc((vid.width+CYCLE) * sizeof(int));
-	intsintable = malloc((vid.width+CYCLE) * sizeof(int));
-	blanktable = malloc((vid.width+CYCLE) * sizeof(int));
+	sintable = malloc((width+CYCLE) * sizeof(int));
+	intsintable = malloc((width+CYCLE) * sizeof(int));
+	blanktable = malloc((width+CYCLE) * sizeof(int));
 
-	newedges = malloc(vid.width * sizeof(edge_t *));
-	removeedges = malloc(vid.width * sizeof(edge_t *));
+	newedges = malloc(width * sizeof(edge_t *));
+	removeedges = malloc(width * sizeof(edge_t *));
 
-	warp_rowptr = malloc((vid.width+AMP2*2) * sizeof(byte*));
-	warp_column = malloc((vid.width+AMP2*2) * sizeof(int));
+	warp_rowptr = malloc((width+AMP2*2) * sizeof(byte*));
+	warp_column = malloc((width+AMP2*2) * sizeof(int));
 
 	// count of "out of items"
 	r_outofsurfaces = false;
@@ -2309,9 +2390,10 @@ SWimp_CreateRender(void)
 
 	R_ReallocateMapBuffers();
 
-	r_warpbuffer = malloc(vid.height * vid.width * sizeof(pixel_t));
+	r_warpbuffer = malloc(height * width * sizeof(pixel_t));
 
-	if ((vid.width >= 2048) && (sizeof(shift20_t) == 4)) // 2k+ resolution and 32 == shift20_t
+	// 2k+ resolution and 32 == shift20_t
+	if ((width >= 2048) && (sizeof(shift20_t) == 4))
 	{
 		shift_size = 18;
 	}
@@ -2320,9 +2402,9 @@ SWimp_CreateRender(void)
 		shift_size = 20;
 	}
 
-	R_InitTurb ();
+	R_InitTurb (width);
 
-	vid_polygon_spans = malloc(sizeof(espan_t) * (vid.height + 1));
+	vid_polygon_spans = malloc(sizeof(espan_t) * (height + 1));
 
 	memset(sw_state.currentpalette, 0, sizeof(sw_state.currentpalette));
 
@@ -2369,26 +2451,26 @@ static void
 R_ScreenShot_f(void)
 {
 	int x, y;
-	byte *buffer = malloc(vid.width * vid.height * 3);
+	byte *buffer = malloc(vid_buffer_width * vid_buffer_height * 3);
 	const unsigned char *palette = sw_state.currentpalette;
 
 	if (!buffer)
 	{
-		R_Printf(PRINT_ALL, "R_ScreenShot: Couldn't malloc %d bytes\n", vid.width * vid.height * 3);
+		R_Printf(PRINT_ALL, "R_ScreenShot: Couldn't malloc %d bytes\n", vid_buffer_width * vid_buffer_height * 3);
 		return;
 	}
 
-	for (x=0; x < vid.width; x ++)
+	for (x=0; x < vid_buffer_width; x ++)
 	{
-		for (y=0; y < vid.height; y ++) {
-			int buffer_pos = y * vid.width + x;
+		for (y=0; y < vid_buffer_height; y ++) {
+			int buffer_pos = y * vid_buffer_width + x;
 			buffer[buffer_pos * 3 + 0] = palette[vid_buffer[buffer_pos] * 4 + 2]; // red
 			buffer[buffer_pos * 3 + 1] = palette[vid_buffer[buffer_pos] * 4 + 1]; // green
 			buffer[buffer_pos * 3 + 2] = palette[vid_buffer[buffer_pos] * 4 + 0]; // blue
 		}
 	}
 
-	ri.Vid_WriteScreenshot(vid.width, vid.height, 3, buffer);
+	ri.Vid_WriteScreenshot(vid_buffer_width, vid_buffer_height, 3, buffer);
 
 	free(buffer);
 }
