@@ -25,8 +25,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <limits.h>
 #include "header/local.h"
 
-model_t	*loadmodel;
-
 static void Mod_LoadSpriteModel(model_t *mod, void *buffer, int modfilelen);
 static void Mod_LoadBrushModel(model_t *mod, void *buffer, int modfilelen);
 static void Mod_LoadAliasModel(model_t *mod, void *buffer, int modfilelen);
@@ -36,14 +34,38 @@ static byte	mod_novis[MAX_MAP_LEAFS/8];
 #define	MAX_MOD_KNOWN	512
 static model_t	mod_known[MAX_MOD_KNOWN];
 static int	mod_numknown;
-
-// the inline * models from the current map are kept seperate
-static model_t	mod_inline[MAX_MOD_KNOWN];
+static int	mod_max = 0;
 
 int	registration_sequence;
 
 //===============================================================================
 
+static qboolean
+Mod_HasFreeSpace(void)
+{
+	int		i, used;
+	model_t	*mod;
+
+	used = 0;
+
+	for (i=0, mod=mod_known ; i < mod_numknown ; i++, mod++)
+	{
+		if (!mod->name[0])
+			continue;
+		if (mod->registration_sequence == registration_sequence)
+		{
+			used ++;
+		}
+	}
+
+	if (mod_max < used)
+	{
+		mod_max = used;
+	}
+
+	// should same size of free slots as currently used
+	return (mod_numknown + mod_max) < MAX_MOD_KNOWN;
+}
 
 /*
 ================
@@ -53,11 +75,13 @@ Mod_Modellist_f
 void
 Mod_Modellist_f (void)
 {
-	int		i;
+	int		i, total, used;
 	model_t	*mod;
-	int		total;
+	qboolean	freeup;
 
 	total = 0;
+	used = 0;
+
 	R_Printf(PRINT_ALL,"Loaded models:\n");
 	for (i=0, mod=mod_known ; i < mod_numknown ; i++, mod++)
 	{
@@ -66,6 +90,7 @@ Mod_Modellist_f (void)
 		if (mod->registration_sequence == registration_sequence)
 		{
 			in_use = "*";
+			used ++;
 		}
 
 		if (!mod->name[0])
@@ -75,6 +100,9 @@ Mod_Modellist_f (void)
 		total += mod->extradatasize;
 	}
 	R_Printf(PRINT_ALL, "Total resident: %i\n", total);
+	// update statistics
+	freeup = Mod_HasFreeSpace();
+	R_Printf(PRINT_ALL, "Used %d of %d models%s.\n", used, mod_max, freeup ? ", has free space" : "");
 }
 
 /*
@@ -85,6 +113,7 @@ Mod_Init
 void
 Mod_Init (void)
 {
+	mod_max = 0;
 	memset (mod_novis, 0xff, sizeof(mod_novis));
 }
 
@@ -96,7 +125,7 @@ Loads in a model for the given name
 ==================
 */
 static model_t *
-Mod_ForName (char *name, qboolean crash)
+Mod_ForName (char *name, model_t *parent_model, qboolean crash)
 {
 	model_t	*mod;
 	unsigned *buf;
@@ -110,16 +139,16 @@ Mod_ForName (char *name, qboolean crash)
 	//
 	// inline models are grabbed only from worldmodel
 	//
-	if (name[0] == '*')
+	if (name[0] == '*' && parent_model)
 	{
 		i = atoi(name+1);
-		if (i < 1 || !r_worldmodel || i >= r_worldmodel->numsubmodels)
+		if (i < 1 || i >= parent_model->numsubmodels)
 		{
 			ri.Sys_Error(ERR_DROP, "%s: bad inline model number",
 					__func__);
 		}
 
-		return &mod_inline[i];
+		return &parent_model->submodels[i];
 	}
 
 	//
@@ -161,8 +190,6 @@ Mod_ForName (char *name, qboolean crash)
 		return NULL;
 	}
 
-	loadmodel = mod;
-
 	//
 	// fill it in
 	//
@@ -189,7 +216,7 @@ Mod_ForName (char *name, qboolean crash)
 		break;
 	}
 
-	loadmodel->extradatasize = Hunk_End();
+	mod->extradatasize = Hunk_End();
 
 	ri.FS_FreeFile(buf);
 
@@ -203,7 +230,7 @@ Mod_PointInLeaf
 ===============
 */
 mleaf_t *
-Mod_PointInLeaf (vec3_t p, model_t *model)
+Mod_PointInLeaf (vec3_t p, const model_t *model)
 {
 	mnode_t		*node;
 
@@ -217,7 +244,7 @@ Mod_PointInLeaf (vec3_t p, model_t *model)
 	while (node->contents == -1)
 	{
 		float d;
-		mplane_t *plane;
+		cplane_t *plane;
 
 		plane = node->plane;
 		d = DotProduct (p,plane->normal) - plane->dist;
@@ -236,8 +263,8 @@ Mod_PointInLeaf (vec3_t p, model_t *model)
 Mod_ClusterPVS
 ==============
 */
-byte *
-Mod_ClusterPVS (int cluster, model_t *model)
+const byte *
+Mod_ClusterPVS (int cluster, const model_t *model)
 {
 	if (cluster == -1 || !model->vis)
 		return mod_novis;
@@ -254,9 +281,6 @@ Mod_ClusterPVS (int cluster, model_t *model)
 ===============================================================================
 */
 
-byte	*mod_base;
-
-
 /*
 =================
 Mod_LoadLighting
@@ -266,7 +290,7 @@ by taking the brightest component
 =================
 */
 static void
-Mod_LoadLighting (lump_t *l)
+Mod_LoadLighting (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int		i, size;
 	byte	*in;
@@ -295,8 +319,8 @@ static int r_leaftovis[MAX_MAP_LEAFS];
 static int r_vistoleaf[MAX_MAP_LEAFS];
 static int r_numvisleafs;
 
-void
-R_NumberLeafs (mnode_t *node)
+static void
+R_NumberLeafs (model_t *loadmodel, mnode_t *node)
 {
 	if (node->contents != -1)
 	{
@@ -313,8 +337,8 @@ R_NumberLeafs (mnode_t *node)
 		return;
 	}
 
-	R_NumberLeafs (node->children[0]);
-	R_NumberLeafs (node->children[1]);
+	R_NumberLeafs (loadmodel, node->children[0]);
+	R_NumberLeafs (loadmodel, node->children[1]);
 }
 
 
@@ -324,7 +348,7 @@ Mod_LoadVisibility
 =================
 */
 static void
-Mod_LoadVisibility (lump_t *l)
+Mod_LoadVisibility (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int		i;
 
@@ -351,7 +375,7 @@ Mod_LoadVertexes
 =================
 */
 static void
-Mod_LoadVertexes (lump_t *l)
+Mod_LoadVertexes (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	dvertex_t	*in;
 	mvertex_t	*out;
@@ -390,14 +414,13 @@ Mod_LoadSubmodels
 =================
 */
 static void
-Mod_LoadSubmodels (lump_t *l)
+Mod_LoadSubmodels (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	dmodel_t	*in;
-	dmodel_t	*out;
+	model_t	*out;
 	int			i, j, count;
 
 	in = (void *)(mod_base + l->fileofs);
-
 	if (l->filelen % sizeof(*in))
 	{
 		ri.Sys_Error(ERR_DROP, "%s: funny lump size in %s",
@@ -412,15 +435,38 @@ Mod_LoadSubmodels (lump_t *l)
 
 	for ( i=0 ; i<count ; i++, in++, out++)
 	{
+		if (i == 0)
+		{
+			// copy parent as template for first model
+			memcpy(out, loadmodel, sizeof(*out));
+		}
+		else
+		{
+			// copy first as template for model
+			memcpy(out, loadmodel->submodels, sizeof(*out));
+		}
+
+		Com_sprintf (out->name, sizeof(out->name), "*%d", i);
+
 		for (j=0 ; j<3 ; j++)
 		{	// spread the mins / maxs by a pixel
 			out->mins[j] = LittleFloat (in->mins[j]) - 1;
 			out->maxs[j] = LittleFloat (in->maxs[j]) + 1;
 			out->origin[j] = LittleFloat (in->origin[j]);
 		}
-		out->headnode = LittleLong (in->headnode);
-		out->firstface = LittleLong (in->firstface);
-		out->numfaces = LittleLong (in->numfaces);
+
+		out->radius = Mod_RadiusFromBounds (out->mins, out->maxs);
+		out->firstnode = LittleLong (in->headnode);
+		out->firstmodelsurface = LittleLong (in->firstface);
+		out->nummodelsurfaces = LittleLong (in->numfaces);
+		// visleafs
+		out->numleafs = 0;
+		//  check limits
+		if (out->firstnode >= loadmodel->numnodes)
+		{
+			ri.Sys_Error(ERR_DROP, "%s: Inline model %i has bad firstnode",
+					__func__, i);
+		}
 	}
 }
 
@@ -430,7 +476,7 @@ Mod_LoadEdges
 =================
 */
 static void
-Mod_LoadEdges (lump_t *l)
+Mod_LoadEdges (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	dedge_t *in;
 	medge_t *out;
@@ -462,7 +508,7 @@ Mod_LoadTexinfo
 =================
 */
 static void
-Mod_LoadTexinfo (lump_t *l)
+Mod_LoadTexinfo (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	texinfo_t *in;
 	mtexinfo_t *out, *step;
@@ -514,6 +560,12 @@ Mod_LoadTexinfo (lump_t *l)
 
 		Com_sprintf (name, sizeof(name), "textures/%s.wal", in->texture);
 		out->image = R_FindImage (name, it_wall);
+		if (!out->image || out->image == r_notexture_mip)
+		{
+			Com_sprintf (name, sizeof(name), "textures/%s.m8", in->texture);
+			out->image = R_FindImage (name, it_wall);
+		}
+
 		if (!out->image)
 		{
 			out->image = r_notexture_mip; // texture not found
@@ -539,7 +591,7 @@ Fills in s->texturemins[] and s->extents[]
 ================
 */
 static void
-CalcSurfaceExtents (msurface_t *s)
+CalcSurfaceExtents (model_t *loadmodel, msurface_t *s)
 {
 	float	mins[2], maxs[2], val;
 	int		i;
@@ -598,7 +650,7 @@ Mod_LoadFaces
 =================
 */
 static void
-Mod_LoadFaces (lump_t *l)
+Mod_LoadFaces (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	dface_t		*in;
 	msurface_t 	*out;
@@ -640,7 +692,7 @@ Mod_LoadFaces (lump_t *l)
 
 		out->texinfo = loadmodel->texinfo + LittleShort (in->texinfo);
 
-		CalcSurfaceExtents (out);
+		CalcSurfaceExtents (loadmodel, out);
 
 		// lighting info is converted from 24 bit on disk to 8 bit
 		for (i=0 ; i<MAXLIGHTMAPS ; i++)
@@ -655,10 +707,14 @@ Mod_LoadFaces (lump_t *l)
 
 		if (!out->texinfo->image)
 			continue;
-		if (out->texinfo->flags & SURF_SKY)
+
+		if (r_fixsurfsky->value)
 		{
-			out->flags |= SURF_DRAWSKY;
-			continue;
+			if (out->texinfo->flags & SURF_SKY)
+			{
+				out->flags |= SURF_DRAWSKY;
+				continue;
+			}
 		}
 
 		if (out->texinfo->flags & SURF_WARP)
@@ -711,7 +767,7 @@ Mod_LoadNodes
 =================
 */
 static void
-Mod_LoadNodes (lump_t *l)
+Mod_LoadNodes (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int		i, count;
 	dnode_t		*in;
@@ -767,7 +823,7 @@ Mod_LoadLeafs
 =================
 */
 static void
-Mod_LoadLeafs (lump_t *l)
+Mod_LoadLeafs (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	dleaf_t 	*in;
 	mleaf_t 	*out;
@@ -821,7 +877,7 @@ Mod_LoadMarksurfaces
 =================
 */
 static void
-Mod_LoadMarksurfaces (lump_t *l)
+Mod_LoadMarksurfaces (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int		i, count;
 	short		*in;
@@ -859,7 +915,7 @@ Mod_LoadSurfedges
 =================
 */
 static void
-Mod_LoadSurfedges (lump_t *l)
+Mod_LoadSurfedges (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int		i, count;
 	int		*in, *out;
@@ -888,10 +944,10 @@ Mod_LoadPlanes
 =================
 */
 static void
-Mod_LoadPlanes (lump_t *l)
+Mod_LoadPlanes (model_t *loadmodel, byte *mod_base, lump_t *l)
 {
 	int i;
-	mplane_t	*out;
+	cplane_t	*out;
 	dplane_t 	*in;
 	int			count;
 
@@ -959,9 +1015,9 @@ Mod_LoadBrushModel(model_t *mod, void *buffer, int modfilelen)
 {
 	int		i;
 	dheader_t	*header;
-	dmodel_t 	*bm;
+	byte	*mod_base;
 
-	if (loadmodel != mod_known)
+	if (mod != mod_known)
 		ri.Sys_Error(ERR_DROP, "%s: Loaded a brush model after the world", __func__);
 
 	header = (dheader_t *)buffer;
@@ -994,7 +1050,7 @@ Mod_LoadBrushModel(model_t *mod, void *buffer, int modfilelen)
 		hunkSize += size;
 	}
 
-	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_PLANES], sizeof(dplane_t), sizeof(mplane_t), 6);
+	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_PLANES], sizeof(dplane_t), sizeof(cplane_t), 6);
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_TEXINFO], sizeof(texinfo_t), sizeof(mtexinfo_t), 6);
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_FACES], sizeof(dface_t), sizeof(msurface_t), 6);
 	hunkSize += calcLumpHunkSize(&header->lumps[LUMP_LEAFFACES], sizeof(short), sizeof(msurface_t *), 0); // yes, out is indeeed a pointer!
@@ -1005,55 +1061,28 @@ Mod_LoadBrushModel(model_t *mod, void *buffer, int modfilelen)
 
 	hunkSize += 1048576; // 1MB extra just in case
 
-	loadmodel->extradata = Hunk_Begin(hunkSize);
+	mod->extradata = Hunk_Begin(hunkSize);
 
-	loadmodel->type = mod_brush;
+	mod->type = mod_brush;
 
 	// load into heap
-	Mod_LoadVertexes (&header->lumps[LUMP_VERTEXES]);
-	Mod_LoadEdges (&header->lumps[LUMP_EDGES]);
-	Mod_LoadSurfedges (&header->lumps[LUMP_SURFEDGES]);
-	Mod_LoadLighting (&header->lumps[LUMP_LIGHTING]);
-	Mod_LoadPlanes (&header->lumps[LUMP_PLANES]);
-	Mod_LoadTexinfo (&header->lumps[LUMP_TEXINFO]);
-	Mod_LoadFaces (&header->lumps[LUMP_FACES]);
-	Mod_LoadMarksurfaces (&header->lumps[LUMP_LEAFFACES]);
-	Mod_LoadVisibility (&header->lumps[LUMP_VISIBILITY]);
-	Mod_LoadLeafs (&header->lumps[LUMP_LEAFS]);
-	Mod_LoadNodes (&header->lumps[LUMP_NODES]);
-	Mod_LoadSubmodels (&header->lumps[LUMP_MODELS]);
+	Mod_LoadVertexes (mod, mod_base, &header->lumps[LUMP_VERTEXES]);
+	Mod_LoadEdges (mod, mod_base, &header->lumps[LUMP_EDGES]);
+	Mod_LoadSurfedges (mod, mod_base, &header->lumps[LUMP_SURFEDGES]);
+	Mod_LoadLighting (mod, mod_base, &header->lumps[LUMP_LIGHTING]);
+	Mod_LoadPlanes (mod, mod_base, &header->lumps[LUMP_PLANES]);
+	Mod_LoadTexinfo (mod, mod_base, &header->lumps[LUMP_TEXINFO]);
+	Mod_LoadFaces (mod, mod_base, &header->lumps[LUMP_FACES]);
+	Mod_LoadMarksurfaces (mod, mod_base, &header->lumps[LUMP_LEAFFACES]);
+	Mod_LoadVisibility (mod, mod_base, &header->lumps[LUMP_VISIBILITY]);
+	Mod_LoadLeafs (mod, mod_base, &header->lumps[LUMP_LEAFS]);
+	Mod_LoadNodes (mod, mod_base, &header->lumps[LUMP_NODES]);
+	Mod_LoadSubmodels (mod, mod_base, &header->lumps[LUMP_MODELS]);
+
 	r_numvisleafs = 0;
-	R_NumberLeafs (loadmodel->nodes);
+	R_NumberLeafs (mod, mod->nodes);
 
-	//
-	// set up the submodels
-	//
-	for (i=0 ; i<mod->numsubmodels ; i++)
-	{
-		model_t	*starmod;
-
-		bm = &mod->submodels[i];
-		starmod = &mod_inline[i];
-
-		*starmod = *loadmodel;
-
-		starmod->firstmodelsurface = bm->firstface;
-		starmod->nummodelsurfaces = bm->numfaces;
-		starmod->firstnode = bm->headnode;
-		if (starmod->firstnode >= loadmodel->numnodes)
-		{
-			ri.Sys_Error(ERR_DROP, "%s: Inline model %i has bad firstnode",
-					__func__, i);
-		}
-
-		VectorCopy (bm->maxs, starmod->maxs);
-		VectorCopy (bm->mins, starmod->mins);
-
-		if (i == 0)
-			*loadmodel = *starmod;
-	}
-
-	R_InitSkyBox ();
+	R_InitSkyBox (mod);
 }
 
 /*
@@ -1198,8 +1227,17 @@ Mod_LoadAliasModel(model_t *mod, void *buffer, int modfilelen)
 	pincmd = (int *) ((byte *)pinmodel + pheader->ofs_glcmds);
 	poutcmd = (int *) ((byte *)pheader + pheader->ofs_glcmds);
 	for (i=0 ; i<pheader->num_glcmds ; i++)
+	{
 		poutcmd[i] = LittleLong (pincmd[i]);
+	}
 
+	if (poutcmd[pheader->num_glcmds-1] != 0)
+	{
+		R_Printf(PRINT_ALL, "%s: Entity %s has possible last element issues with %d verts.\n",
+			__func__,
+			mod->name,
+			poutcmd[pheader->num_glcmds-1]);
+	}
 
 	// register all skins
 	memcpy ((char *)pheader + pheader->ofs_skins, (char *)pinmodel + pheader->ofs_skins,
@@ -1306,7 +1344,7 @@ RE_RegisterModel (char *name)
 {
 	model_t	*mod;
 
-	mod = Mod_ForName (name, false);
+	mod = Mod_ForName (name, r_worldmodel, false);
 	if (mod)
 	{
 		int i;
@@ -1352,14 +1390,20 @@ RE_EndRegistration (void)
 	int	i;
 	model_t	*mod;
 
+	if (Mod_HasFreeSpace() && R_ImageHasFreeSpace())
+	{
+		// should be enough space for load next maps
+		return;
+	}
+
 	for (i=0, mod=mod_known ; i<mod_numknown ; i++, mod++)
 	{
 		if (!mod->name[0])
 			continue;
 		if (mod->registration_sequence != registration_sequence)
-		{	// don't need this model
-			Hunk_Free (mod->extradata);
-			memset (mod, 0, sizeof(*mod));
+		{
+			// don't need this model
+			Mod_Free (mod);
 		}
 	}
 
