@@ -17,10 +17,10 @@
  *
  * =======================================================================
  *
- * This file implements an interface to libvorbis for decoding
+ * This file implements an interface to stb_vorbis.c for decoding
  * OGG/Vorbis files. Strongly spoken this file isn't part of the sound
  * system but part of the main client. It justs converts Vorbis streams
- * into normal, raw Wave stream which are injected into the backends as 
+ * into normal, raw Wave stream which are injected into the backends as
  * if they were normal "raw" samples. At this moment only background
  * music playback and in theory .cin movie file playback is supported.
  *
@@ -40,7 +40,8 @@
 #define STB_VORBIS_NO_PUSHDATA_API
 #include "header/stb_vorbis.h"
 
-static cvar_t *ogg_shuffle;        /* Shuffle playback */
+static cvar_t *ogg_enabled;       /* Backend is enabled */
+static cvar_t *ogg_shuffle;       /* Shuffle playback */
 static cvar_t *ogg_ignoretrack0;  /* Toggle track 0 playing */
 static cvar_t *ogg_volume;        /* Music volume. */
 static int ogg_curfile;           /* Index of currently played file. */
@@ -50,7 +51,7 @@ static ogg_status_t ogg_status;   /* Status indicator. */
 static stb_vorbis *ogg_file;      /* Ogg Vorbis file. */
 static qboolean ogg_started;      /* Initialization flag. */
 
-enum { MAX_NUM_OGGTRACKS = 32 };
+enum { MAX_NUM_OGGTRACKS = 128 };
 static char* ogg_tracks[MAX_NUM_OGGTRACKS];
 static int ogg_maxfileindex;
 
@@ -200,9 +201,8 @@ OGG_InitTrackList(void)
 			}
 
 			// the GOG case: music/Track02.ogg to Track21.ogg
-			int gogTrack = getMappedGOGtrack(8, gameType);
-
-			snprintf(testFileName, MAX_OSPATH, "%sTrack%02i.ogg", fullMusicPath, gogTrack);
+			snprintf(testFileName, MAX_OSPATH, "%sTrack%02i.ogg",
+				fullMusicPath, getMappedGOGtrack(8, gameType));
 
 			if(Sys_IsFile(testFileName))
 			{
@@ -239,13 +239,13 @@ static OGG_Read(void)
 	short samples[4096] = {0};
 
 	int read_samples = stb_vorbis_get_samples_short_interleaved(ogg_file, ogg_file->channels, samples,
-		sizeof(samples) / ogg_file->channels);
+		sizeof(samples) / sizeof(short));
 
 	if (read_samples > 0)
 	{
 		ogg_numsamples += read_samples;
 
-		S_RawSamples(read_samples, ogg_file->sample_rate, ogg_file->channels, ogg_file->channels,
+		S_RawSamples(read_samples, ogg_file->sample_rate, sizeof(short), ogg_file->channels,
 			(byte *)samples, ogg_volume->value);
 	}
 	else
@@ -275,6 +275,15 @@ OGG_Stream(void)
 		return;
 	}
 
+	/* OGG playback was disabled since the last package frame.
+	   Shutdown the backend to stop all playing tracks and to
+	   prevend OGG_PLayTrack() from starting new ones. */
+	if (ogg_enabled->value != 1)
+	{
+		OGG_Shutdown();
+		return;
+	}
+
 	if (ogg_status == PLAY)
 	{
 #ifdef USE_OPENAL
@@ -329,6 +338,11 @@ OGG_PlayTrack(int trackNo)
 		return; // sound is not initialized
 	}
 
+	if (ogg_started == false)
+	{
+		return;
+	}
+
 	// Track 0 means "stop music".
 	if(trackNo == 0)
 	{
@@ -373,13 +387,13 @@ OGG_PlayTrack(int trackNo)
 
 	if ((trackNo < 2) || (trackNo > ogg_maxfileindex))
 	{
-		Com_Printf("OGG_PlayTrack: %d out of range.\n", trackNo);
+		Com_Printf("%s: %d out of range.\n", __func__, trackNo);
 		return;
 	}
 
 	if(ogg_tracks[trackNo] == NULL)
 	{
-		Com_Printf("OGG_PlayTrack: Don't have a .ogg file for track %d\n", trackNo);
+		Com_Printf("%s: Don't have a .ogg file for track %d\n", __func__, trackNo);
 	}
 
 	/* Check running music. */
@@ -407,19 +421,20 @@ OGG_PlayTrack(int trackNo)
 
 	if (f == NULL)
 	{
-		Com_Printf("OGG_PlayTrack: could not open file %s for track %d: %s.\n", ogg_tracks[trackNo], trackNo, strerror(errno));
+		Com_Printf("%s: could not open file %s for track %d: %s.\n", __func__, ogg_tracks[trackNo], trackNo, strerror(errno));
 		ogg_tracks[trackNo] = NULL;
 
 		return;
 	}
 
 	int res = 0;
+
+	// fclose is not required on error with close_on_free=true
 	ogg_file = stb_vorbis_open_file(f, true, &res, NULL);
 
 	if (res != 0)
 	{
-		Com_Printf("OGG_PlayTrack: '%s' is not a valid Ogg Vorbis file (error %i).\n", ogg_tracks[trackNo], res);
-		fclose(f);
+		Com_Printf("%s: '%s' is not a valid Ogg Vorbis file (error %i).\n", __func__, ogg_tracks[trackNo], res);
 
 		return;
 	}
@@ -647,8 +662,7 @@ OGG_Init(void)
 	ogg_shuffle = Cvar_Get("ogg_shuffle", "0", CVAR_ARCHIVE);
 	ogg_ignoretrack0 = Cvar_Get("ogg_ignoretrack0", "0", CVAR_ARCHIVE);
 	ogg_volume = Cvar_Get("ogg_volume", "0.7", CVAR_ARCHIVE);
-
-	cvar_t *ogg_enabled = Cvar_Get("ogg_enable", "1", CVAR_ARCHIVE);
+	ogg_enabled = Cvar_Get("ogg_enable", "1", CVAR_ARCHIVE);
 
 	if (ogg_enabled->value != 1)
 	{
@@ -680,7 +694,7 @@ OGG_Shutdown(void)
 	// Music must be stopped.
 	OGG_Stop();
 
-	// Free file lsit.
+	// Free file list.
 	for(int i=0; i<MAX_NUM_OGGTRACKS; ++i)
 	{
 		if(ogg_tracks[i] != NULL)
@@ -696,3 +710,64 @@ OGG_Shutdown(void)
 
 	ogg_started = false;
 }
+
+void
+OGG_LoadAsWav(char *filename, wavinfo_t *info, void **buffer)
+{
+	void * temp_buffer = NULL;
+	int size = FS_LoadFile(filename, &temp_buffer);
+	short *final_buffer = NULL;
+	stb_vorbis * ogg_file = NULL;
+	int res = 0;
+
+	if (!temp_buffer)
+	{
+		/* no such file */
+		return;
+	}
+
+	/* load vorbis file from memory */
+	ogg_file = stb_vorbis_open_memory(temp_buffer, size, &res, NULL);
+	if (!res && ogg_file->channels > 0)
+	{
+		int read_samples = 0;
+
+		/* fill in wav structure */
+		info->rate = ogg_file->sample_rate;
+		info->width = 2;
+		info->channels = ogg_file->channels;
+		info->loopstart = -1;
+		/* return length * channels */
+		info->samples = stb_vorbis_stream_length_in_samples(ogg_file) / ogg_file->channels;
+		info->dataofs = 0;
+
+		/* alloc memory for uncompressed wav */
+		final_buffer = Z_Malloc(info->samples * sizeof(short) * ogg_file->channels);
+
+		/* load sampleas to buffer */
+		read_samples = stb_vorbis_get_samples_short_interleaved(
+			ogg_file, info->channels, final_buffer,
+			info->samples * ogg_file->channels);
+
+		if (read_samples > 0)
+		{
+			/* fix sample list size*/
+			if (read_samples < info->samples)
+			{
+				info->samples = read_samples;
+			}
+
+			/* copy to final result */
+			*buffer = final_buffer;
+		}
+		else
+		{
+			/* something is going wrong */
+			Z_Free(final_buffer);
+			final_buffer = NULL;
+		}
+
+		stb_vorbis_close(ogg_file);
+	}
+	FS_FreeFile(temp_buffer);
+}
