@@ -46,8 +46,8 @@
 #define SDL_LOOPATTENUATE 0.003
 
 /* Globals */
-cvar_t *s_sdldriver;
-int *snd_p;
+static cvar_t *s_sdldriver;
+static int *snd_p;
 static sound_t *backend;
 static portable_samplepair_t paintbuffer[SDL_PAINTBUFFER_SIZE];
 static int beginofs;
@@ -73,7 +73,8 @@ static const float lpf_default_gain_hf = 0.25F;
 static LpfContext lpf_context;
 static qboolean lpf_is_enabled;
 
-static void lpf_initialize(LpfContext* lpf_context, float gain_hf, int target_frequency)
+static void
+lpf_initialize(LpfContext* lpf_context, float gain_hf, int target_frequency)
 {
 	assert(target_frequency > 0);
 	assert(lpf_context);
@@ -110,7 +111,8 @@ static void lpf_initialize(LpfContext* lpf_context, float gain_hf, int target_fr
 	lpf_context->is_history_initialized = false;
 }
 
-static void lpf_update_samples(LpfContext* lpf_context,int sample_count, portable_samplepair_t* samples)
+static void
+lpf_update_samples(LpfContext* lpf_context,int sample_count, portable_samplepair_t* samples)
 {
 	assert(lpf_context);
 	assert(sample_count >= 0);
@@ -171,20 +173,11 @@ static void lpf_update_samples(LpfContext* lpf_context,int sample_count, portabl
  * the SDL output buffer and places it
  * at the appropriate position.
  */
-void
+static void
 SDL_TransferPaintBuffer(int endtime)
 {
-	int i;
-	int lpos;
-	int ls_paintedtime;
-	int out_idx;
-	int count;
 	int out_mask;
-	int *p;
-	int snd_linear_count;
-	int step;
 	int val;
-	short *snd_out;
 	unsigned char *pbuf;
 
 	pbuf = sound.buffer;
@@ -206,11 +199,18 @@ SDL_TransferPaintBuffer(int endtime)
 
 	if ((sound.samplebits == 16) && (sound.channels == 2))
 	{
+		int ls_paintedtime;
+
 		snd_p = (int *)paintbuffer;
 		ls_paintedtime = paintedtime;
 
 		while (ls_paintedtime < endtime)
 		{
+			int i;
+			short *snd_out;
+			int snd_linear_count;
+			int lpos;
+
 			lpos = ls_paintedtime & ((sound.samples >> 1) - 1);
 
 			snd_out = (short *)pbuf + (lpos << 1);
@@ -263,6 +263,11 @@ SDL_TransferPaintBuffer(int endtime)
 	}
 	else
 	{
+		int count;
+		int step;
+		int *p;
+		int out_idx;
+
 		p = (int *)paintbuffer;
 		count = (endtime - paintedtime) * sound.channels;
 		out_mask = sound.samples - 1;
@@ -322,10 +327,9 @@ SDL_TransferPaintBuffer(int endtime)
 /*
  * Mixes an 8 bit sample into a channel.
  */
-void
+static void
 SDL_PaintChannelFrom8(channel_t *ch, sfxcache_t *sc, int count, int offset)
 {
-	int data;
 	int *lscale, *rscale;
 	unsigned char *sfx;
 	int i;
@@ -349,6 +353,8 @@ SDL_PaintChannelFrom8(channel_t *ch, sfxcache_t *sc, int count, int offset)
 
 	for (i = 0; i < count; i++, samp++)
 	{
+		int data;
+
 		data = sfx[i];
 		samp->left += lscale[data];
 		samp->right += rscale[data];
@@ -360,11 +366,9 @@ SDL_PaintChannelFrom8(channel_t *ch, sfxcache_t *sc, int count, int offset)
 /*
  * Mixes an 16 bit sample into a channel
  */
-void
+static void
 SDL_PaintChannelFrom16(channel_t *ch, sfxcache_t *sc, int count, int offset)
 {
-	int data;
-	int left, right;
 	int leftvol, rightvol;
 	signed short *sfx;
 	int i;
@@ -378,6 +382,9 @@ SDL_PaintChannelFrom16(channel_t *ch, sfxcache_t *sc, int count, int offset)
 
 	for (i = 0; i < count; i++, samp++)
 	{
+		int data;
+		int left, right;
+
 		data = sfx[i];
 		left = (data * leftvol) >> 8;
 		right = (data * rightvol) >> 8;
@@ -392,11 +399,10 @@ SDL_PaintChannelFrom16(channel_t *ch, sfxcache_t *sc, int count, int offset)
  * Mixes all pending sounds into
  * the available output channels.
  */
-void
+static void
 SDL_PaintChannels(int endtime)
 {
 	int i;
-	int end;
 	channel_t *ch;
 	sfxcache_t *sc;
 	int ltime, count;
@@ -406,6 +412,8 @@ SDL_PaintChannels(int endtime)
 
 	while (paintedtime < endtime)
 	{
+		int end;
+
 		/* if paintbuffer is smaller than SDL buffer */
 		end = endtime;
 
@@ -512,24 +520,29 @@ SDL_PaintChannels(int endtime)
 			}
 		}
 
-        if (lpf_is_enabled && snd_is_underwater)
-            lpf_update_samples(&lpf_context, end - paintedtime, paintbuffer);
-        else
-            lpf_context.is_history_initialized = false;
+		if (lpf_is_enabled && snd_is_underwater)
+		{
+			lpf_update_samples(&lpf_context, end - paintedtime, paintbuffer);
+		}
+		else
+		{
+			lpf_context.is_history_initialized = false;
+		}
 
 		if (s_rawend >= paintedtime)
 		{
 			/* add from the streaming sound source */
-			int s;
 			int stop;
 
 			stop = (end < s_rawend) ? end : s_rawend;
 
 			for (i = paintedtime; i < stop; i++)
 			{
+				int s;
+
 				s = i & (MAX_RAW_SAMPLES - 1);
 				paintbuffer[i - paintedtime].left += s_rawsamples[s].left;
-                paintbuffer[i - paintedtime].right += s_rawsamples[s].right;
+				paintbuffer[i - paintedtime].right += s_rawsamples[s].right;
 			}
 		}
 
@@ -571,7 +584,7 @@ SDL_DriftBeginofs(float timeofs)
 /*
  * Spatialize a sound effect based on it's origin.
  */
-void
+static void
 SDL_SpatializeOrigin(vec3_t origin, float master_vol, float dist_mult, int *left_vol, int *right_vol)
 {
 	vec_t dot;
@@ -790,8 +803,6 @@ void
 SDL_ClearBuffer(void)
 {
 	int clear;
-	int i;
-	unsigned char *ptr = sound.buffer;
 
 	if (sound_started == SS_NOT)
 	{
@@ -813,6 +824,9 @@ SDL_ClearBuffer(void)
 
 	if (sound.buffer)
 	{
+		int i;
+		unsigned char *ptr = sound.buffer;
+
 		i = sound.samples * sound.samplebits / 8;
 
 		while (i--)
@@ -829,7 +843,7 @@ SDL_ClearBuffer(void)
  * Calculates the absolute timecode
  * of current playback.
  */
-void
+static void
 SDL_UpdateSoundtime(void)
 {
 	static int buffers;
@@ -861,11 +875,10 @@ SDL_UpdateSoundtime(void)
  * Updates the volume scale table
  * based on current volume setting.
  */
-void
+static void
 SDL_UpdateScaletable(void)
 {
-	int i, j;
-	int scale;
+	int i;
 
 	if (s_volume->value > 2.0f)
 	{
@@ -880,6 +893,8 @@ SDL_UpdateScaletable(void)
 
 	for (i = 0; i < 32; i++)
 	{
+		int j, scale;
+
 		scale = (int)(i * 8 * 256 * s_volume->value);
 
 		for (j = 0; j < 256; j++)
@@ -895,18 +910,19 @@ SDL_UpdateScaletable(void)
  * performed.
  */
 qboolean
-SDL_Cache(sfx_t *sfx, wavinfo_t *info, byte *data)
+SDL_Cache(sfx_t *sfx, wavinfo_t *info, byte *data, short volume,
+		  int begin_length, int  end_length,
+		  int attack_length, int fade_length)
 {
 	float stepscale;
 	int i;
 	int len;
 	int sample;
-	int srcsample;
 	sfxcache_t *sc;
 	unsigned int samplefrac = 0;
 
 	stepscale = (float)info->rate / sound.speed;
-    len = (int)(info->samples / stepscale);
+	len = (int)(info->samples / stepscale);
 
 	if ((info->samples == 0) || (len == 0))
 	{
@@ -923,13 +939,19 @@ SDL_Cache(sfx_t *sfx, wavinfo_t *info, byte *data)
 	}
 
 	sc->loopstart = info->loopstart;
-    sc->stereo = 0;
+	sc->stereo = info->channels - 1;
 	sc->length = (int)(info->samples / stepscale);
 	sc->speed = sound.speed;
+	sc->volume = volume;
+	sc->begin = begin_length * 1000 / info->rate;
+	sc->end = end_length * 1000 / info->rate;
+	sc->fade = fade_length * 1000 / info->rate;
+	sc->attack = attack_length * 1000 / info->rate;
 
 	if ((int)(info->samples / stepscale) == 0)
 	{
-		Com_Printf("ResampleSfx: Invalid sound file '%s' (zero length)\n", sfx->name);
+		Com_Printf("%s: Invalid sound file '%s' (zero length)\n",
+			__func__, sfx->name);
 		Z_Free(sfx->cache);
 		sfx->cache = NULL;
 		return false;
@@ -952,6 +974,8 @@ SDL_Cache(sfx_t *sfx, wavinfo_t *info, byte *data)
 	/* resample / decimate to the current source rate */
 	for (i = 0; i < (int)(info->samples / stepscale); i++)
 	{
+		int srcsample;
+
 		srcsample = samplefrac >> 8;
 		samplefrac += (int)(stepscale * 256);
 
@@ -1083,20 +1107,20 @@ SDL_Update(void)
 	channel_t *ch;
 	int i;
 	int samps;
-	int total;
 	unsigned int endtime;
 
-    if (s_underwater->modified) {
-        s_underwater->modified = false;
-        lpf_is_enabled = ((int)s_underwater->value != 0);
-    }
+	if (s_underwater->modified) {
+		s_underwater->modified = false;
+		lpf_is_enabled = ((int)s_underwater->value != 0);
+	}
 
-    if (s_underwater_gain_hf->modified) {
-        s_underwater_gain_hf->modified = false;
+	if (s_underwater_gain_hf->modified) {
+		s_underwater_gain_hf->modified = false;
 
-        lpf_initialize(
-            &lpf_context, s_underwater_gain_hf->value, backend->speed);
-    }
+		lpf_initialize(
+			&lpf_context, s_underwater_gain_hf->value,
+			backend->speed);
+	}
 
 	/* if the loading plaque is up, clear everything
 	   out to make sure we aren't looping a dirty
@@ -1149,6 +1173,8 @@ SDL_Update(void)
 	/* debugging output */
 	if (s_show->value)
 	{
+		int total;
+
 		total = 0;
 		ch = channels;
 
@@ -1173,7 +1199,7 @@ SDL_Update(void)
 		return;
 	}
 
-    /* Mix the samples */
+	/* Mix the samples */
 	SDL_LockAudio();
 
 	/* Updates SDL time */
@@ -1187,7 +1213,7 @@ SDL_Update(void)
 	/* check to make sure that we haven't overshot */
 	if (paintedtime < soundtime)
 	{
-		Com_DPrintf("S_Update_ : overflow\n");
+		Com_DPrintf("%s: overflow\n", __func__);
 		paintedtime = soundtime;
 	}
 
@@ -1274,7 +1300,7 @@ SDL_Callback(void *data, Uint8 *stream, int length)
 		playpos = (length2 / (backend->samplebits / 8));
 	}
 
-    if (playpos >= samplesize)
+	if (playpos >= samplesize)
 	{
 		playpos = 0;
 	}
@@ -1416,9 +1442,9 @@ SDL_BackendInit(void)
 	backend->buffer = calloc(1, samplesize);
 	s_numchannels = MAX_CHANNELS;
 
-    s_underwater->modified = true;
-    s_underwater_gain_hf->modified = true;
-    lpf_initialize(&lpf_context, lpf_default_gain_hf, backend->speed);
+	s_underwater->modified = true;
+	s_underwater_gain_hf->modified = true;
+	lpf_initialize(&lpf_context, lpf_default_gain_hf, backend->speed);
 
 	SDL_UpdateScaletable();
 	SDL_PauseAudio(0);
@@ -1438,13 +1464,13 @@ void
 SDL_BackendShutdown(void)
 {
 	Com_Printf("Closing SDL audio device...\n");
-    SDL_PauseAudio(1);
-    SDL_CloseAudio();
-    SDL_QuitSubSystem(SDL_INIT_AUDIO);
-    free(backend->buffer);
-    backend->buffer = NULL;
-    playpos = samplesize = 0;
-    snd_inited = 0;
-    Com_Printf("SDL audio device shut down.\n");
+	SDL_PauseAudio(1);
+	SDL_CloseAudio();
+	SDL_QuitSubSystem(SDL_INIT_AUDIO);
+	free(backend->buffer);
+	backend->buffer = NULL;
+	playpos = samplesize = 0;
+	snd_inited = 0;
+	Com_Printf("SDL audio device shut down.\n");
 }
 
