@@ -58,8 +58,8 @@ vec3_t listener_forward;
 vec3_t listener_right;
 vec3_t listener_up;
 
-playsound_t s_playsounds[MAX_PLAYSOUNDS];
-playsound_t s_freeplays;
+static playsound_t s_playsounds[MAX_PLAYSOUNDS];
+static playsound_t s_freeplays;
 playsound_t s_pendingplays;
 
 cvar_t *s_volume;
@@ -75,14 +75,14 @@ cvar_t* s_doppler;
 cvar_t* s_ps_sorting;
 
 channel_t channels[MAX_CHANNELS];
-int num_sfx;
+static int num_sfx;
+static int sound_max;
 int paintedtime;
 int s_numchannels;
 int s_rawend;
-int s_registration_sequence;
+static int s_registration_sequence = 0;
 portable_samplepair_t s_rawsamples[MAX_RAW_SAMPLES];
-qboolean snd_initialized = false;
-sfx_t known_sfx[MAX_SFX];
+static sfx_t known_sfx[MAX_SFX];
 sndstarted_t sound_started = SS_NOT;
 sound_t sound;
 static qboolean s_registering;
@@ -206,6 +206,235 @@ S_IsSilencedMuzzleFlash(const wavinfo_t* info, const void* raw_data, const char*
 	return true;
 }
 
+static void
+S_LoadVorbis(const char *path, const char* name, wavinfo_t *info, void **buffer)
+{
+	int	len;
+	char namewe[256];
+	char filename[MAX_QPATH];
+	const char* ext;
+
+	if (!path)
+	{
+		return;
+	}
+
+	ext = COM_FileExtension(path);
+	if(!ext[0])
+	{
+		/* file has no extension */
+		return;
+	}
+
+	len = strlen(path);
+
+	if (len < 5)
+	{
+		return;
+	}
+
+	/* Remove the extension */
+	memset(namewe, 0, sizeof(namewe));
+	memcpy(namewe, path, len - (strlen(ext) + 1));
+
+	/* Combine with ogg */
+	Q_strlcpy(filename, namewe, sizeof(filename));
+
+	/* Add the extension */
+	Q_strlcat(filename, ".ogg", sizeof(filename));
+
+	OGG_LoadAsWav(filename, info, buffer);
+}
+
+static void
+S_GetVolume(const byte *data, int sound_length, int width, double *sound_volume)
+{
+	/* update sound volume */
+	*sound_volume = 0;
+	if (width == 2)
+	{
+		short *sound_data = (short *)data;
+		short *sound_end = sound_data + sound_length;
+		while (sound_data < sound_end)
+		{
+			short sound_sample = LittleShort(*sound_data);
+			*sound_volume += (sound_sample * sound_sample);
+			sound_data ++;
+		}
+	}
+	else if (width == 1)
+	{
+		byte *sound_data = (byte *)data;
+		byte *sound_end = sound_data + sound_length;
+		while (sound_data < sound_end)
+		{
+			// normilize to 16bit sound;
+			short sound_sample = *sound_data << 8;
+			*sound_volume += (sound_sample * sound_sample);
+			sound_data ++;
+		}
+	}
+	if (sound_length != 0)
+	{
+		*sound_volume /= sound_length;
+		*sound_volume = sqrtf(*sound_volume);
+	}
+}
+
+static void
+S_GetStatistics(const byte *data, int sound_length, int width, int channels,
+	double sound_volume, int *begin_length, int *end_length,
+	int *attack_length, int *fade_length)
+{
+	/* attack length */
+	short sound_max = 0;
+	/* calculate max value*/
+	if (width == 2)
+	{
+		short *sound_data = (short *)data;
+		short *sound_end = sound_data + sound_length;
+		while (sound_data < sound_end)
+		{
+			short sound_sample = LittleShort(*sound_data);
+			if (sound_max < abs(sound_sample))
+			{
+				sound_max = abs(sound_sample);
+			}
+			sound_data ++;
+		}
+	}
+	else if (width == 1)
+	{
+		byte *sound_data = (byte *)data;
+		byte *sound_end = sound_data + sound_length;
+		while (sound_data < sound_end)
+		{
+			// normilize to 16bit sound;
+			short sound_sample = *sound_data << 8;
+			if (sound_max < abs(sound_sample))
+			{
+				sound_max = abs(sound_sample);
+			}
+			sound_data ++;
+		}
+	}
+
+	// use something in middle
+	sound_max = (sound_max + sound_volume) / 2;
+
+	// calculate attack/fade length
+	if (width == 2)
+	{
+		// calculate attack/fade length
+		short *sound_data = (short *)data;
+		short *delay_data = sound_data;
+		short *fade_data = sound_data;
+		short *sound_end = sound_data + sound_length;
+		short sound_sample = 0;
+		short sound_treshold = sound_max / 2;
+
+		/* delay calculate */
+		do
+		{
+			sound_sample = LittleShort(*sound_data);
+			sound_data ++;
+		}
+		while (sound_data < sound_end && abs(sound_sample) < sound_treshold);
+		/* delay_data == (short *)(data + info.dataofs) */
+		*begin_length = (sound_data - delay_data) / channels;
+		delay_data = sound_data;
+		fade_data = sound_data;
+
+		/* attack calculate */
+		do
+		{
+			sound_sample = LittleShort(*sound_data);
+			sound_data ++;
+		}
+		while (sound_data < sound_end && abs(sound_sample) < sound_max);
+		/* fade_data == delay_data */
+		*attack_length = (sound_data - delay_data) / channels;
+		fade_data = sound_data;
+
+		/* end calculate */
+		sound_data = sound_end;
+		do
+		{
+			sound_data --;
+			sound_sample = LittleShort(*sound_data);
+		}
+		while (sound_data > fade_data && abs(sound_sample) < sound_treshold);
+		*end_length = (sound_end -  sound_data) / channels;
+		sound_end = sound_data;
+
+		/* fade calculate */
+		do
+		{
+			sound_data --;
+			sound_sample = LittleShort(*sound_data);
+		}
+		while (sound_data > fade_data && abs(sound_sample) < sound_max);
+		*fade_length = (sound_end - sound_data) / channels;
+	}
+	else if (width == 1)
+	{
+		// calculate attack/fade length
+		byte *sound_data = (byte *)data;
+		byte *delay_data = sound_data;
+		byte *fade_data = sound_data;
+		byte *sound_end = sound_data + sound_length;
+		short sound_sample = 0;
+		short sound_treshold = sound_max / 2;
+
+		/* delay calculate */
+		do
+		{
+			// normilize to 16bit sound;
+			sound_sample = *sound_data << 8;
+			sound_data ++;
+		}
+		while (sound_data < sound_end && abs(sound_sample) < sound_treshold);
+		/* delay_data == (short *)(data + info.dataofs) */
+		*begin_length = (sound_data - delay_data) / channels;
+		delay_data = sound_data;
+		fade_data = sound_data;
+
+		/* attack calculate */
+		do
+		{
+			// normilize to 16bit sound;
+			sound_sample = *sound_data << 8;
+			sound_data ++;
+		}
+		while (sound_data < sound_end && abs(sound_sample) < sound_max);
+		/* fade_data == delay_data */
+		*attack_length = (sound_data - delay_data) / channels;
+		fade_data = sound_data;
+
+		/* end calculate */
+		sound_data = sound_end;
+		do
+		{
+			sound_data --;
+			// normilize to 16bit sound;
+			sound_sample = *sound_data << 8;
+		}
+		while (sound_data > fade_data && abs(sound_sample) < sound_treshold);
+		*end_length = (sound_end -  sound_data) / channels;
+		sound_end = sound_data;
+
+		/* fade calculate */
+		do
+		{
+			sound_data --;
+			// normilize to 16bit sound;
+			sound_sample = *sound_data << 8;
+		}
+		while (sound_data > fade_data && abs(sound_sample) < sound_max);
+		*fade_length = (sound_end - sound_data) / channels;
+	}
+}
+
 /*
  * Loads one sample into memory
  */
@@ -213,10 +442,14 @@ sfxcache_t *
 S_LoadSound(sfx_t *s)
 {
 	char namebuffer[MAX_QPATH];
-	byte *data;
+	byte *data = NULL;
 	wavinfo_t info;
 	sfxcache_t *sc;
-	int size;
+	double sound_volume = 0;
+	int begin_length = 0;
+	int attack_length = 0;
+	int fade_length = 0;
+	int end_length = 0;
 	char *name;
 
 	if (s->name[0] == '*')
@@ -252,7 +485,18 @@ S_LoadSound(sfx_t *s)
 		Com_sprintf(namebuffer, sizeof(namebuffer), "sound/%s", name);
 	}
 
-	size = FS_LoadFile(namebuffer, (void **)&data);
+	S_LoadVorbis(namebuffer, s->name, &info, (void **)&data);
+
+	// can't load ogg file
+	if (!data)
+	{
+		int size = FS_LoadFile(namebuffer, (void **)&data);
+
+		if (data)
+		{
+			info = GetWavinfo(s->name, data, size);
+		}
+	}
 
 	if (!data)
 	{
@@ -261,11 +505,14 @@ S_LoadSound(sfx_t *s)
 		return NULL;
 	}
 
-	info = GetWavinfo(s->name, data, size);
+	/*
+	Com_Printf("%s: rate:%d\n\twidth:%d\n\tchannels:%d\n\tloopstart:%d\n\tsamples:%d\n\tdataofs:%d\n",
+		s->name, info.rate, info.width, info.channels, info.loopstart, info.samples, info.dataofs);
+	*/
 
-	if (info.channels != 1)
+	if (info.channels < 1 || info.channels > 2)
 	{
-		Com_Printf("%s is a stereo sample\n", s->name);
+		Com_Printf("%s has an invalid number of channels\n", s->name);
 		FS_FreeFile(data);
 		return NULL;
 	}
@@ -275,17 +522,28 @@ S_LoadSound(sfx_t *s)
 		s->is_silenced_muzzle_flash = true;
 	}
 
+	S_GetVolume(data + info.dataofs, info.samples * info.channels,
+		info.width, &sound_volume);
+
+	S_GetStatistics(data + info.dataofs, info.samples * info.channels,
+		info.width, info.channels, sound_volume, &begin_length, &end_length,
+		&attack_length, &fade_length);
+
 #if USE_OPENAL
 	if (sound_started == SS_OAL)
 	{
-		sc = AL_UploadSfx(s, &info, data + info.dataofs);
+		sc = AL_UploadSfx(s, &info, data + info.dataofs, sound_volume,
+						  begin_length, end_length,
+						  attack_length, fade_length);
 	}
 	else
 #endif
 	{
 		if (sound_started == SS_SDL)
 		{
-			if (!SDL_Cache(s, &info, data + info.dataofs))
+			if (!SDL_Cache(s, &info, data + info.dataofs, sound_volume,
+						  begin_length, end_length,
+						  attack_length, fade_length))
 			{
 				Com_Printf("Pansen!\n");
 				FS_FreeFile(data);
@@ -301,7 +559,7 @@ S_LoadSound(sfx_t *s)
 /*
  * Returns the name of a sound
  */
-sfx_t *
+static sfx_t *
 S_FindName(char *name, qboolean create)
 {
 	int i;
@@ -309,17 +567,18 @@ S_FindName(char *name, qboolean create)
 
 	if (!name)
 	{
-		Com_Error(ERR_FATAL, "S_FindName: NULL\n");
+		Com_Error(ERR_FATAL, "%s: NULL\n", __func__);
 	}
 
 	if (!name[0])
 	{
-		Com_Error(ERR_FATAL, "S_FindName: empty name\n");
+		Com_Error(ERR_FATAL, "%s: empty name\n", __func__);
 	}
 
 	if (strlen(name) >= MAX_QPATH)
 	{
-		Com_Error(ERR_FATAL, "Sound name too long: %s", name);
+		Com_Error(ERR_FATAL, "%s :Sound name too long: %s",
+			__func__, name);
 	}
 
 	/* see if already loaded */
@@ -349,7 +608,7 @@ S_FindName(char *name, qboolean create)
 	{
 		if (num_sfx == MAX_SFX)
 		{
-			Com_Error(ERR_FATAL, "S_FindName: out of sfx_t");
+			Com_Error(ERR_FATAL, "%s: out of sfx_t", __func__);
 		}
 
 		num_sfx++;
@@ -368,7 +627,7 @@ S_FindName(char *name, qboolean create)
  * Registers an alias name
  * for a sound
  */
-sfx_t *
+static sfx_t *
 S_AliasName(char *aliasname, char *truename)
 {
 	sfx_t *sfx;
@@ -391,7 +650,7 @@ S_AliasName(char *aliasname, char *truename)
 	{
 		if (num_sfx == MAX_SFX)
 		{
-			Com_Error(ERR_FATAL, "S_FindName: out of sfx_t");
+			Com_Error(ERR_FATAL, "%s: out of sfx_t", __func__);
 		}
 
 		num_sfx++;
@@ -441,14 +700,13 @@ S_RegisterSound(char *name)
 	return sfx;
 }
 
-struct sfx_s *
+static struct sfx_s *
 S_RegisterSexedSound(entity_state_t *ent, char *base)
 {
 	int n;
 	struct sfx_s *sfx;
 	char model[MAX_QPATH];
 	char sexedFilename[MAX_QPATH];
-	char maleFilename[MAX_QPATH];
 
 	/* determine what model the client is using */
 	model[0] = 0;
@@ -497,6 +755,8 @@ S_RegisterSexedSound(entity_state_t *ent, char *base)
 		}
 		else
 		{
+			char maleFilename[MAX_QPATH];
+
 			/* no, revert to the male sound in the pak0.pak */
 			Com_sprintf(maleFilename, sizeof(maleFilename),
 					"player/male/%s", base + 1);
@@ -507,6 +767,37 @@ S_RegisterSexedSound(entity_state_t *ent, char *base)
 	return sfx;
 }
 
+static qboolean
+S_HasFreeSpace(void)
+{
+	sfx_t *sfx;
+	int i, used;
+
+	used = 0;
+
+	/* check used slots */
+	for (i = 0, sfx = known_sfx; i < num_sfx; i++, sfx++)
+	{
+		if (!sfx->name[0])
+		{
+			continue;
+		}
+
+		if (sfx->registration_sequence == s_registration_sequence)
+		{
+			used ++;
+		}
+	}
+
+	if (sound_max < used)
+	{
+		sound_max = used;
+	}
+
+	// should same size of free slots as currently used
+	return (num_sfx + used) < MAX_SFX;
+}
+
 /*
  * Called after registering of
  * sound has ended
@@ -517,29 +808,32 @@ S_EndRegistration(void)
 	int i;
 	sfx_t *sfx;
 
-	/* free any sounds not from this registration sequence */
-	for (i = 0, sfx = known_sfx; i < num_sfx; i++, sfx++)
+	if (!S_HasFreeSpace())
 	{
-		if (!sfx->name[0])
+		/* free any sounds not from this registration sequence */
+		for (i = 0, sfx = known_sfx; i < num_sfx; i++, sfx++)
 		{
-			continue;
-		}
-
-		if (sfx->registration_sequence != s_registration_sequence)
-		{
-			/* it is possible to have a leftover */
-			if (sfx->cache)
+			if (!sfx->name[0])
 			{
-				Z_Free(sfx->cache); /* from a server that didn't finish loading */
+				continue;
 			}
 
-			if (sfx->truename)
+			if (sfx->registration_sequence != s_registration_sequence)
 			{
-				Z_Free(sfx->truename);
-			}
+				/* it is possible to have a leftover */
+				if (sfx->cache)
+				{
+					Z_Free(sfx->cache); /* from a server that didn't finish loading */
+				}
 
-			sfx->cache = NULL;
-			sfx->name[0] = 0;
+				if (sfx->truename)
+				{
+					Z_Free(sfx->truename);
+				}
+
+				sfx->cache = NULL;
+				sfx->name[0] = 0;
+			}
 		}
 	}
 
@@ -572,7 +866,7 @@ S_PickChannel(int entnum, int entchannel)
 
 	if (entchannel < 0)
 	{
-		Com_Error(ERR_DROP, "S_PickChannel: entchannel<0");
+		Com_Error(ERR_DROP, "%s: entchannel<0", __func__);
 	}
 
 	/* Check for replacement sound, or find the best one to replace */
@@ -627,7 +921,7 @@ S_PickChannel(int entnum, int entchannel)
 /*
  * Picks a free playsound
  */
-playsound_t *
+static playsound_t *
 S_AllocPlaysound(void)
 {
 	playsound_t *ps;
@@ -651,7 +945,7 @@ S_AllocPlaysound(void)
 /*
  * Frees a playsound
  */
-void
+static void
 S_FreePlaysound(playsound_t *ps)
 {
 	/* unlink from channel */
@@ -816,6 +1110,8 @@ S_StartSound(vec3_t origin, int entnum, int entchannel, sfx_t *sfx,
 		vec3_t orientation, direction;
 		vec_t distance_direction;
 		int dir_x, dir_y, dir_z;
+		int effect_duration = 0;
+		int effect_volume = -1;
 
 		VectorSubtract(listener_forward, listener_up, orientation);
 
@@ -839,7 +1135,25 @@ S_StartSound(vec3_t origin, int entnum, int entchannel, sfx_t *sfx,
 		dir_y = 16 * orientation[1] * direction[1];
 		dir_z = 16 * orientation[2] * direction[2];
 
-		Haptic_Feedback(sfx->name, 16 - distance_direction / 32, dir_x, dir_y, dir_z);
+		if (sfx->cache)
+		{
+			effect_duration = sfx->cache->length;
+
+			if (sfx->cache->stereo)
+			{
+				effect_duration /= 2;
+			}
+
+			/* sound near player has 16 points */
+			effect_volume = sfx->cache->volume / 16;
+		}
+
+		Haptic_Feedback(
+			sfx->name, (16 - distance_direction / 32) * effect_volume,
+			effect_duration,
+			sfx->cache->begin, sfx->cache->end,
+			sfx->cache->attack, sfx->cache->fade,
+			dir_x, dir_y, dir_z);
 	}
 
 	ps->entnum = entnum;
@@ -920,7 +1234,7 @@ S_StartLocalSound(char *sound)
 
 	if (!sfx)
 	{
-		Com_Printf("S_StartLocalSound: can't cache %s\n", sound);
+		Com_Printf("%s: can't cache %s\n", __func__, sound);
 		return;
 	}
 
@@ -978,11 +1292,12 @@ void
 S_BuildSoundList(int *sounds)
 {
 	int i;
-	int num;
-	entity_state_t *ent;
 
 	for (i = 0; i < cl.frame.num_entities; i++)
 	{
+		int num;
+		entity_state_t *ent;
+
 		if (i >= MAX_EDICTS)
 		{
 			break;
@@ -1078,17 +1393,18 @@ S_Update(vec3_t origin, vec3_t forward, vec3_t right, vec3_t up)
  * Plays one sample. Called
  * by the "play" cmd.
  */
-void
+static void
 S_Play(void)
 {
 	int i;
-	char name[256];
-	sfx_t *sfx;
 
 	i = 1;
 
 	while (i < Cmd_Argc())
 	{
+		char name[256];
+		sfx_t *sfx;
+
 		if (!strrchr(Cmd_Argv(i), '.'))
 		{
 			Q_strlcpy(name, Cmd_Argv(i), sizeof(name) - 4);
@@ -1114,16 +1430,18 @@ S_Play(void)
 /*
  * List all loaded sounds
  */
-void
+static void
 S_SoundList(void)
 {
 	int i;
 	sfx_t *sfx;
 	sfxcache_t *sc;
-	int size, total;
+	int size, total, used;
 	int numsounds;
+	qboolean freeup;
 
 	total = 0;
+	used = 0;
 	numsounds = 0;
 
 	for (sfx = known_sfx, i = 0; i < num_sfx; i++, sfx++)
@@ -1133,15 +1451,27 @@ S_SoundList(void)
 			continue;
 		}
 
+		if (sfx->registration_sequence == s_registration_sequence)
+		{
+			used++;
+		}
+
 		sc = sfx->cache;
 
 		if (sc)
 		{
 			size = sc->length * sc->width * (sc->stereo + 1);
 			total += size;
-			Com_Printf("%s(%2db) %8i : %s\n",
+			Com_Printf("%s(%2db) %8i(%d ch) %s %2.1f dB %.1fs:%.1f..%.1f..%.1f..%.1f\n",
 					sc->loopstart != -1 ? "L" : " ",
-					sc->width * 8, size, sfx->name);
+					sc->width * 8, size,
+					(sc->stereo + 1), sfx->name,
+					10 * log10((float)sc->volume / (2 << 15)),
+					(float)sc->length / 1000,
+					(float)sc->begin / 1000,
+					(float)sc->attack / 1000,
+					(float)sc->fade / 1000,
+					(float)sc->end / 1000);
 		}
 		else
 		{
@@ -1160,6 +1490,8 @@ S_SoundList(void)
 
 	Com_Printf("Total resident: %i bytes (%.2f MB) in %d sounds\n", total,
 			(float)total / 1024 / 1024, numsounds);
+	freeup = S_HasFreeSpace();
+	Com_Printf("Used %d of %d sounds%s.\n", used, sound_max, freeup ? ", has free space" : "");
 }
 
 /* ----------------------------------------------------------------- */
@@ -1168,7 +1500,7 @@ S_SoundList(void)
  * Prints information about the
  * active sound backend
  */
-void
+static void
 S_SoundInfo_f(void)
 {
 	if (sound_started == SS_NOT)
@@ -1216,8 +1548,8 @@ S_Init(void)
 	s_show = Cvar_Get("s_show", "0", 0);
 	s_testsound = Cvar_Get("s_testsound", "0", 0);
 	s_ambient = Cvar_Get("s_ambient", "1", 0);
-    s_underwater = Cvar_Get("s_underwater", "1", CVAR_ARCHIVE);
-    s_underwater_gain_hf = Cvar_Get("s_underwater_gain_hf", "0.25", CVAR_ARCHIVE);
+	s_underwater = Cvar_Get("s_underwater", "1", CVAR_ARCHIVE);
+	s_underwater_gain_hf = Cvar_Get("s_underwater_gain_hf", "0.25", CVAR_ARCHIVE);
 	s_doppler = Cvar_Get("s_doppler", "0", CVAR_ARCHIVE);
 	s_ps_sorting = Cvar_Get("s_ps_sorting", "1", CVAR_ARCHIVE);
 
@@ -1249,6 +1581,7 @@ S_Init(void)
 
 	num_sfx = 0;
 	paintedtime = 0;
+	sound_max = 0;
 
 	OGG_Init();
 
