@@ -89,6 +89,15 @@ CreateSDLWindow(int flags, int w, int h)
 		last_position_x = last_position_y = SDL_WINDOWPOS_UNDEFINED_DISPLAY((int)vid_displayindex->value);
 	}
 
+	/* Force the window to minimize when focus is lost. This was the
+	 * default behavior until SDL 2.0.12 and changed with 2.0.14.
+	 * The windows staying maximized has some odd implications for
+	 * window ordering under Windows and some X11 window managers
+	 * like kwin. See:
+	 *  * https://github.com/libsdl-org/SDL/issues/4039
+	 *  * https://github.com/libsdl-org/SDL/issues/3656 */
+	SDL_SetHint(SDL_HINT_VIDEO_MINIMIZE_ON_FOCUS_LOSS, "1");
+
 	window = SDL_CreateWindow("Yamagi Quake II", last_position_x, last_position_y, w, h, flags);
 
 	if (window)
@@ -162,7 +171,7 @@ CreateSDLWindow(int flags, int w, int h)
 				SDL_DestroyWindow(window);
 				window = NULL;
 
-				Com_Printf("Can't get display mode: %s\n", SDL_GetError());
+				Com_Printf("Still in wrong display mode: %ix%i instead of %ix%i\n", real_mode.w, real_mode.h, w, h);
 
 				return false;
 			}
@@ -219,11 +228,11 @@ GetFullscreenType()
 {
 	if (SDL_GetWindowFlags(window) & SDL_WINDOW_FULLSCREEN_DESKTOP)
 	{
-		return 1;
+		return 2;
 	}
 	else if (SDL_GetWindowFlags(window) & SDL_WINDOW_FULLSCREEN)
 	{
-		return 2;
+		return 1;
 	}
 	else
 	{
@@ -431,7 +440,9 @@ GLimp_Shutdown(void)
 {
 	ShutdownGraphics();
 
-	if (SDL_WasInit(SDL_INIT_EVERYTHING) == SDL_INIT_VIDEO)
+	// SDL_INIT_VIDEO implies SDL_INIT_EVENTS
+	const Uint32 subsystems = SDL_INIT_VIDEO | SDL_INIT_EVENTS;
+	if (SDL_WasInit(SDL_INIT_EVERYTHING) == subsystems)
 	{
 		SDL_Quit();
 	}
@@ -457,11 +468,11 @@ GLimp_InitGraphics(int fullscreen, int *pwidth, int *pheight)
 
 	if (fullscreen == 1)
 	{
-		fs_flag = SDL_WINDOW_FULLSCREEN_DESKTOP;
+		fs_flag = SDL_WINDOW_FULLSCREEN;
 	}
 	else if (fullscreen == 2)
 	{
-		fs_flag = SDL_WINDOW_FULLSCREEN;
+		fs_flag = SDL_WINDOW_FULLSCREEN_DESKTOP;
 	}
 
 	/* Only do this if we already have a working window and a fully
@@ -522,7 +533,7 @@ GLimp_InitGraphics(int fullscreen, int *pwidth, int *pheight)
 	}
 
 	/* Mkay, now the hard work. Let's create the window. */
-	cvar_t *gl_msaa_samples = Cvar_Get("gl_msaa_samples", "0", CVAR_ARCHIVE);
+	cvar_t *gl_msaa_samples = Cvar_Get("r_msaa_samples", "0", CVAR_ARCHIVE);
 
 	while (1)
 	{
@@ -530,16 +541,26 @@ GLimp_InitGraphics(int fullscreen, int *pwidth, int *pheight)
 		{
 			if((flags & SDL_WINDOW_OPENGL) && gl_msaa_samples->value)
 			{
+				int msaa_samples = gl_msaa_samples->value;
+
+				if (msaa_samples > 0)
+				{
+					msaa_samples /= 2;
+				}
+
 				Com_Printf("SDL SetVideoMode failed: %s\n", SDL_GetError());
-				Com_Printf("Reverting to %s r_mode %i (%ix%i) without MSAA.\n",
+				Com_Printf("Reverting to %s r_mode %i (%ix%i) with %dx MSAA.\n",
 					        (flags & fs_flag) ? "fullscreen" : "windowed",
-					        (int) Cvar_VariableValue("r_mode"), width, height);
+					        (int) Cvar_VariableValue("r_mode"), width, height,
+					        msaa_samples);
 
 				/* Try to recover */
-				Cvar_SetValue("gl_msaa_samples", 0);
+				Cvar_SetValue("r_msaa_samples", msaa_samples);
 
-				SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 0);
-				SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 0);
+				SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS,
+					msaa_samples > 0 ? 1 : 0);
+				SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES,
+					msaa_samples);
 			}
 			else if (width != 640 || height != 480 || (flags & fs_flag))
 			{
