@@ -26,6 +26,8 @@
  * =======================================================================
  */
 
+#include <ctype.h>
+
 #include "header/common.h"
 
 #define MAX_ALIAS_NAME 32
@@ -298,9 +300,9 @@ Cbuf_AddLateCommands(void)
 {
 	int i, j;
 	int s;
-	char *text, *build, c;
+	char *text, c;
 	int argc;
-	qboolean ret;
+	qboolean has_args = false;
 
 	/* build the combined string to parse from */
 	s = 0;
@@ -330,40 +332,31 @@ Cbuf_AddLateCommands(void)
 	}
 
 	/* pull out the commands */
-	build = Z_Malloc(s + 1);
-	build[0] = 0;
-
 	for (i = 0; i < s - 1; i++)
 	{
 		if (text[i] == '+')
 		{
 			i++;
 
-			for (j = i; (text[j] != '+') && (text[j] != '-') && (text[j] != 0); j++)
+			for (j = i; (text[j] != '+') && !(text[j] == '-' && text[j-1] == ' ') && (text[j] != 0); j++)
 			{
 			}
 
 			c = text[j];
 			text[j] = 0;
 
-			strcat(build, text + i);
-			strcat(build, "\n");
+			Cbuf_AddText(text + i);
+			Cbuf_AddText("\n");
+
+			has_args = true;
 			text[j] = c;
 			i = j - 1;
 		}
 	}
 
-	ret = (build[0] != 0);
-
-	if (ret)
-	{
-		Cbuf_AddText(build);
-	}
-
 	Z_Free(text);
-	Z_Free(build);
 
-	return ret;
+	return has_args;
 }
 
 /*
@@ -786,12 +779,6 @@ Cmd_Exists(char *cmd_name)
 	return false;
 }
 
-int
-qsort_strcomp(const void *s1, const void *s2)
-{
-	return strcmp(*(char **)s1, *(char **)s2);
-}
-
 char *
 Cmd_CompleteCommand(char *partial)
 {
@@ -877,7 +864,7 @@ Cmd_CompleteCommand(char *partial)
 		}
 
 		/* Sort it */
-		qsort(pmatch, i, sizeof(pmatch[0]), qsort_strcomp);
+		qsort(pmatch, i, sizeof(pmatch[0]), Q_sort_strcomp);
 
 		Com_Printf("\n\n");
 
@@ -945,12 +932,12 @@ Cmd_CompleteMapCommand(char *partial)
 			mapName = strtok(mapName, ".");
 
 			/* check for exact match */
-			if (!strcmp(partial, mapName))
+			if (!Q_strcasecmp(partial, mapName))
 			{
 				strcpy(retval, partial);
 			}
 			/* check for partial match */
-			else if (!strncmp(partial, mapName, len))
+			else if (!Q_strncasecmp(partial, mapName, len))
 			{
 				pmatch[nbMatches] = mapName;
 				nbMatches++;
@@ -975,7 +962,7 @@ Cmd_CompleteMapCommand(char *partial)
 			{
 				for (k = 1; k < nbMatches; k++)
 				{
-					if (j >= strlen(pmatch[k]) || pmatch[0][j] != pmatch[k][j])
+					if (j >= strlen(pmatch[k]) || tolower((unsigned char)pmatch[0][j]) != tolower((unsigned char)pmatch[k][j]))
 					{
 						partialFillContinue = false;
 						break;
@@ -1135,3 +1122,16 @@ Cmd_Init(void)
 	Cmd_AddCommand("wait", Cmd_Wait_f);
 }
 
+void
+Cmd_Shutdown(void)
+{
+	cmdalias_t *next;
+
+	while (cmd_alias != NULL)
+	{
+		next = cmd_alias->next;
+		Z_Free(cmd_alias->value);
+		Z_Free(cmd_alias);
+		cmd_alias = next;
+	}
+}
