@@ -25,13 +25,14 @@
  * =======================================================================
  */
 
+#include <libgen.h>
+
 #include "header/common.h"
 #include "header/glob.h"
 #include "unzip/unzip.h"
 
 #include "../client/sound/header/vorbis.h"
 
-
 #define MAX_HANDLES 512
 #define MAX_MODS 32
 #define MAX_PAKS 100
@@ -368,7 +369,7 @@ FS_FCloseFile(fileHandle_t f)
  * for streaming data out of either a pak file or a seperate file.
  */
 int
-FS_FOpenFile(const char *name, fileHandle_t *f, qboolean gamedir_only)
+FS_FOpenFile(const char *rawname, fileHandle_t *f, qboolean gamedir_only)
 {
 	char path[MAX_OSPATH], lwrName[MAX_OSPATH];
 	fsHandle_t *handle;
@@ -376,6 +377,58 @@ FS_FOpenFile(const char *name, fileHandle_t *f, qboolean gamedir_only)
 	fsSearchPath_t *search;
 	int i;
 
+	// Remove self references and empty dirs from the requested path.
+	// ZIPs and PAKs don't support them, but they may be hardcoded in
+	// some custom maps or models.
+	char name[MAX_QPATH] = {};
+	size_t namelen = strlen(rawname);
+	for (int input = 0, output = 0; input < namelen; input++)
+	{
+		// Remove self reference.
+		if (rawname[input] == '.')
+		{
+			if (output > 0)
+			{
+				// Inside the path.
+				if (name[output - 1] == '/' && rawname[input + 1] == '/')
+				{
+					input++;
+					continue;
+				}
+			}
+			else
+			{
+				// At the beginning. Note: This is save because the Quake II
+				// VFS doesn't have a current working dir. Paths are always
+				// absolute.
+				if (rawname[input + 1] == '/')
+				{
+					continue;
+				}
+			}
+		}
+
+		// Empty dir.
+		if (rawname[input] == '/')
+		{
+			if (rawname[input + 1] == '/')
+			{
+				continue;
+			}
+		}
+
+		// Pathes starting with a /. I'm not sure if this is
+		// a problem. It shouldn't hurt to remove the leading
+		// slash, though.
+		if (rawname[input] == '/' && output == 0)
+		{
+			continue;
+		}
+
+		name[output] = rawname[input];
+		output++;
+	}
+
 	file_from_protected_pak = false;
 	handle = FS_HandleForFile(name, f);
 	Q_strlcpy(handle->name, name, sizeof(handle->name));
@@ -687,6 +740,52 @@ FS_FreeFile(void *buffer)
 	Z_Free(buffer);
 }
 
+fsRawPath_t *FS_FreeRawPaths(fsRawPath_t *start, fsRawPath_t *end)
+{
+	fsRawPath_t *cur = start;
+	fsRawPath_t *next;
+
+	while (cur != end)
+	{
+		next = cur->next;
+		Z_Free(cur);
+		cur = next;
+	}
+
+	return cur;
+}
+
+fsSearchPath_t *FS_FreeSearchPaths(fsSearchPath_t *start, fsSearchPath_t *end)
+{
+	fsSearchPath_t *cur = start;
+	fsSearchPath_t *next;
+
+	while (cur != end)
+	{
+		if (cur->pack)
+		{
+			if (cur->pack->pak)
+			{
+				fclose(cur->pack->pak);
+			}
+
+			if (cur->pack->pk3)
+			{
+				unzClose(cur->pack->pk3);
+			}
+
+			Z_Free(cur->pack->files);
+			Z_Free(cur->pack);
+		}
+
+		next = cur->next;
+		Z_Free(cur);
+		cur = next;
+	}
+
+	return cur;
+}
+
 /*
  * Takes an explicit (not game tree related) path to a pak file.
  *
@@ -1254,6 +1353,107 @@ FS_FreeList(char **list, int nfiles)
 	free(list);
 }
 
+/*
+ * Combs all Raw search paths to find game dirs containing PAK/PK2/PK3 files.
+ * Returns an alphabetized array of unique relative dir names.
+ */
+char**
+FS_ListMods(int *nummods)
+{
+	int nmods = 0, numdirchildren, numpacksinchilddir, searchpathlength;
+	char findnamepattern[MAX_OSPATH], modname[MAX_QPATH], searchpath[MAX_OSPATH];
+	char **dirchildren, **packsinchilddir, **modnames;
+
+	modnames = malloc((MAX_QPATH + 1) * (MAX_MODS + 1));
+	memset(modnames, 0, (MAX_QPATH + 1) * (MAX_MODS + 1));
+
+	// iterate over all Raw paths
+	for (fsRawPath_t *search = fs_rawPath; search; search = search->next)
+	{
+		searchpathlength = strlen(search->path);
+		if(!searchpathlength)
+		{
+			continue;
+		}
+
+		// make sure this Raw path ends with a '/' otherwise FS_ListFiles will open its parent dir
+		if(search->path[searchpathlength - 1] != '/')
+		{
+			Com_sprintf(searchpath, sizeof(searchpath), "%s/*", search->path);
+		}
+		else
+		{
+			Com_sprintf(searchpath, sizeof(searchpath), "%s*", search->path);
+		}
+
+		dirchildren = FS_ListFiles(searchpath, &numdirchildren, 0, 0);
+
+		if (dirchildren == NULL)
+		{
+			continue;
+		}
+
+		// iterate over the children of this Raw path (unless we've already got enough mods)
+		for (int i = 0; i < numdirchildren && nmods < MAX_MODS; i++)
+		{
+			if(dirchildren[i] == NULL)
+			{
+				continue;
+			}
+
+			numpacksinchilddir = 0;
+
+			// iterate over supported pack types, but ignore ZIP files (they cause false positives)
+			for (int j = 0; j < sizeof(fs_packtypes) / sizeof(fs_packtypes[0]); j++)
+			{
+				if (strcmp("zip", fs_packtypes[j].suffix) != 0)
+				{
+					Com_sprintf(findnamepattern, sizeof(findnamepattern), "%s/*.%s", dirchildren[i], fs_packtypes[j].suffix);
+
+					packsinchilddir = FS_ListFiles(findnamepattern, &numpacksinchilddir, 0, 0);
+					FS_FreeList(packsinchilddir, numpacksinchilddir);
+
+					// if this dir has some pack files, add it if not already in the list
+					if (numpacksinchilddir > 0)
+					{
+						qboolean matchfound = false;
+
+						Com_sprintf(modname, sizeof(modname), "%s", strrchr(dirchildren[i], '/') + 1);
+
+						for (int k = 0; k < nmods; k++)
+						{
+							if (strcmp(modname, modnames[k]) == 0)
+							{
+								matchfound = true;
+								break;
+							}
+						}
+
+						if (!matchfound)
+						{
+							modnames[nmods] = malloc(strlen(modname) + 1);
+							strcpy(modnames[nmods], modname);
+
+							nmods++;
+						}
+
+						break;
+					}
+				}
+			}
+		}
+
+		FS_FreeList(dirchildren, numdirchildren);
+	}
+
+	modnames[nmods] = 0;
+
+	qsort(modnames, nmods, sizeof(modnames[0]), Q_sort_strcomp);
+
+	*nummods = nmods;
+	return modnames;
+}
+
 /*
  * Directory listing.
  */
@@ -1416,12 +1616,15 @@ FS_GetNextRawPath(const char* lastRawPath)
 
 void
 FS_AddDirToSearchPath(char *dir, qboolean create) {
+	char *file;
 	char **list;
 	char path[MAX_OSPATH];
-	int i, j;
+	char *tmp;
+	int i, j, k;
 	int nfiles;
 	fsPack_t *pack = NULL;
 	fsSearchPath_t *search;
+	qboolean nextpak;
 	size_t len = strlen(dir);
 
 	// The directory must not end with an /. It would
@@ -1439,7 +1642,8 @@ FS_AddDirToSearchPath(char *dir, qboolean create) {
 	// be the last directory added to the search path.
 	Q_strlcpy(fs_gamedir, dir, sizeof(fs_gamedir));
 
-	if (create) {
+	if (create)
+	{
 		FS_CreatePath(fs_gamedir);
 	}
 
@@ -1449,11 +1653,15 @@ FS_AddDirToSearchPath(char *dir, qboolean create) {
 	search->next = fs_searchPaths;
 	fs_searchPaths = search;
 
-	// We need to add numbered paks in the directory in
-	// sequence and all other paks after them. Otherwise
-	// the gamedata may break.
-	for (i = 0; i < sizeof(fs_packtypes) / sizeof(fs_packtypes[0]); i++) {
-		for (j = 0; j < MAX_PAKS; j++) {
+
+	// Numbered paks contain the official game data, they
+	// need to be added first and are marked protected.
+	// Files from protected paks are never offered for
+	// download.
+	for (i = 0; i < sizeof(fs_packtypes) / sizeof(fs_packtypes[0]); i++)
+	{
+		for (j = 0; j < MAX_PAKS; j++)
+		{
 			Com_sprintf(path, sizeof(path), "%s/pak%d.%s", dir, j, fs_packtypes[i].suffix);
 
 			switch (fs_packtypes[i].format)
@@ -1490,8 +1698,13 @@ FS_AddDirToSearchPath(char *dir, qboolean create) {
 		}
 	}
 
-	// And as said above all other pak files.
-	for (i = 0; i < sizeof(fs_packtypes) / sizeof(fs_packtypes[0]); i++) {
+	// All other pak files are added after the numbered paks.
+	// They aren't sorted in any way, but added in the same
+	// sequence as they're returned by FS_ListFiles. This is
+	// fragile and file system dependend. We cannot change
+	// this, since it might break existing installations.
+	for (i = 0; i < sizeof(fs_packtypes) / sizeof(fs_packtypes[0]); i++)
+	{
 		Com_sprintf(path, sizeof(path), "%s/*.%s", dir, fs_packtypes[i].suffix);
 
 		// Nothing here, next pak type please.
@@ -1500,14 +1713,33 @@ FS_AddDirToSearchPath(char *dir, qboolean create) {
 			continue;
 		}
 
-		Com_sprintf(path, sizeof(path), "%s/pak*.%s", dir, fs_packtypes[i].suffix);
-
 		for (j = 0; j < nfiles - 1; j++)
 		{
-			// If the pak starts with the string 'pak' it's ignored.
-			// This is somewhat stupid, it would be better to ignore
-			// just pak%d...
-			if (glob_match(path, list[j]))
+			// Sort out numbered paks. This is as inefficient as
+			// it can be, but it doesn't matter. This is done only
+			// once at client or game startup.
+			nextpak = false;
+
+			for (k = 0; k < MAX_PAKS; k++)
+			{
+				// basename() may alter the given string.
+				// We need to work around that...
+				tmp = strdup(list[j]);
+				file = basename(tmp);
+
+				Com_sprintf(path, sizeof(path), "pak%d.%s", k, fs_packtypes[i].suffix);
+
+				if (Q_strcasecmp(path, file) == 0)
+				{
+					nextpak = true;
+					free(tmp);
+					break;
+				}
+
+				free(tmp);
+			}
+
+			if (nextpak)
 			{
 				continue;
 			}
@@ -1568,6 +1800,19 @@ void FS_BuildGenericSearchPath(void) {
 	Sys_Mkdir(path);
 }
 
+// filesystem.c is used by the client and the server,
+// it includes common.h only. Not the client not the
+// server header. The game reset logic messes with
+// client state, so we need some forwar declarations
+// here.
+#ifndef DEDICATED_ONLY
+// Variables
+extern qboolean menu_startdemoloop;
+
+// Functions
+void CL_WriteConfiguration(void);
+#endif
+
 void
 FS_BuildGameSpecificSearchPath(char *dir)
 {
@@ -1577,7 +1822,12 @@ FS_BuildGameSpecificSearchPath(char *dir)
 	char path[MAX_OSPATH];
 	int i;
 	fsRawPath_t *search;
-	fsSearchPath_t *next;
+
+#ifndef DEDICATED_ONLY
+	// Write the config. Otherwise changes made by the
+	// current mod are lost.
+	CL_WriteConfiguration();
+#endif
 
 	// empty string means baseq2
 	if(dir[0] == '\0')
@@ -1596,28 +1846,7 @@ FS_BuildGameSpecificSearchPath(char *dir)
 	// We may already have specialised directories in our search
 	// path. This can happen if the server changes the mod. Let's
 	// remove them.
-	while (fs_searchPaths != fs_baseSearchPaths)
-	{
-		if (fs_searchPaths->pack)
-		{
-			if (fs_searchPaths->pack->pak)
-			{
-				fclose(fs_searchPaths->pack->pak);
-			}
-
-			if (fs_searchPaths->pack->pk3)
-			{
-				unzClose(fs_searchPaths->pack->pk3);
-			}
-
-			Z_Free(fs_searchPaths->pack->files);
-			Z_Free(fs_searchPaths->pack);
-		}
-
-		next = fs_searchPaths->next;
-		Z_Free(fs_searchPaths);
-		fs_searchPaths = next;
-	}
+	fs_searchPaths = FS_FreeSearchPaths(fs_searchPaths, fs_baseSearchPaths);
 
 	/* Close open files for game dir. */
 	for (i = 0; i < MAX_HANDLES; i++)
@@ -1686,6 +1915,21 @@ FS_BuildGameSpecificSearchPath(char *dir)
 		free(mapnames);
 		mapnames = NULL;
 	}
+
+	// Start the demoloop, if requested. This is kind of hacky: Normaly the
+	// demo loop would be started by the menu, after changeing the 'game'
+	// cvar. However, the demo loop is implemented by aliases. Since the
+	// game isn't changed right after the cvar is set (but when the control
+	// flow enters this function) the aliases evaulate to the wrong game.
+	// Work around that by injection the demo loop into the command buffer
+	// here, right after the game was changed. Do it only when the game was
+	// changed though the menu, otherwise we might break into the demo loop
+	// after we've received a latched cvar from the server.
+	if (menu_startdemoloop)
+	{
+		Cbuf_AddText("d1\n");
+		menu_startdemoloop = false;
+	}
 #endif
 }
 
@@ -1703,11 +1947,53 @@ const char* FS_GetFilenameForHandle(fileHandle_t f)
 
 // --------
 
-void FS_AddDirToRawPath (const char *dir, qboolean create) {
-	fsRawPath_t *search;
+static void FS_AddDirToRawPath (const char *rawdir, qboolean create, qboolean required) {
+	char dir[MAX_OSPATH] = {0};
 
-	// Add the directory
-	search = Z_Malloc(sizeof(fsRawPath_t));
+	// Get the realpath.
+	if (!Sys_Realpath(rawdir, dir, sizeof(dir)))
+	{
+		if (required)
+		{
+			Com_Error(ERR_FATAL, "Couldn't add required directory %s to search path\n", rawdir);
+		}
+
+		return;
+	}
+
+	// Convert backslashes to forward slashes.
+	for (int i = 0; i < strlen(dir); i++)
+	{
+		if (dir[i] == '\\')
+		{
+			dir[i] = '/';
+		}
+	}
+
+	// Make sure that the dir doesn't end with a slash.
+	for (size_t s = strlen(dir) - 1; s >= 0; s--)
+	{
+		if (dir[s] == '/')
+		{
+			dir[s] = '\0';
+		}
+		else
+		{
+			break;
+		}
+	}
+
+	// Bail out if the dir was already added.
+	for (fsRawPath_t *search = fs_rawPath; search; search = search->next)
+	{
+		if (strcmp(search->path, dir) == 0)
+		{
+			return;
+		}
+	}
+
+	// Add the directory.
+	fsRawPath_t *search = Z_Malloc(sizeof(fsRawPath_t));
 	Q_strlcpy(search->path, dir, sizeof(search->path));
 	search->create = create;
 	search->next = fs_rawPath;
@@ -1716,36 +2002,43 @@ void FS_AddDirToRawPath (const char *dir, qboolean create) {
 
 
 void FS_BuildRawPath(void) {
-	// Add $HOME/.yq2 (MUST be the last dir!)
+	// Add $HOME/.yq2, MUST be the last dir! Required,
+	// otherwise the config cannot be written.
 	if (!is_portable) {
 		const char *homedir = Sys_GetHomeDir();
 
 		if (homedir != NULL) {
-			FS_AddDirToRawPath(homedir, true);
+			FS_AddDirToRawPath(homedir, true, true);
 		}
 	}
 
-	// Add $binarydir
+	// Add binary dir. Required, because the renderer
+	// libraries are loaded from it.
 	const char *binarydir = Sys_GetBinaryDir();
 
 	if(binarydir[0] != '\0')
 	{
-		FS_AddDirToRawPath(binarydir, false);
+		FS_AddDirToRawPath(binarydir, false, true);
 	}
 
-	// Add $basedir/
-	FS_AddDirToRawPath(datadir, false);
+	// Add data dir. Required, when the user gives us
+	// a data dir he expects it in a working state.
+	FS_AddDirToRawPath(datadir, false, true);
 
-	// Add SYSTEMDIR
+	// Add SYSTEMDIR. Optional, the user may have a
+	// binary compiled with SYSTEMWIDE (installed from
+	// packages), but no systemwide game data.
 #ifdef SYSTEMWIDE
-	FS_AddDirToRawPath(SYSTEMDIR, false);
+	FS_AddDirToRawPath(SYSTEMDIR, false, false);
 #endif
 
 	// The CD must be the last directory of the path,
 	// otherwise we cannot be sure that the game won't
-	// stream the videos from the CD.
+	// stream the videos from the CD. Required, if the
+	// user sets a CD path, he expects data getting
+	// read from the CD.
 	if (fs_cddir->string[0] != '\0') {
-		FS_AddDirToRawPath(fs_cddir->string, false);
+		FS_AddDirToRawPath(fs_cddir->string, false, true);
 	}
 }
 
@@ -1802,101 +2095,12 @@ FS_InitFilesystem(void)
 	Com_Printf("Using '%s' for writing.\n", fs_gamedir);
 }
 
-extern int qsort_strcomp(const void *s1, const void *s2);
 
-/*
- * Combs all Raw search paths to find game dirs containing PAK/PK2/PK3 files.
- * Returns an alphabetized array of unique relative dir names.
- */
-char**
-FS_ListMods(int *nummods)
+void
+FS_ShutdownFilesystem(void)
 {
-	int nmods = 0, numdirchildren, numpacksinchilddir, searchpathlength;
-	char findnamepattern[MAX_OSPATH], modname[MAX_QPATH], searchpath[MAX_OSPATH];
-	char **dirchildren, **packsinchilddir, **modnames;
+	fs_searchPaths = FS_FreeSearchPaths(fs_searchPaths, NULL);
+	fs_rawPath = FS_FreeRawPaths(fs_rawPath, NULL);
 
-	modnames = malloc((MAX_QPATH + 1) * (MAX_MODS + 1));
-	memset(modnames, 0, (MAX_QPATH + 1) * (MAX_MODS + 1));
-
-	// iterate over all Raw paths
-	for (fsRawPath_t *search = fs_rawPath; search; search = search->next)
-	{
-		searchpathlength = strlen(search->path);
-		if(!searchpathlength)
-		{
-			continue;
-		}
-
-		// make sure this Raw path ends with a '/' otherwise FS_ListFiles will open its parent dir
-		if(search->path[searchpathlength - 1] != '/')
-		{
-			Com_sprintf(searchpath, sizeof(searchpath), "%s/*", search->path);
-		}
-		else
-		{
-			Com_sprintf(searchpath, sizeof(searchpath), "%s*", search->path);
-		}
-
-		dirchildren = FS_ListFiles(searchpath, &numdirchildren, 0, 0);
-
-		// iterate over the children of this Raw path (unless we've already got enough mods)
-		for (int i = 0; i < numdirchildren && nmods < MAX_MODS; i++)
-		{
-			if(dirchildren[i] == NULL)
-			{
-				continue;
-			}
-
-			numpacksinchilddir = 0;
-
-			// iterate over supported pack types, but ignore ZIP files (they cause false positives)
-			for (int j = 0; j < sizeof(fs_packtypes) / sizeof(fs_packtypes[0]); j++)
-			{
-				if (strcmp("zip", fs_packtypes[j].suffix) != 0)
-				{
-					Com_sprintf(findnamepattern, sizeof(findnamepattern), "%s/*.%s", dirchildren[i], fs_packtypes[j].suffix);
-
-					packsinchilddir = FS_ListFiles(findnamepattern, &numpacksinchilddir, 0, 0);
-					FS_FreeList(packsinchilddir, numpacksinchilddir);
-
-					// if this dir has some pack files, add it if not already in the list
-					if (numpacksinchilddir > 0)
-					{
-						qboolean matchfound = false;
-
-						Com_sprintf(modname, sizeof(modname), "%s", strrchr(dirchildren[i], '/') + 1);
-
-						for (int k = 0; k < nmods; k++)
-						{
-							if (strcmp(modname, modnames[k]) == 0)
-							{
-								matchfound = true;
-								break;
-							}
-						}
-
-						if (!matchfound)
-						{
-							modnames[nmods] = malloc(strlen(modname) + 1);
-							strcpy(modnames[nmods], modname);
-
-							nmods++;
-						}
-
-						break;
-					}
-				}
-			}
-		}
-
-		FS_FreeList(dirchildren, numdirchildren);
-	}
-
-	modnames[nmods] = 0;
-
-	qsort(modnames, nmods, sizeof(modnames[0]), qsort_strcomp);
-
-	*nummods = nmods;
-	return modnames;
+	fs_baseSearchPaths = NULL;
 }
-
