@@ -34,7 +34,6 @@ cvar_t *timescale;
 cvar_t *fixedtime;
 cvar_t *cl_maxfps;
 cvar_t *dedicated;
-cvar_t *busywait;
 
 extern cvar_t *logfile_active;
 extern jmp_buf abortframe; /* an ERR_DROP occured, exit the entire frame */
@@ -42,6 +41,7 @@ extern zhead_t z_chain;
 
 #ifndef DEDICATED_ONLY
 FILE *log_stats_file;
+cvar_t *busywait;
 cvar_t *cl_async;
 cvar_t *cl_timedemo;
 cvar_t *vid_maxfps;
@@ -67,6 +67,7 @@ int curtime;
 
 #ifndef DEDICATED_ONLY
 void Key_Init(void);
+void Key_Shutdown(void);
 void SCR_EndLoadingPlaque(void);
 #endif
 
@@ -80,6 +81,8 @@ char userGivenGame[MAX_QPATH];
 // Hack for the signal handlers.
 qboolean quitnextframe;
 
+static void Qcommon_Frame(int usec);
+
 // ----
 
 static void
@@ -106,24 +109,30 @@ Qcommon_Buildstring(void)
 #ifndef DEDICATED_ONLY
 	printf("Client build options:\n");
 
+#ifdef USE_CURL
+	printf(" + cURL HTTP downloads\n");
+#else
+	printf(" - cURL HTTP downloads\n");
+#endif
+
 #ifdef USE_OPENAL
 	printf(" + OpenAL audio\n");
 #else
 	printf(" - OpenAL audio\n");
 #endif
+
+#ifdef SYSTEMWIDE
+	printf(" + Systemwide installation\n");
+#else
+	printf(" - Systemwide installation\n");
+#endif
 #endif
 
 	printf("Platform: %s\n", YQ2OSTYPE);
 	printf("Architecture: %s\n", YQ2ARCH);
 }
 
-#ifndef DEDICATED_ONLY
-#define FRAMEDELAY 5
-#else
-#define FRAMEDELAY 850
-#endif
-
-void
+static void
 Qcommon_Mainloop(void)
 {
 	long long newtime;
@@ -132,34 +141,47 @@ Qcommon_Mainloop(void)
 	/* The mainloop. The legend. */
 	while (1)
 	{
-		// Throttle the game a little bit.
-		if (busywait->value)
+#ifndef DEDICATED_ONLY
+		if (!cl_timedemo->value)
 		{
-			long long spintime = Sys_Microseconds();
-
-			while (1)
+			// Throttle the game a little bit.
+			if (busywait->value)
 			{
+				long long spintime = Sys_Microseconds();
+
+				while (1)
+				{
+					/* Give the CPU a hint that this is a very tight
+					   spinloop. One PAUSE instruction each loop is
+					   enough to reduce power consumption and head
+					   dispersion a lot, it's 95°C against 67°C on
+					   a Kaby Lake laptop. */
 #if defined (__GNUC__) && (__i386 || __x86_64__)
-				/* Give the CPU a hint that this is a very tight
-				   spinloop. One PAUSE instruction each loop is
-				   enough to reduce power consumption and head
-				   dispersion a lot, it's 95°C against 67°C on
-				   a Kaby Lake laptop. */
-				asm("pause");
+					asm("pause");
+#elif defined(__aarch64__) || (defined(__ARM_ARCH) && __ARM_ARCH >= 7) || defined(__ARM_ARCH_6K__)
+					asm("yield");
 #endif
 
-				if (Sys_Microseconds() - spintime >= FRAMEDELAY)
-				{
-					break;
+					if (Sys_Microseconds() - spintime >= 5)
+					{
+						break;
+					}
 				}
 			}
+			else
+			{
+				Sys_Nanosleep(5000);
+			}
 		}
-		else
-		{
-			Sys_Nanosleep(FRAMEDELAY * 1000);
-		}
+#else
+		Sys_Nanosleep(850000);
+#endif
 
 		newtime = Sys_Microseconds();
+
+		// Save global time for network- und input code.
+		curtime = (int)(newtime / 1000ll);
+
 		Qcommon_Frame(newtime - oldtime);
 		oldtime = newtime;
 	}
@@ -170,12 +192,14 @@ void Qcommon_ExecConfigs(qboolean gameStartUp)
 	Cbuf_AddText("exec default.cfg\n");
 	Cbuf_AddText("exec yq2.cfg\n");
 	Cbuf_AddText("exec config.cfg\n");
-	if(gameStartUp)
+	Cbuf_AddText("exec autoexec.cfg\n");
+
+	if (gameStartUp)
 	{
-		// only when the game is first started we execute autoexec.cfg and set the cvars from commandline
-		Cbuf_AddText("exec autoexec.cfg\n");
+		/* Process cmd arguments only startup. */
 		Cbuf_AddEarlyCommands(true);
 	}
+
 	Cbuf_Execute();
 }
 
@@ -195,6 +219,8 @@ static qboolean checkForHelp(int argc, char **argv)
 				printf("Yamagi Quake II v%s\n", YQ2VERSION);
 				printf("Most interesting commandline arguments:\n");
 				printf("-h or --help: Show this help\n");
+				printf("-cfgdir <path>\n");
+				printf("  set the name of your config directory\n");
 				printf("-datadir <path>\n");
 				printf("  set path to your Quake2 game data (the directory baseq2/ is in)\n");
 				printf("-portable\n");
@@ -313,9 +339,9 @@ Qcommon_Init(int argc, char **argv)
 	char *s;
 	s = va("%s %s %s %s", YQ2VERSION, YQ2ARCH, BUILD_DATE, YQ2OSTYPE);
 	Cvar_Get("version", s, CVAR_SERVERINFO | CVAR_NOSET);
-	busywait = Cvar_Get("busywait", "1", CVAR_ARCHIVE);
 
 #ifndef DEDICATED_ONLY
+	busywait = Cvar_Get("busywait", "1", CVAR_ARCHIVE);
 	cl_async = Cvar_Get("cl_async", "1", CVAR_ARCHIVE);
 	cl_timedemo = Cvar_Get("timedemo", "0", 0);
 	dedicated = Cvar_Get("dedicated", "0", CVAR_NOSET);
@@ -375,7 +401,7 @@ Qcommon_Init(int argc, char **argv)
 }
 
 #ifndef DEDICATED_ONLY
-void
+static void
 Qcommon_Frame(int usec)
 {
 	// Used for the dedicated server console.
@@ -506,11 +532,6 @@ Qcommon_Frame(int usec)
 		Cvar_SetValue("cl_maxfps", 60);
 	}
 
-
-	// Save global time for network- und input code.
-	curtime = Sys_Milliseconds();
-
-
 	// Calculate target and renderframerate.
 	if (R_IsVSyncActive())
 	{
@@ -581,9 +602,15 @@ Qcommon_Frame(int usec)
 			}
 		}
 	}
-	else if (clienttimedelta < 1000 || servertimedelta < 1000)
+	else
 	{
-		return;
+		/* minimal frame time in timedemo fps * 5 ~ 5000 */
+		int minframetime;
+
+		minframetime = 1000 * 1000 / 5 / vid_maxfps->value;
+
+		if (clienttimedelta < minframetime || servertimedelta < minframetime)
+			return;
 	}
 
 
@@ -651,7 +678,7 @@ Qcommon_Frame(int usec)
 	}
 }
 #else
-void
+static void
 Qcommon_Frame(int usec)
 {
 	// For the dedicated server terminal console.
@@ -703,10 +730,6 @@ Qcommon_Frame(int usec)
 	}
 
 
-	// Save global time for network- und input code.
-	curtime = Sys_Milliseconds();
-
-
 	// Target framerate.
 	pfps = (int)cl_maxfps->value;
 
@@ -748,5 +771,12 @@ Qcommon_Frame(int usec)
 void
 Qcommon_Shutdown(void)
 {
+	FS_ShutdownFilesystem();
 	Cvar_Fini();
+
+#ifndef DEDICATED_ONLY
+	Key_Shutdown();
+#endif
+
+	Cmd_Shutdown();
 }
