@@ -32,7 +32,7 @@
 #include "shared.h"
 #include "crc.h"
 
-#define YQ2VERSION "7.45"
+#define YQ2VERSION "8.01"
 #define BASEDIRNAME "baseq2"
 
 #ifndef YQ2OSTYPE
@@ -355,6 +355,7 @@ void Cbuf_InsertFromDefer(void);
 typedef void (*xcommand_t)(void);
 
 void Cmd_Init(void);
+void Cmd_Shutdown(void);
 
 void Cmd_AddCommand(char *cmd_name, xcommand_t function);
 
@@ -677,6 +678,7 @@ char **FS_ListFiles2(char *findname, int *numfiles,
 void FS_FreeList(char **list, int nfiles);
 
 void FS_InitFilesystem(void);
+void FS_ShutdownFilesystem(void);
 void FS_BuildGameSpecificSearchPath(char *dir);
 char *FS_Gamedir(void);
 char *FS_NextPath(char *prevpath);
@@ -716,11 +718,13 @@ void Com_MDPrintf(char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
 YQ2_ATTR_NORETURN void Com_Error(int code, char *fmt, ...) __attribute__ ((format (printf, 2, 3)));
 YQ2_ATTR_NORETURN void Com_Quit(void);
 
-/* Ugly hack: Apprently (our?) MinGW-gcc under Windows
-   doesn't support %zd and requires %Id. */
+/* Ugly work around for unsupported
+ * format specifiers unter mingw. */
 #ifdef WIN32
+#define YQ2_COM_PRId64 "%I64d"
 #define YQ2_COM_PRIdS "%Id"
 #else
+#define YQ2_COM_PRId64 "%ld"
 #define YQ2_COM_PRIdS "%zd"
 #endif
 
@@ -776,7 +780,6 @@ void Z_FreeTags(int tag);
 void Qcommon_Init(int argc, char **argv);
 void Qcommon_ExecConfigs(qboolean addEarlyCmds);
 const char* Qcommon_GetInitialGame(void);
-void Qcommon_Frame(int msec);
 void Qcommon_Shutdown(void);
 
 #define NUMVERTEXNORMALS 162
@@ -821,6 +824,7 @@ void *Sys_GetGameAPI(void *parms);
 void Sys_UnloadGame(void);
 void Sys_GetWorkDir(char *buffer, size_t len);
 qboolean Sys_SetWorkDir(char *path);
+qboolean Sys_Realpath(const char *in, char *out, size_t size);
 
 // Windows only (system.c)
 #ifdef _WIN32
