@@ -196,14 +196,19 @@ ThrowGib(edict_t *self, char *gibname, int damage, int type)
 		return;
 	}
 
-	gibsthisframe++;
-
-	if (gibsthisframe > MAX_GIBS)
+	if (gibsthisframe >= MAX_GIBS)
 	{
 		return;
 	}
 
-	gib = G_Spawn();
+	gib = G_SpawnOptional();
+
+	if (!gib)
+	{
+		return;
+	}
+
+	gibsthisframe++;
 
 	VectorScale(self->size, 0.5, size);
 	VectorAdd(self->absmin, size, origin);
@@ -373,14 +378,20 @@ ThrowDebris(edict_t *self, char *modelname, float speed, vec3_t origin)
 		return;
 	}
 
-	debristhisframe++;
-
-	if (debristhisframe > MAX_DEBRIS)
+	if (debristhisframe >= MAX_DEBRIS)
 	{
 		return;
 	}
 
-	chunk = G_Spawn();
+	chunk = G_SpawnOptional();
+
+	if (!chunk)
+	{
+		return;
+	}
+
+	debristhisframe++;
+
 	VectorCopy(origin, chunk->s.origin);
 	gi.setmodel(chunk, modelname);
 	v[0] = 100 * crandom();
@@ -1610,6 +1621,40 @@ commander_body_drop(edict_t *self)
 	self->s.origin[2] += 2;
 }
 
+void
+commander_body_die(edict_t *self, edict_t *inflictor /* unused */,
+		edict_t *attacker /* unused */, int damage, vec3_t point /* unused */)
+{
+	int n;
+
+	if (!self)
+	{
+		return;
+	}
+
+	/* check for gib */
+	if (self->health <= self->gib_health)
+	{
+		gi.sound(self, CHAN_BODY, gi.soundindex("tank/pain.wav"), 1, ATTN_NORM, 0);
+
+		for (n = 0; n < 1; n++)
+		{
+			ThrowGib(self, "models/objects/gibs/sm_meat/tris.md2", damage, GIB_ORGANIC);
+		}
+
+		for (n = 0; n < 4; n++)
+		{
+			ThrowGib(self, "models/objects/gibs/sm_metal/tris.md2", damage, GIB_METALLIC);
+		}
+
+		ThrowGib(self, "models/objects/gibs/chest/tris.md2", damage, GIB_ORGANIC);
+		ThrowHead(self, "models/objects/gibs/gear/tris.md2", damage, GIB_METALLIC);
+		self->deadflag = DEAD_DEAD;
+
+		return;
+	}
+}
+
 void
 SP_monster_commander_body(edict_t *self)
 {
@@ -1626,8 +1671,19 @@ SP_monster_commander_body(edict_t *self)
 	VectorSet(self->maxs, 32, 32, 48);
 	self->use = commander_body_use;
 	self->takedamage = DAMAGE_YES;
-	self->flags = FL_GODMODE;
 	self->s.renderfx |= RF_FRAMELERP;
+
+	if (g_commanderbody_nogod->value)
+	{
+		self->deadflag = DEAD_DEAD;
+		self->svflags |= SVF_MONSTER | SVF_DEADMONSTER;
+		self->die = commander_body_die;
+	}
+	else
+	{
+		self->flags = FL_GODMODE;
+	}
+
 	gi.linkentity(self);
 
 	gi.soundindex("tank/thud.wav");
