@@ -519,6 +519,9 @@ extern cvar_t *spectator_password;
 extern cvar_t *needpass;
 extern cvar_t *g_select_empty;
 extern cvar_t *dedicated;
+extern cvar_t *g_footsteps;
+extern cvar_t *g_fix_triggered;
+extern cvar_t *g_commanderbody_nogod;
 
 extern cvar_t *filterban;
 
@@ -546,6 +549,7 @@ extern cvar_t *flood_waitdelay;
 extern cvar_t *sv_maplist;
 
 extern cvar_t *aimfix;
+extern cvar_t *g_machinegun_norecoil;
 
 #define world (&g_edicts[0])
 
@@ -625,6 +629,7 @@ void G_UseTargets(edict_t *ent, edict_t *activator);
 void G_SetMovedir(vec3_t angles, vec3_t movedir);
 
 void G_InitEdict(edict_t *e);
+edict_t *G_SpawnOptional(void);
 edict_t *G_Spawn(void);
 void G_FreeEdict(edict_t *e);
 
@@ -1045,7 +1050,7 @@ struct edict_s
 	int dmg;
 	int radius_dmg;
 	float dmg_radius;
-	int sounds; /* make this a spawntemp var? */
+	int sounds; /* now also used for player death sound aggregation */
 	int count;
 
 	edict_t *chain;
