@@ -65,7 +65,7 @@ insane_moan(edict_t *self)
 	}
 
 	/* suppress screaming so pain sound can play */
-	if (self->delay > level.time)
+	if (self->fly_sound_debounce_time > level.time)
 	{
 		return;
 	}
@@ -82,7 +82,7 @@ insane_scream(edict_t *self)
 	}
 
 	/* suppress screaming so pain sound can play */
-	if (self->delay > level.time)
+	if (self->fly_sound_debounce_time > level.time)
 	{
 		return;
 	}
@@ -644,9 +644,7 @@ insane_run(edict_t *self)
 	{
 		self->monsterinfo.currentmove = &insane_move_runcrawl;
 	}
-	else
-
-	if (random() <= 0.5) /* Else, mix it up */
+	else if (frandk() <= 0.5) /* Else, mix it up */
 	{
 		self->monsterinfo.currentmove = &insane_move_run_normal;
 	}
@@ -697,9 +695,9 @@ insane_pain(edict_t *self, edict_t *other /* unused */,
 							l, r)), 1, ATTN_IDLE, 0);
 
 	/* suppress screaming and moaning for 1 second so pain sound plays */
-	self->delay = level.time + 1;
+	self->fly_sound_debounce_time = level.time + 1;
 
-	if (skill->value == 3)
+	if (skill->value == SKILL_HARDPLUS)
 	{
 		return; /* no pain anims in nightmare */
 	}
