@@ -71,7 +71,7 @@
  * load older savegames. This should be bumped if the files
  * in tables/ are changed, otherwise strange things may happen.
  */
-#define SAVEGAMEVER "YQ2-3"
+#define SAVEGAMEVER "YQ2-4"
 
 #ifndef BUILD_DATE
 #define BUILD_DATE __DATE__
@@ -142,6 +142,14 @@ typedef struct
 	mmove_t *mmovePtr;
 } mmoveList_t;
 
+typedef struct
+{
+    char ver[32];
+    char game[32];
+    char os[32];
+    char arch[32];
+} savegameHeader_t;
+
 /* ========================================================= */
 
 /*
@@ -236,6 +244,9 @@ InitGame(void)
 	coop_elevator_delay = gi.cvar("coop_elevator_delay", "1.0", CVAR_ARCHIVE);
 	skill = gi.cvar("skill", "1", CVAR_LATCH);
 	maxentities = gi.cvar("maxentities", "1024", CVAR_LATCH);
+	g_footsteps = gi.cvar("g_footsteps", "1", CVAR_ARCHIVE);
+	g_fix_triggered = gi.cvar ("g_fix_triggered", "0", 0);
+	g_commanderbody_nogod = gi.cvar("g_commanderbody_nogod", "0", CVAR_ARCHIVE);
 
 	/* change anytime vars */
 	dmflags = gi.cvar("dmflags", "0", CVAR_SERVERINFO);
@@ -262,6 +273,7 @@ InitGame(void)
 
 	/* others */
 	aimfix = gi.cvar("aimfix", "0", CVAR_ARCHIVE);
+	g_machinegun_norecoil = gi.cvar("g_machinegun_norecoil", "0", CVAR_ARCHIVE);
 
 	/* items */
 	InitItems();
@@ -772,12 +784,9 @@ ReadClient(FILE *f, gclient_t *client, short save_ver)
 void
 WriteGame(const char *filename, qboolean autosave)
 {
+	savegameHeader_t sv;
 	FILE *f;
 	int i;
-	char str_ver[32];
-	char str_game[32];
-	char str_os[32];
-	char str_arch[32];
 
 	if (!autosave)
 	{
@@ -792,20 +801,14 @@ WriteGame(const char *filename, qboolean autosave)
 	}
 
 	/* Savegame identification */
-	memset(str_ver, 0, sizeof(str_ver));
-	memset(str_game, 0, sizeof(str_game));
-	memset(str_os, 0, sizeof(str_os));
-	memset(str_arch, 0, sizeof(str_arch));
+	memset(&sv, 0, sizeof(sv));
 
-	Q_strlcpy(str_ver, SAVEGAMEVER, sizeof(str_ver) - 1);
-	Q_strlcpy(str_game, GAMEVERSION, sizeof(str_game) - 1);
-	Q_strlcpy(str_os, YQ2OSTYPE, sizeof(str_os) - 1);
-	Q_strlcpy(str_arch, YQ2ARCH, sizeof(str_arch) - 1);
+	Q_strlcpy(sv.ver, SAVEGAMEVER, sizeof(sv.ver) - 1);
+	Q_strlcpy(sv.game, GAMEVERSION, sizeof(sv.game) - 1);
+	Q_strlcpy(sv.os, YQ2OSTYPE, sizeof(sv.os) - 1);
+	Q_strlcpy(sv.arch, YQ2ARCH, sizeof(sv.arch) - 1);
 
-	fwrite(str_ver, sizeof(str_ver), 1, f);
-	fwrite(str_game, sizeof(str_game), 1, f);
-	fwrite(str_os, sizeof(str_os), 1, f);
-	fwrite(str_arch, sizeof(str_arch), 1, f);
+	fwrite(&sv, sizeof(sv), 1, f);
 
 	game.autosaved = autosave;
 	fwrite(&game, sizeof(game), 1, f);
@@ -827,12 +830,9 @@ WriteGame(const char *filename, qboolean autosave)
 void
 ReadGame(const char *filename)
 {
+	savegameHeader_t sv;
 	FILE *f;
 	int i;
-	char str_ver[32];
-	char str_game[32];
-	char str_os[32];
-	char str_arch[32];
 
 	short save_ver = 0;
 
@@ -846,90 +846,89 @@ ReadGame(const char *filename)
 	}
 
 	/* Sanity checks */
-	fread(str_ver, sizeof(str_ver), 1, f);
-	fread(str_game, sizeof(str_game), 1, f);
-	fread(str_os, sizeof(str_os), 1, f);
-	fread(str_arch, sizeof(str_arch), 1, f);
+	fread(&sv, sizeof(sv), 1, f);
 
-	if (!strcmp(str_ver, SAVEGAMEVER))
+	static const struct {
+		const char* verstr;
+		int vernum;
+	} version_mappings[] = {
+		{"YQ2-1", 1},
+		{"YQ2-2", 2},
+		{"YQ2-3", 3},
+		{"YQ2-4", 4},
+	};
+
+	for (i=0; i < sizeof(version_mappings)/sizeof(version_mappings[0]); ++i)
 	{
-		save_ver = 3;
-
-		if (strcmp(str_game, GAMEVERSION))
+		if (strcmp(version_mappings[i].verstr, sv.ver) == 0)
 		{
-			fclose(f);
-			gi.error("Savegame from another game.so.\n");
-		}
-		else if (strcmp(str_os, YQ2OSTYPE))
-		{
-			fclose(f);
-			gi.error("Savegame from another os.\n");
-		}
-		else if (strcmp(str_arch, YQ2ARCH))
-		{
-			fclose(f);
-			gi.error("Savegame from another architecture.\n");
+			save_ver = version_mappings[i].vernum;
+			break;
 		}
 	}
-	else if (!strcmp(str_ver, "YQ2-2"))
-	{
-		save_ver = 2;
 
-		if (strcmp(str_game, GAMEVERSION))
-		{
-			fclose(f);
-			gi.error("Savegame from another game.so.\n");
-		}
-		else if (strcmp(str_os, YQ2OSTYPE))
-		{
-			fclose(f);
-			gi.error("Savegame from another os.\n");
-		}
-		else if (strcmp(str_arch, YQ2ARCH))
-		{
-			fclose(f);
-			gi.error("Savegame from another architecture.\n");
-		}
-	}
-	else if (!strcmp(str_ver, "YQ2-1"))
-	{
-		save_ver = 1;
-
-		if (strcmp(str_game, GAMEVERSION))
-		{
-			fclose(f);
-			gi.error("Savegame from another game.so.\n");
-		}
-		else if (strcmp(str_os, OSTYPE_1))
-		{
-			fclose(f);
-			gi.error("Savegame from another os.\n");
-		}
-
-		if (!strcmp(str_os, "Windows"))
-		{
-			/* Windows was forced to i386 */
-			if (strcmp(str_arch, "i386"))
-			{
-				fclose(f);
-				gi.error("Savegame from another architecture.\n");
-			}
-		}
-		else
-		{
-			if (strcmp(str_arch, ARCH_1))
-			{
-				fclose(f);
-				gi.error("Savegame from another architecture.\n");
-			}
-		}
-	}
-	else
+	if (save_ver == 0) // not found in mappings table
 	{
 		fclose(f);
 		gi.error("Savegame from an incompatible version.\n");
 	}
 
+	if (save_ver == 1)
+	{
+		if (strcmp(sv.game, GAMEVERSION) != 0)
+		{
+			fclose(f);
+			gi.error("Savegame from another game.so.\n");
+		}
+		else if (strcmp(sv.os, OSTYPE_1) != 0)
+		{
+			fclose(f);
+			gi.error("Savegame from another os.\n");
+		}
+
+#ifdef _WIN32
+		/* Windows was forced to i386 */
+		if (strcmp(sv.arch, "i386") != 0)
+		{
+			fclose(f);
+			gi.error("Savegame from another architecture.\n");
+		}
+#else
+		if (strcmp(sv.arch, ARCH_1) != 0)
+		{
+			fclose(f);
+			gi.error("Savegame from another architecture.\n");
+		}
+#endif
+	}
+	else // all newer savegame versions
+	{
+		if (strcmp(sv.game, GAMEVERSION) != 0)
+		{
+			fclose(f);
+			gi.error("Savegame from another game.so.\n");
+		}
+		else if (strcmp(sv.os, YQ2OSTYPE) != 0)
+		{
+			fclose(f);
+			gi.error("Savegame from another os.\n");
+		}
+		else if (strcmp(sv.arch, YQ2ARCH) != 0)
+		{
+#if defined(_WIN32) && (defined(__i386__) || defined(_M_IX86))
+			// before savegame version "YQ2-4" (and after version 1),
+			// the official Win32 binaries accidentally had the YQ2ARCH "AMD64"
+			// instead of "i386" set due to a bug in the Makefile.
+			// This quirk allows loading those savegames anyway
+			if (save_ver >= 4 || strcmp(sv.arch, "AMD64") != 0)
+#endif
+			{
+				fclose(f);
+				gi.error("Savegame from another architecture.\n");
+			}
+		}
+	}
+
 	g_edicts = gi.TagMalloc(game.maxentities * sizeof(g_edicts[0]), TAG_GAME);
 	globals.edicts = g_edicts;
 
